# -*- coding: utf-8 -*-
require 'date'
require 'sidfm_controller_support'
require 'sidfm_host_controller_support'
require 'gen_ansible_playbook'

class SidfmHostController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private

  include SidfmControllerSupport, SidfmHostControllerSupport
  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles

  before_action :get_id

  before_action :get_view_state
  after_action :put_view_state
  
  before_action :get_vm_version
  before_action :clear_import_session
  before_action :clear_export_session, only: [ :index ]
  before_action -> { get_summary_table_lines('host') }

  # 権限チェック
  def check_roles
    unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    unless @sidfm_roles.include?(:view) then
      redirect_to user_path(User.current), :notice => af_log_user('アクセス権限がありません。') and return
    end
    if [ 'update_status', 'update_due_dates', 'send_mail' ].include?(action_name) and !@sidfm_roles.include?(:use) then
      redirect_to sidfm_vulnerabilities_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    @role_admin = @sidfm_roles.include?(:admin)
    @user_category_ids = if @role_admin then
                           nil
                         else
                           User.current.categories.pluck(:id)
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => @user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
                     end
  end

  # idを取得する。また、アクセスしているユーザがidに対応するホストを参照可能か確認する。
  def get_id
    @id = params[:id].to_i
    if !@role_admin and !@user_host_ids.include?(@id) then
      logger.error("Access Denied: user_id=#{User.current.id}, target_host_id=#{@id}, user_granted_host_ids=#{@user_host_ids.join(',')}")
      redirect_to sidfm_hosts_path, :notice => af_log_user('対象ホストに対するアクセスを許可されていません。') and return
    end
    hosts = SidfmdHost.enable_hosts.categorized.where(:id => @id)
    @host = hosts.count > 0 ? hosts.first : nil
    redirect_to sidfm_hosts_path unless @host
  end

  # 対象となるコンテンツ全てを含む配列を返す。
  # カテゴリ選択のみ、反映させる
  def create_all_content
    @vuls[:all_content] = @host.create_content(:corresp_required => @corresp_required)
  end
 
  def create_vul_view_content
    @vuls[:all_content] = SidfmdVulView.create_host_content(
      :role_admin => @role_admin,
      :user_host_ids => @user_host_ids,
      :host_id => @id
    )
    @all_html_ids = @vuls[:all_content].pluck(:html_id)
  end

  FILTER_KEYS = {}

  def init_filter_keys
    if FILTER_KEYS.size == 0 then
      default_host_filters.each do |key, value|
        FILTER_KEYS[key] = value
      end
    end
    @default_filters = {}
    FILTER_KEYS.each do |key, value|
      @default_filters[key] = FILTER_KEYS[key][:default]
    end
  end

  def check_filters
    init_filter_keys
    @vuls[:filters] = establish_filters_consistency(@vuls[:filters], FILTER_KEYS)
    @vuls[:filters] = filter_filters(
      :filters => @vuls[:filters],
      :role_admin => @role_admin,
      :user_category_ids => @user_category_ids
    )
    #ホスト別状況画面から遷移した時は、プロダクト検索の意味合いが異なるため、リセットする
    if FILTER_KEYS[:products][:default].keys.size != @vuls[:filters][:products].keys.size then
      @vuls[:filters][:products] = FILTER_KEYS[:products][:default]
    end
    #products = {}
    #FILTER_KEYS[:products][:default].each do |key, val| #ホスト別状況画面とパラメータが異なるため、修正する
    #  products[key] = @vuls[:filters][:products][key] ? @vuls[:filters][:products][key] : val
    #end
    #@vuls[:filters][:products] = products
  end

  DAY_SECONDS = 24 * 60 * 60

  # フィルタの適用
  # 未使用。カスタムタグ検索未実装
  def apply_filters
    if @vuls[:filters][:cvss_vector].size > 0 && !@vuls[:filters][:cvss_vector][:is_target_unregistered] then 
      #CVSS指標フィルタ用のデータ
      filter_cvss_v2 = ''
      filter_cvss_v3 = ''
      cvss_v2_vectors = SidfmdIssueCve.get_cvss_v2_vectors.keys.map{|vector| vector.to_sym}
      cvss_v3_vectors = SidfmdIssueCve.get_cvss_v3_vectors.keys.map{|vector| vector.to_sym}
      selected_cvss_v2 = @vuls[:filters][:cvss_vector][:cvss_v2]
      selected_cvss_v3 = @vuls[:filters][:cvss_vector][:cvss_v3]
      is_selected_cvss_v2 = false
      is_selected_cvss_v3 = false
      cvss_v2_vectors.each_with_index do |vector, idx|
        is_selected_cvss_v2 = true if selected_cvss_v2[vector]
        filter_cvss_v2 += '/' unless idx == 0
        filter_cvss_v2 += "#{vector}:#{(selected_cvss_v2[vector] || '.')}"
      end
      cvss_v3_vectors.each_with_index do |vector, idx|
        is_selected_cvss_v3 = true if selected_cvss_v3[vector]
        filter_cvss_v3 += '/' unless idx == 0
        filter_cvss_v3 += "#{vector}:#{(selected_cvss_v3[vector] || '.')}"
      end
      filter_cvss_v2 = Regexp.new("\\A" + filter_cvss_v2 + "\\z")
      filter_cvss_v3 = Regexp.new("\\A" + filter_cvss_v3 + "\\z")
    end

    #履歴は対象のhtml_idを先に取得する
    is_filter_history = false
    filter_history_html_ids = []
    if @vuls[:filters][:journal][:period] != '::' then
      if /^(in|rev):(\d{4}\/\d{2}\/\d{2}|):(\d{4}\/\d{2}\/\d{2}|)$/ =~ @vuls[:filters][:journal][:period] then
        orig_flg_rev = Regexp.last_match(1)
        orig_begin_date = Regexp.last_match(2)
        orig_end_date = Regexp.last_match(3)
        if orig_flg_rev.size > 0 and orig_begin_date.size == 0 and orig_end_date.size == 0 then
          @vuls[:filters][:journal][:period] = '::'
        else
          flg_rev = orig_flg_rev == 'rev'
          begin_date = orig_begin_date.size > 0 ? Date.strptime(orig_begin_date, '%Y/%m/%d') : nil
          end_date = orig_end_date.size > 0 ? Date.strptime(orig_end_date, '%Y/%m/%d') : nil
          begin_date = begin_date.to_time if begin_date
          end_date = end_date.to_time + DAY_SECONDS - 1 if end_date
          is_filter_history = true
   
          filter_history_html_ids = Journal.joins(:issue)
                                           .joins('INNER JOIN sidfmd_issues ON issues.id = sidfmd_issues.issue_id')
                                           .joins('LEFT OUTER JOIN journal_details ON journal_details.journal_id = journals.id')
                                           .where('sidfmd_issues.sidfmd_host_id = ?', @id)
          if flg_rev then
            if begin_date and end_date then
              filter_history_html_ids = filter_history_html_ids.where.not(:journals => { :created_on => begin_date..end_date })
            elsif begin_date then
              filter_history_html_ids = filter_history_html_ids.where.not('journals.created_on >= ?', begin_date)
            elsif end_date then
              filter_history_html_ids = filter_history_html_ids.where.not('journals.created_on <= ?', end_date)
            else
              # last resort
              @vuls[:filters][:journal][:period] = '::'
              is_filter_history = false
            end
          else
            if begin_date and end_date then
              filter_history_html_ids = filter_history_html_ids.where(:journals => { :created_on => begin_date..end_date })
            elsif begin_date then
              filter_history_html_ids = filter_history_html_ids.where('journals.created_on >= ?', begin_date)
            elsif end_date then
              filter_history_html_ids = filter_history_html_ids.where('journals.created_on <= ?', end_date)
            else
              # last resort
              @vuls[:filters][:journal][:period] = '::'
              is_filter_history = false
            end
          end
          if @vuls[:filters][:journal][:filter_status_change] and @vuls[:filters][:journal][:filter_duedate_change] then
            f_html_ids = filter_history_html_ids.where(:journal_details => {:prop_key => 'status_id'}).pluck('sidfmd_issues.html_id') &
                         filter_history_html_ids.where(:journal_details => {:prop_key => 'due_date'}).pluck('sidfmd_issues.html_id')
            if f_html_ids.any? then
              filter_history_html_ids = filter_history_html_ids.where(%|
                sidfmd_issues.html_id IN (SELECT unnest(ARRAY[#{f_html_ids.join(', ')}]))
              |)
            else
              filter_history_html_ids = filter_history_html_ids.where(:journals => { :id => 0 })
            end
          elsif @vuls[:filters][:journal][:filter_status_change] then
            filter_history_html_ids = filter_history_html_ids.where(:journal_details => { :prop_key => 'status_id' }) 
          elsif @vuls[:filters][:journal][:filter_duedate_change] then
            filter_history_html_ids = filter_history_html_ids.where(:journal_details => { :prop_key => 'due_date' })
          end
          filter_history_html_ids = filter_history_html_ids.pluck('sidfmd_issues.html_id')
        end
      else
        @vuls[:filters][:date] = '::'
      end
    end

    ret = @vuls[:all_content].find_all {|item|
      # 要注意
      if @vuls[:filters][:warning] == 'warning' then
        next false unless item[:warning][@vuls[:idx]]
      elsif @vuls[:filters][:warning] == 'others' then
        next false if item[:warning][@vuls[:idx]]
      end
      # 対応状況
      if /\A(\d):(\d)\z/ =~ @vuls[:filters][:status] then
          status1 = Regexp.last_match(1).to_i
          status2 = Regexp.last_match(2).to_i
          next false unless (item[:status_id] == status1 or item[:status_id] == status2)
      elsif @vuls[:filters][:status] != '0' then
        next false unless item[:status_id] == @vuls[:filters][:status].to_i
      end
      # SRI
      if @vuls[:filters][:sris].any? then
        next false unless @vuls[:filters][:sris].include?(item[:sri].to_s)
      end
      # CVSS
      if @vuls[:filters][:cvsses].any? then
        next false unless @vuls[:filters][:cvsses].include?(item[:cvss][:label].to_s)
      end
      #CVSS指標
      if @vuls[:filters][:cvss_vector].size > 0 then
        is_hit = false
        vul_cves = get_filtered_cves(item[:cves], false)
        if @vuls[:filters][:cvss_vector][:is_target_unregistered] then
          vul_cves.each do |cve|
            break if is_hit
            is_hit = true if cve[:c2_vector].empty? && cve[:c3_vector].empty?
          end
        else
          vul_cves.each do |cve|
            if is_selected_cvss_v2 and is_selected_cvss_v3 then
              is_hit = true if filter_cvss_v2 =~ cve[:c2_vector] and filter_cvss_v3 =~ cve[:c3_vector] 
            elsif is_selected_cvss_v3 then
              is_hit = true if filter_cvss_v3 =~ cve[:c3_vector]
            else
              is_hit = true if filter_cvss_v2 =~ cve[:c2_vector]
            end
            break if is_hit
          end
        end
        next false unless is_hit
      end
      # SofTek影響範囲
      if @vuls[:filters][:area] != '' then
        next false unless (item[:html_id] > 0 and  @vuls[:filters][:area].to_i == item[:softek][:area_id])
      end
      # 攻撃コード
      if @vuls[:filters][:exploitable] != '' then
        next false unless item[:exploitable]
      end
      # 履歴
      if is_filter_history then
        next false unless filter_history_html_ids.include?(item[:html_id])
      end
      #if @vuls[:filters][:journal] != '' then
      #  next false unless journal_html_ids.include?(item[:html_id])
      #end
      # 対策予定日設定済
      if @vuls[:filters][:due_date] != '::' then
        if /\A(|notset):(|\d{4}\/\d{2}\/\d{2}):(|\d{4}\/\d{2}\/\d{2})\z/ =~ @vuls[:filters][:due_date] then
          notset = ('notset' == Regexp.last_match(1))
          begindate = Regexp.last_match(2)
          enddate = Regexp.last_match(3)
          if notset then
            next false unless item[:due_date].nil?
          else
            next false if item[:due_date].nil?
            begin
              begindate = Date.parse(begindate)
            rescue Exception => e
              begindate = nil
            end
            begin
              enddate = Date.parse(enddate)
            rescue Exception => e
              enddate = nil
            end
            if begindate and enddate then
              next false unless (item[:due_date] >= begindate and item[:due_date] <= enddate)
            elsif begindate then
              next false unless item[:due_date] >= begindate
            elsif enddate then
              next false unless item[:due_date] <= enddate
            end
          end
        end
      end
      # キーワード検索
      if @vuls[:filters][:keyword] != '' then
        if /\A\s*(\d+)\s*\z/ =~ @vuls[:filters][:keyword] then
          html_id = Regexp.last_match(1).to_i
          regexp = Regexp.new('.*' + Regexp.escape(@vuls[:filters][:keyword]) + '.*', Regexp::IGNORECASE)
          next false unless (regexp =~ item[:title] or item[:cves].select{|cve| (regexp =~ cve[:cve_id]) != nil}.any? or html_id == item[:html_id])
        elsif /\A\s*KB(\d+)\s*\z/i =~ @vuls[:filters][:keyword] then
          kbid = -Regexp.last_match(1).to_i
          next false unless (Regexp.new('.*' + Regexp.escape(@vuls[:filters][:keyword]) + '.*', Regexp::IGNORECASE) =~ item[:title] or kbid == item[:html_id])
        else
          regexp = Regexp.new('.*' + Regexp.escape(@vuls[:filters][:keyword]) + '.*', Regexp::IGNORECASE)
          next false unless regexp =~ item[:title] or item[:cves].select{|cve| (regexp =~ cve[:cve_id]) != nil}.any?
        end
      end
      # 日付指定検索
      if @vuls[:filters][:date] != '::' then
        if /\A(in|rev):(\d{4}\/\d{2}\/\d{2}|):(\d{4}\/\d{2}\/\d{2}|)\z/ =~ @vuls[:filters][:date] then
          orig_flg_rev = Regexp.last_match(1)
          orig_begin_date = Regexp.last_match(2)
          orig_end_date = Regexp.last_match(3)
          if orig_flg_rev.size > 0 and orig_begin_date.size == 0 and orig_end_date.size == 0 then
            @vuls[:filters][:date] = '::'
            next false
          end
          flg_rev = orig_flg_rev == 'rev' ? true : false
          begin_date = orig_begin_date.size > 0 ? Date.strptime(orig_begin_date, '%Y/%m/%d') : nil
          end_date = orig_end_date.size > 0 ? Date.strptime(orig_end_date, '%Y/%m/%d') : nil
          if flg_rev then
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              next false if begin_date <= item[:created_on][:value] and end_date >= item[:created_on][:value]
            elsif begin_date then
              begin_date = begin_date.to_time
              next false if begin_date <= item[:created_on][:value]
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              next false if end_date >= item[:created_on][:value]
            else
              # last resort
              @vuls[:filters][:date] = '::'
              next false
            end
          else
            if begin_date and end_date then
              begin_date = begin_date.to_time
              end_date = end_date.to_time + DAY_SECONDS - 1
              next false unless begin_date <= item[:created_on][:value] and end_date >= item[:created_on][:value]
            elsif begin_date then
              begin_date = begin_date.to_time
              next false unless begin_date <= item[:created_on][:value]
            elsif end_date then
              end_date = end_date.to_time + DAY_SECONDS - 1
              next false unless end_date >= item[:created_on][:value]
            else
              # last resort
              @vuls[:filters][:date] = '::'
              next false
            end
          end
        else
          @vuls[:filters][:date] = '::'
          next false
        end
      end
      true
    }
    #ret.sort! {|x, y|
    #  SidfmdIssue::SORT_PRIORITIES[x[:status_id]] <=> SidfmdIssue::SORT_PRIORITIES[y[:status_id]] ||
    #  y[:created_on][:value] <=> x[:created_on][:value]
    #}
    case @vuls[:order]
    when 'warning_asc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:display], x[:warning][@vuls[:idx]] ? 500 : 1000 ]
      }
      ret.sort! {|x, y|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = xx[2] <=> yy[2]
        if r == 0 then
          r = xx[1] <=> yy[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'warning_desc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:display], x[:warning][@vuls[:idx]] ? 500 : 1000 ]
      }
      ret.sort! {|y, x|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = xx[2] <=> yy[2]
        if r == 0 then
          r = xx[1] <=> yy[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'status_asc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:display], x[:status_id] ]
      }
      ret.sort! {|x, y|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = xx[2] <=> yy[2]
        if r == 0 then
          r = xx[1] <=> yy[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'status_desc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:display], x[:status_id] ]
      } 
      ret.sort! {|y, x|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = xx[2] <=> yy[2]
        if r == 0 then
          r = xx[1] <=> yy[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'sri_asc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:display], x[:sri] ]
      }
      ret.sort! {|x, y|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = yy[2] <=> xx[2]
        if r == 0 then
          r = xx[1] <=> yy[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'sri_desc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:display], x[:sri] ]
      }
      ret.sort! {|y, x|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = yy[2] <=> xx[2]
        if r == 0 then
          r = xx[1] <=> yy[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'cvss_asc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:display], x[:cvss][:score] ? x[:cvss][:score] : -10.0  ]
      }
      ret.sort! {|x, y|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = yy[2] <=> xx[2]
        if r == 0 then
          r = xx[1] <=> yy[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'cvss_desc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:display], x[:cvss][:score] ? x[:cvss][:score] : -10.0  ]
      }
      ret.sort! {|y, x|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = yy[2] <=> xx[2]
        if r == 0 then
          r = xx[1] <=> yy[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'area_asc' then
      temp_ev = proc {|x|
        [
         x[:title].downcase,
         x[:created_on][:display],
         x[:html_id] > 0 ? x[:softek][:area_id] : -10,
        ]
      }
      ret.sort! {|x, y|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = yy[2] <=> xx[2]
        if r == 0 then
          r = xx[1] <=> yy[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'area_desc' then
      temp_ev = proc {|x|
        [
         x[:title].downcase,
         x[:created_on][:display],
         x[:html_id] > 0 ? x[:softek][:area_id] : -10,
        ]
      }
      ret.sort! {|y, x|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = yy[2] <=> xx[2]
        if r == 0 then
          r = xx[1] <=> yy[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'exploitable_asc' then
      temp_ev = proc {|x|
        [
         x[:title].downcase, 
         x[:created_on][:display],
         x[:html_id] > 0 ? (x[:cvss][:label] != :notreg ? (x[:exploitable] ? 500 : 1000) : 1500) : 1250
        ]
      }
      ret.sort! {|x, y|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = xx[2] <=> yy[2]
        if r == 0 then
          r = yy[1] <=> xx[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'exploitable_desc' then
      temp_ev = proc {|x|
        [ x[:title].downcase,
          x[:created_on][:display],
          x[:html_id] > 0 ? (x[:cvss][:label] != :notreg ? (x[:exploitable] ? 500 : 1000) : 1500) : 1250 ]
      }
      ret.sort! {|y, x|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = xx[2] <=> yy[2]
        if r == 0 then
          r = yy[1] <=> xx[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'elapseddays_asc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:elapsed_days] ]
      }
      ret.sort! {|x, y|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = yy[1] <=> xx[1]
        if r == 0 then
          r = xx[0] <=> yy[0]
        end
        r
      }
    when 'elapseddays_desc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:elapsed_days] ]
      }
      ret.sort! {|y, x|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = yy[1] <=> xx[1]
        if r == 0 then
          r = xx[0] <=> yy[0]
        end
        r
      }
    when 'title_asc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:html_id] ]
      }
      ret.sort! {|x, y|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = xx[1] <=> yy[1]
        if r == 0 then
          r = xx[0] <=> yy[0]
        end
        r
      }
    when 'title_desc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:html_id] ]
      }
      ret.sort! {|y, x|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = xx[1] <=> yy[1]
        if r == 0 then
          r = xx[0] <=> yy[0]
        end
        r
      }
    when 'journals_asc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:display], x[:journals].size ]
      }
      ret.sort! {|x, y|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = yy[2] <=> xx[2]
        if r == 0 then
          r = yy[1] <=> xx[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'journals_desc' then
      temp_ev = proc {|x|
        [ x[:title].downcase, x[:created_on][:display], x[:journals].size ]
      }
      ret.sort! {|y, x|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = yy[2] <=> xx[2]
        if r == 0 then
          r = yy[1] <=> xx[1]
          if r == 0 then
            r = xx[0] <=> yy[0]
          end
        end
        r
      }
    when 'duedate_asc' then
      temp_ev = proc {|x|
        [ x[:title].downcase,
          x[:created_on][:display],
          x[:due_date] ? x[:due_date] : Date.new(1970,1,1),
          x[:due_date] ? 500 : 1000 ]
      }
      ret.sort! {|x, y|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = xx[3] <=> yy[3]
        if r == 0 then
          r = xx[2] <=> yy[2]
          if r == 0 then
            r = xx[1] <=> yy[1]
            if r == 0 then
              r = xx[0] <=> yy[0]
            end
          end
        end
        r
      }
    when 'duedate_desc' then
      temp_ev = proc {|x|
        [ x[:title].downcase,
          x[:created_on][:display],
          x[:due_date] ? x[:due_date] : Date.new(1970,1,1),
          x[:due_date] ? 500 : 1000 ]
      }
      ret.sort! {|y, x|
        xx = temp_ev.call(x)
        yy = temp_ev.call(y)
        r = xx[3] <=> yy[3]
        if r == 0 then
          r = xx[2] <=> yy[2]
          if r == 0 then
            r = xx[1] <=> yy[1]
            if r == 0 then
              r = xx[0] <=> yy[0]
            end
          end
        end
        r
      }
    end
    ret
  end

  #vul_viewでのフィルタ適用
  def apply_filters_to_vul_view_content

    @vuls[:all_content] = filter_host_content(
      :content => @vuls[:all_content],
      :filters => @vuls[:filters],
      :role_admin => @role_admin,
      :user_category_ids => @user_category_ids,
      :user_host_ids => @user_host_ids,
      :host_id => @id,
      :index => @vuls[:idx],
      :corresp_required => @corresp_required
    )
   
    #ソート処理
 
    @vuls[:all_content] = sort_host_content(
      :content => @vuls[:all_content],
      :order => @vuls[:order],
      :host_id => @id,
      :index => @vuls[:idx],
      :corresp_required => @corresp_required
    )

  end

=begin
  TIMELINE_SPAN = 6
  TIMELINE_SPAN_UNIT = :month

  # タイムラインのデータを返す
  def get_timeline_data
    ret = {}
    ret[:now] = Time.current
    ret[:localtime] = ret[:now].localtime
    ret[:timescale] = []
    TIMELINE_SPAN.times do |i|
      z = ret[:localtime].ago((TIMELINE_SPAN - 1 - i).send(TIMELINE_SPAN_UNIT))
      r = {
        :time => z,
        :year => z.year,
        :month => z.month,
        :step => TIMELINE_SPAN - 1 - i,
        :is_first => (i == 0),
        :is_last => (i == (TIMELINE_SPAN - 1)),
        :name => z.strftime('%Y/%m'),
      }
      ret[:timescale] << r
    end
    ret[:status_counts] = {
      SidfmdIssue::STATUS_NOTCONFIRMED => [0] * ret[:timescale].size,
      SidfmdIssue::STATUS_ONGOING      => [0] * ret[:timescale].size,
      SidfmdIssue::STATUS_FIXED        => [0] * ret[:timescale].size,
      SidfmdIssue::STATUS_NOTAFFECT    => [0] * ret[:timescale].size,
    }
    ret[:status_names] = {}
    SidfmdIssue::STATUS_ALL.each do |s|
      ret[:status_names][s] = SidfmdIssue.status_id_to_s(s)
    end
    case @vuls[:idx]
    when :sri then
      ret[:risk_syms] = {
        0 => 'none', 1 => 'low', 2 => 'medium', 3 => 'high', 4 => 'critical', 5 => 'fixed',
      }
      ret[:risk_names] = {
        0 => '無', 1 => '低', 2 => '中', 3 => '高', 4 => '重大', 5 => '済',
      }
      ret[:risk_counts] = ret[:risk_syms].each_key.collect {|x|
        [0] * ret[:timescale].size
      }
    when :cvss then
      ret[:risk_labels] = [ :notreg, :d4, :d3, :d2, :d1, :fixed ]
      ret[:risk_names] = {
        :notreg => '未登録',
        :d4 => '10.0〜9.0',
        :d3 => '8.9〜7.0',
        :d2 => '6.9〜4.0',
        :d1 => '3.9〜0.0',
        :fixed => '済',
      }
      ret[:risk_counts] = {}
      ret[:risk_labels].each do |i|
        ret[:risk_counts][i] = [0] * ret[:timescale].size
      end
    end

    @vuls[:all_content].each do |item|
      ret[:timescale].each do |step|
        next if item[:created_on][:value] > step[:time]
        # issue
        issue_status_id = item[:status_id]
        # journals
        temp_j = item[:journals].collect {|x|
          if x[:created_on] <= step[:time] and x[:details] and
              x[:details].find {|x| x[:prop_key] == 'status_id' } then
            x
          else
            nil
          end
        }
        temp_j.compact!
        temp_j.sort! {|x, y|
          x[:created_on] == y[:created_on] ?
          x[:id] <=> y[:id] :
          x[:created_on] <=> y[:created_on] }
        journal_status_id = nil
        if temp_j.size > 0 then
          temp_j.last[:details].each do |i|
            if i[:prop_key] == 'status_id' then
              journal_status_id = i[:value].to_i
              break
            end
          end
        end
        count_index = TIMELINE_SPAN - 1 - step[:step]
        if journal_status_id then
          ret[:status_counts][journal_status_id][count_index] += 1
          if SidfmdIssue::STATUS_UNFINISHED.include?(journal_status_id) then
            case @vuls[:idx]
            when :sri then
              ret[:risk_counts][item[:sri]][count_index] += 1
            when :cvss then
              ret[:risk_counts][item[:cvss][:label]][count_index] += 1
            end
          else
            case @vuls[:idx]
            when :sri then
              ret[:risk_counts][5][count_index] += 1
            when :cvss then
              ret[:risk_counts][:fixed][count_index] += 1
            end
          end
        elsif issue_status_id then
          ret[:status_counts][issue_status_id][count_index] += 1
          if SidfmdIssue::STATUS_UNFINISHED.include?(issue_status_id) then
            case @vuls[:idx]
            when :sri then
              ret[:risk_counts][item[:sri]][count_index] += 1
            when :cvss then
              ret[:risk_counts][item[:cvss][:label]][count_index] += 1
            end
          else
            case @vuls[:idx]
            when :sri then
              ret[:risk_counts][5][count_index] += 1
            when :cvss then
              ret[:risk_counts][:fixed][count_index] += 1
            end
          end
        end
      end
    end

    ret
  end
=end

  # パラメータ解析
  def parse_params
    pages_params = params.fetch(:pages, {}).permit(:order, :offset, :per_page, :grp, :idx, :filters, :checked, :all_checked, :is_update_one_row, :timeline_risk_period, :timeline_status_period)

    @vuls = { :url => request.url }

    @vuls[:grp] = (pages_params[:grp] || 'count').to_sym
    @vuls[:idx] = (pages_params[:idx] || @view_state[:idx].to_s).to_sym
    @view_state[:idx] = @vuls[:idx]
    @vuls[:offset] = (pages_params[:offset] || 0).to_i
    @vuls[:per_page] = (@summary_table_lines || pages_params[:per_page] || 10).to_i
    @vuls[:order] = pages_params[:order] || 'title_asc'
    @vuls[:checked] = JSON.parse(pages_params[:checked] || '[]')
    @vuls[:all_checked] = pages_params[:all_checked] || false
    @vuls[:filters] = JSON.parse((pages_params[:filters] || '{}'), symbolize_names: true)
    check_filters

    @corresp_required = Sidfm.corresp_required
    
    create_vul_view_content
    @target_scsids = SidfmdVvScsid.where(:sidfmd_host_id => @id).where(:sidfmd_vul_view_id => @vuls[:all_content].select(:id))
    warning_scsids = if @vuls[:idx] == :sri then
                       @target_scsids.where('sri >= ?', @corresp_required[:sri])
                     else
                       @target_scsids.where('cvss >= ? OR cvss IS NULL', @corresp_required[:cvss])
                     end
    @vuls[:corresp_required_vul_size] = warning_scsids.count
    @vuls[:fixed_corresp_required_vul_size] = warning_scsids.where(:status_id => SidfmdIssue::STATUS_FINISHED).count
    @vuls[:not_fixed_corresp_required_vul_size] = @vuls[:corresp_required_vul_size] - @vuls[:fixed_corresp_required_vul_size]
    @vuls[:all_vul_size] = @vuls[:all_content].size
    @is_update_one_row = (pages_params[:is_update_one_row] ? (pages_params[:is_update_one_row] == 'true' ? true : false) : false)
    @vuls[:timeline_risk_period] = (pages_params[:timeline_risk_period] || "month").to_sym
    @vuls[:timeline_status_period] = (pages_params[:timeline_status_period] || "month").to_sym
  end

  def filter_content
    @status = {
      :exploit => [ { :value => @vuls[:not_fixed_corresp_required_vul_size] } ]
    }
    @vuls[:distribution_status] = {
      :notconfirmed => 0, :ongoing => 0, :fixed => 0, :notaffect => 0,
    }
    @vuls[:distribution_risk] =
      case @vuls[:idx]
      when :sri then
        { :critical => 0, :high => 0, :medium => 0, :low => 0, :none => 0, :fixed => 0, }
      when :cvss then
        { :notreg => 0, :d4 => 0, :d3 => 0, :d2 => 0, :d1 => 0, :fixed => 0, }
      end
    @vuls[:all_content].each do |i|
      # status
      case i[:status_id]
      when SidfmdIssue::STATUS_NOTCONFIRMED then
        @vuls[:distribution_status][:notconfirmed] += 1
      when SidfmdIssue::STATUS_ONGOING then
        @vuls[:distribution_status][:ongoing] += 1
      when SidfmdIssue::STATUS_FIXED then
        @vuls[:distribution_status][:fixed] += 1
      when SidfmdIssue::STATUS_NOTAFFECT then
        @vuls[:distribution_status][:notaffect] += 1
      end
      # risk
      if SidfmdIssue::STATUS_UNFINISHED.include?(i[:status_id]) then
        case @vuls[:idx]
        when :sri then
          case i[:sri]
          when SidfmdIssueSri::VALUES[0] then
            @vuls[:distribution_risk][:critical] += 1
          when SidfmdIssueSri::VALUES[1] then
            @vuls[:distribution_risk][:high] += 1
          when SidfmdIssueSri::VALUES[2] then
            @vuls[:distribution_risk][:medium] += 1
          when SidfmdIssueSri::VALUES[3] then
            @vuls[:distribution_risk][:low] += 1
          when SidfmdIssueSri::VALUES[4] then
            @vuls[:distribution_risk][:none] += 1
          end
        when :cvss then
          case i[:cvss][:label]
          when :notreg then
            @vuls[:distribution_risk][:notreg] += 1
          when :d4 then
            @vuls[:distribution_risk][:d4] += 1
          when :d3 then
            @vuls[:distribution_risk][:d3] += 1
          when :d2 then
            @vuls[:distribution_risk][:d2] += 1
          when :d1 then
            @vuls[:distribution_risk][:d1] += 1
          end
        end
      else
        @vuls[:distribution_risk][:fixed] += 1
      end
    end

    @vuls[:content_filtered] = apply_filters
    @vuls[:total_size] = @vuls[:content_filtered].size

    @vuls[:statuses] = SidfmdIssue.statuses

    @due_dates = @vuls[:all_content].collect {|x| x[:due_date]}.compact
    @system_categories = SidfmdCategory.system
    @not_system_categories = SidfmdCategory.not_system
    if !@role_admin then
      @not_system_categories = @not_system_categories.where(:id => @user_category_ids)
    end
    @statuses = IssueStatus.all.order(:id)
    #@timeline_data = (@vuls[:grp] == :distribution) ? get_timeline_data : []
  end

  def filter_vul_view_content
    @status = {
      :exploit => [ { :value => @vuls[:not_fixed_corresp_required_vul_size] } ]
    }
    @vuls[:distribution_status] = {
      :notconfirmed => 0, :ongoing => 0, :fixed => 0, :notaffect => 0,
    }
    @vuls[:distribution_risk] =
      case @vuls[:idx]
      when :sri then
        { :critical => 0, :high => 0, :medium => 0, :low => 0, :none => 0, :fixed => 0, }
      when :cvss then
        { :notreg => 0, :d4 => 0, :d3 => 0, :d2 => 0, :d1 => 0, :fixed => 0, }
      end
    # status
    @vuls[:distribution_status][:notconfirmed] = @target_scsids.where(:status_id => SidfmdIssue::STATUS_NOTCONFIRMED ).count
    @vuls[:distribution_status][:ongoing] = @target_scsids.where(:status_id => SidfmdIssue::STATUS_ONGOING ).count
    @vuls[:distribution_status][:fixed] = @target_scsids.where(:status_id => SidfmdIssue::STATUS_FIXED ).count
    @vuls[:distribution_status][:notaffect] = @target_scsids.where(:status_id => SidfmdIssue::STATUS_NOTAFFECT ).count
    # risk
    status_unfinished_vuls = @target_scsids.where(:status_id => SidfmdIssue::STATUS_UNFINISHED)
    if @vuls[:idx] == :sri then
      @vuls[:distribution_risk][:critical] = status_unfinished_vuls.where(:sri => SidfmdIssueSri::VALUES[0]).count
      @vuls[:distribution_risk][:high] = status_unfinished_vuls.where(:sri => SidfmdIssueSri::VALUES[1]).count
      @vuls[:distribution_risk][:medium] = status_unfinished_vuls.where(:sri => SidfmdIssueSri::VALUES[2]).count
      @vuls[:distribution_risk][:low] = status_unfinished_vuls.where(:sri => SidfmdIssueSri::VALUES[3]).count
      @vuls[:distribution_risk][:none] = status_unfinished_vuls.where(:sri => SidfmdIssueSri::VALUES[4]).count
    else
      @vuls[:distribution_risk][:notreg] = status_unfinished_vuls.where(:cvss => nil).count
      @vuls[:distribution_risk][:d4] = status_unfinished_vuls.where('cvss >= ?', SidfmdIssueCve::LABEL[:d4][:bottom]).count
      @vuls[:distribution_risk][:d3] = status_unfinished_vuls.where('cvss < ?', SidfmdIssueCve::LABEL[:d3][:top])
                                                     .where('cvss >= ?', SidfmdIssueCve::LABEL[:d3][:bottom]).count
      @vuls[:distribution_risk][:d2] = status_unfinished_vuls.where('cvss < ?', SidfmdIssueCve::LABEL[:d2][:top])
                                                     .where('cvss >= ?', SidfmdIssueCve::LABEL[:d2][:bottom]).count
      @vuls[:distribution_risk][:d1] = status_unfinished_vuls.where('cvss < ?', SidfmdIssueCve::LABEL[:d1][:top]).count
    end
    @vuls[:distribution_risk][:fixed] = @target_scsids.where.not(:status_id => SidfmdIssue::STATUS_UNFINISHED).count
  
    @vuls[:content_filtered] = apply_filters_to_vul_view_content
    @vuls[:total_size] = @vuls[:content_filtered].count
    
    @vuls[:statuses] = SidfmdIssue.statuses

    @due_dates = @target_scsids.where.not(:due_date => nil).pluck(:due_date)
    @system_categories = SidfmdCategory.system
    @not_system_categories = SidfmdCategory.not_system
    if !@role_admin then
      @not_system_categories = @not_system_categories.where(:id => @user_category_ids)
    end
    @statuses = IssueStatus.all.order(:id)

  end

  def create_host_info
    ret = {}
    queues = @host.sidfmd_queues.order(:created_at => :desc)
    queue = queues.first
    if queue then
      ret[:queue] = queue
      ret[:os] = @host.get_os
      ret[:pkgs] = queue.sidfmd_queue_pkgs
      ret.delete(:pkgs) if ret[:pkgs].count == 0
      win_apps = nil
      queue_win = queue.sidfmd_queue_win
      if queue_win then
        win_apps = queue_win.sidfmd_queue_win_apps
      end
      ret[:apps] = (win_apps && win_apps.exists?) ? win_apps : queue.sidfmd_queue_apps
      ret.delete(:apps) if ret[:apps].count == 0
    end
    apps = @host.sidfmd_host_apps
    if apps then
      ret[:lapps] = apps
      ret.delete(:lapps) if ret[:lapps].count == 0
    end
    #ローカルプロダクト
    lapps = @host.sidfmd_host_lapps
    if lapps then
      ret[:lproducts] = lapps
      ret.delete(:lproducts) if ret[:lproducts].count == 0 
    end
    ret
  end

  #脆弱性のcve(s)として採用するcve(s)を取得する
  #CVEIDが同じものがある場合は、c3_update_datetimeがあればc3_update_datetime, 
  #c2_update_datetimeがあればc2_update_datetimeで最も新しいもの1つを採用する。
  #両方無い場合は配列の順番で後のものを採用する。
  def get_filtered_cves(vul_cves, is_for_summary = true)
    filtered_cves = []
    if vul_cves.any? then
      target_cve = nil
      create_cve_data = ->(cve){
        cvss_v2 = if !cve[:c2_vector].empty? then
                    cve[:c2_vector].split('/').map{|c| c.split(':')}.to_h
                  else
                    nil
                  end
        cvss_v3 = if !cve[:c3_vector].empty? then
                    cve[:c3_vector].split('/').map{|c| c.split(':')}.to_h
                  else
                    nil
                  end
        return {
          :cve_id => cve[:cve_id],
          :cvss_v2 => cvss_v2,
          :cvss_v3 => cvss_v3,
        }
      }
      none_update_time_vuls = {}
      vul_cves.each do |cve|
        if cve[:c3_update_datetime].empty? && cve[:c2_update_datetime].empty? then
          none_update_time_vuls[cve[:cve_id]] = [] unless none_update_time_vuls[cve[:cve_id]]
          none_update_time_vuls[cve[:cve_id]] << cve
        end 
      end
     
      vul_cves.sort_by{|cve| cve[:cve_id]}.each_with_index do |cve, idx|
        if idx == 0 then
          target_cve = cve
        else
          if target_cve[:cve_id] == cve[:cve_id] then
            if !cve[:c3_update_datetime].empty?  then
              if target_cve[:c3_update_datetime].empty? then
                target_cve = cve
              else
                target_cve = cve if target_cve[:c3_update_datetime] < cve[:c3_update_datetime]
              end
            elsif target_cve[:c3_update_datetime].empty? && !cve[:c2_update_datetime].empty? then
              if target_cve[:c2_update_datetime].empty? then 
                target_cve = cve
              else
                target_cve = cve if target_cve[:c2_update_datetime] < cve[:c2_update_datetime]
              end
            elsif target_cve[:c3_update_datetime].empty? && target_cve[:c2_update_datetime].empty?
              target_cve = none_update_time_vuls[[:cve_id]].last
            end
          else
            if is_for_summary then
              filtered_cves << create_cve_data.(target_cve)
            else
              filtered_cves << target_cve
            end
            target_cve = cve
          end
        end
        if idx == vul_cves.size - 1 then
          if is_for_summary then
            filtered_cves << create_cve_data.(target_cve)
          else
            filtered_cves << target_cve
          end
        end
      end
    end
    return filtered_cves
  end

  def get_vul_ctags_data(vul_view_ids)
    ctags = SidfmdCtag.joins(:sidfmd_vv_ctags)
      .where(:sidfmd_vv_ctags => {
        :sidfmd_vul_view_id => vul_view_ids,
        :sidfmd_host_id => @id
      })
      .order(%| sidfmd_ctags.priority DESC |)
      .order(%| LOWER(sidfmd_ctags.keyword COLLATE "C") |)
    unless @role_admin then
      ctags = ctags.where("sidfmd_ctags.sidfmd_category_id IS NULL OR sidfmd_ctags.sidfmd_category_id IN (?)",  @user_category_ids)
    end
    ctags
  end

  def create_vul_ctags(vul_view_ids)
    @vul_ctags = {}
    ctags = get_vul_ctags_data(vul_view_ids)
    ctags.select(
      "sidfmd_ctags.keyword AS tag_name",
      "sidfmd_ctags.color AS text_color",
      "sidfmd_ctags.bgcolor AS bg_color",
      "sidfmd_vv_ctags.sidfmd_vul_view_id AS view_id"
    )
    .each do |ctag|
      @vul_ctags[ctag.view_id] = [] unless @vul_ctags[ctag.view_id]
      @vul_ctags[ctag.view_id] << {
        :keyword => ctag.tag_name,
        :text_color => ctag.text_color,
        :bg_color => ctag.bg_color
      }
    end
  end

  #JSON/TSVファイル出力のカスタムタグ
  def create_vul_ctags_for_file(vul_view_ids)
    @vul_ctags_for_file = {}
    ctags = get_vul_ctags_data(vul_view_ids)
    ctags.select(
      "sidfmd_ctags.keyword",
      "sidfmd_ctags.sidfmd_category_id",
      "sidfmd_ctags.color",
      "sidfmd_ctags.bgcolor",
      "sidfmd_ctags.priority",
      "sidfmd_vv_ctags.sidfmd_vul_view_id AS view_id"
    ).each do |ctag|
      @vul_ctags_for_file[ctag.view_id] = [] unless @vul_ctags_for_file[ctag.view_id]
      @vul_ctags_for_file[ctag.view_id] << {
        :keyword => ctag.keyword,
        :category_id => ctag.sidfmd_category_id,
        :color => ctag.color,
        :background_color => ctag.bgcolor,
        :priority => ctag.priority
      }
    end
  end

  def create_htmlid_create_update_time(html_ids)
    @htmlid_dates = SidfmdHtmlidDate.create_htmlid_create_update_time(html_ids)
  end

  def create_summary_vul_html(vul_view, **opts)
    title = SidfmdVulView.display_title(vul_view.title)
    sidfmd_issue = SidfmdIssue.where(:html_id => vul_view.html_id).where(:sidfmd_host_id => @id).first
    is_warning =
      if SidfmdIssue::STATUS_UNFINISHED.include?(opts[:scsid].status_id) then
        if opts[:idx] == :sri then
          (opts[:scsid].sri >= opts[:corresp_required][:sri])
        else
          (opts[:scsid].cvss.nil? or opts[:scsid].cvss >= opts[:corresp_required][:cvss])
        end
      else
        false
      end
    cvss_label = SidfmdIssueCve.labelling(opts[:scsid].cvss).to_s
    data = {
      :chkbox => view_context.create_summary_vul_chkbox(vul_view.html_id, opts[:checked]),
      :warning => view_context.create_summary_taiou(is_warning),
      :status => view_context.create_summary_status(
        [ opts[:scsid].status_id ],
        opts[:scsid].status_id,
        false
      ),
      :sri => view_context.create_summary_vul_sri(opts[:scsid].sri, vul_view.html_id),
      :cvss => view_context.create_summary_vul_cvss(
        vul_view.html_id,
        vul_view.html_id,
        opts[:scsid].cvss,
        cvss_label,
        vul_view.sidfmd_vv_cves.any?,
        "host"
      ),
      :area => view_context.create_summary_vul_area(vul_view.softek_area_id, vul_view.html_id),
      :exploitable => view_context.create_summary_vul_exploit(
        (SidfmdIssue::STATUS_UNFINISHED.include?(opts[:scsid].status_id) ? vul_view.exploitable : nil),
        cvss_label,
        vul_view.html_id
      ),
      :elapseddays => view_context.create_summary_vul_detail_elapsed_days(opts[:scsid].created_on, opts[:scsid].status_id),
      :title => view_context.create_summary_vul_title(
        :title => title,
        :short_description => vul_view.short_description,
        :html_id => vul_view.html_id,
        :create_datetime => @htmlid_dates[vul_view.html_id] ? @htmlid_dates[vul_view.html_id][:create_datetime] : nil,
        :update_datetime => @htmlid_dates[vul_view.html_id] ? @htmlid_dates[vul_view.html_id][:update_datetime] : nil,
        :link_opts => { :class => "link-to-detail", 'data-htmlid' => vul_view.html_id }
      ),
      :detail => view_context.create_summary_host_detail_affected_items(vul_view, sidfmd_issue),
      :journals => view_context.create_summary_host_detail_history(
        vul_view,
        Journal.where(:journalized_id => sidfmd_issue ? sidfmd_issue.issue_id : nil).count,
        opts[:user_id]
      ),
      :duedate => view_context.create_summary_duedate(opts[:scsid].due_date, opts[:scsid].status_id),
      :ctags => view_context.create_summary_custom_tag(
        (@vul_ctags[vul_view.id] ? @vul_ctags[vul_view.id].first : nil),
        vul_view.html_id,
        title,
        @sidfm_roles,
        { :sidfmd_host_id => @id }
      ),
      :action => view_context.create_summary_detail_action(
        vul_view,
        opts[:scsid].status_id,
        "#modalEditStatus",
        "#modalEditSchedule",
        "vul"
      ) 
    }
  end

  public

  def index
    f_params = params.fetch(:form, {}).permit(:history)
    parse_params
    filter_vul_view_content
    if session['hostid_with_htmlid'] then
      if session['hostid_with_htmlid']['vul'] then
        @vuls[:content_filtered].each do |vul|
          h_id = vul[:html_id].to_s
          if session['hostid_with_htmlid']['vul'][h_id] then
            session['hostid_with_htmlid']['vul'][h_id][1] = true
          end
        end
      end
      if session['hostid_with_htmlid']['host'] && !@is_update_one_row then
        session['hostid_with_htmlid']['host'].each_key do |hostid|
          session['hostid_with_htmlid']['host'][hostid][1] = false
        end
      end
    end
    @host_disabled_apps = SidfmdHostDisabledApp.where(:sidfmd_host_id => @id).pluck(
      :name,
      :version,
      :path
    ).map{|app|
      { :name => app[0], :version => app[1], :path => app[2] }
    }
    @hostinfo = create_host_info
    no_system_categories = @host.no_system_categories(@role_admin ? nil : User.current) 
    @self_user_categories = 
      SidfmdCategory.where(
        :id => SidfmdCategoriesUser.where(:user_id => User.current).select(:sidfmd_category_id)
      ).map{|c| c.is_system ? Hash["system_c", c.name] : Hash["custom_c", c.name]}
    @hostinfo_for_keywords = {
      osname: @hostinfo[:os] ? @hostinfo[:os][:osname] : '---',
      synchro: (@host.last_checked_at && Time.current - @host.last_checked_at <= 2.0 * 24.0 * 60.0 * 60.0) ? '同期中' : '非同期',
      last_checked_at: @host.last_checked_at ? @host.last_checked_at.strftime('%Y/%m/%d %H:%M:%S') :  @hostinfo[:queue] ? @hostinfo[:queue].created_at.strftime('%Y/%m/%d %H:%M:%S') : '未同期',
      agent: @host.is_agent ? '使用' : '不使用',
      s_category: @host.has_system_category? ? @host.system_category.name : '---',
      c_category: no_system_categories.any? ? no_system_categories.map{|c| c.sidfmd_category.name} : ['カスタムカテゴリ無し'],
      pkgs_quantity: @hostinfo[:pkgs] ? @hostinfo[:pkgs].length : 0,
      apps: (@hostinfo[:apps] ? @hostinfo[:apps].map{|app|
        (app.class == SidfmdQueueWinApp ? "#{app.display_name}" : "#{app.name} #{app.version}\n#{app.path}")
      } : ['なし']),
      lapps: @hostinfo[:lapps] ? @hostinfo[:lapps].map{|lapp| lapp.get_name } : ['なし'],
      lproducts: @hostinfo[:lproducts] ? @hostinfo[:lproducts].map{|p| p.get_name} : ['なし'],
      memo: @host.memo,
      hostname: @host.hostname,
      ext_name: @host.ext_name,
      display_name: @host.display_name,
    }
    @history_filter_vul = nil
    if f_params[:history] then
      history = JSON.parse(f_params[:history])
      html_id = history[0].to_i
      host_id = history[1].to_s
      @history_filter_vul = @vuls[:content_filtered].where(:html_id => html_id).select(
        :html_id,
      ).first
      #count = 0
      #@vuls[:content_filtered].each do |vul|
      #  if vul[:html_id] == html_id then
      #    break
      #  else 
      #    count += 1;
      #  end
      #end
      #@vuls[:offset] = count
      #@history = html_id
      session[:confirming_journal] = {:host_id => host_id, :html_id => html_id}
    end 
    @vuls[:content] = @vuls[:content_filtered].offset(@vuls[:offset]).limit(@vuls[:per_page])#@vuls[:content_filtered].slice(@vuls[:offset], @vuls[:per_page])
    @vuls[:content] = [] unless @vuls[:content]

    html_ids = []
    #journalの数をキャッシュせずに反映させる
    @vuls_journals_num = {}
    @vuls[:content].each do |vul|
      html_ids << vul[:html_id]
      sidfmd_issue = SidfmdIssue.where(:sidfmd_host_id => @host.id)
                                .where(:html_id => vul[:html_id])
      @vuls_journals_num[vul[:html_id]] = sidfmd_issue.first ? Journal.where(:journalized_id => sidfmd_issue.first.issue_id).count : 0
      #cves
      #vul[:cves] = get_filtered_cves vul[:cves]
    end
    pvt_filters = SidfmdPvtFilter.get_filters(SidfmdGrpFilter::KEYWORD_HOST, User.current.id)
    grp_filters = SidfmdGrpFilter.get_filters(SidfmdGrpFilter::KEYWORD_HOST, (@role_admin ? nil : @user_category_ids))
    @stored_filters = pvt_filters + grp_filters

    #@vuls[:content].each{|a| a.delete(:journals)}
   
    #ローカル脆弱性の影響するローカルプロダクト
=begin
    @issue_lproduct_names = {}
    lproducts = SidfmdLocalVulLapp.joins(:sidfmd_local_vul).joins(
      'LEFT OUTER JOIN sidfmd_issues ON sidfmd_local_vuls.html_id = sidfmd_issues.html_id'
    )
    .joins('LEFT OUTER JOIN sidfmd_host_lapps ON sidfmd_issues.sidfmd_host_id = sidfmd_host_lapps.sidfmd_host_id')
    .where('sidfmd_issues.html_id IN (?)', html_ids)
    .where("sidfmd_host_lapps.sidfmd_host_id = #{@host.id}")
    .where('sidfmd_host_lapps.sidfmd_lproduct_id IS NOT NULL')
    .select(
      'sidfmd_issues.id AS issue_id',
      'sidfmd_issues.html_id AS html_id',
      'sidfmd_host_lapps.sidfmd_lproduct_id AS host_lapp_lpid',
      'sidfmd_host_lapps.sidfmd_lversion_id AS host_lapp_lvid'
    )
    if lproducts.exists? then
      local_vul_lpids_of_all = {}
      local_vul_lvids = {}
      local_vul_lpids = {}
      lpids = []
      SidfmdLocalVul.joins(:sidfmd_local_vul_lapps)
                    .where('sidfmd_local_vuls.html_id IN (?)', html_ids)
                    .select(
                      'sidfmd_local_vul_lapps.sidfmd_lproduct_id AS lpid', 
                      'sidfmd_local_vul_lapps.sidfmd_lversion_id AS lvid',
                      'sidfmd_local_vuls.html_id AS html_id'
                    )
                    .each do |lapp|
                      if lapp.lvid == 0 then #バージョンが「全て」の場合
                        local_vul_lpids_of_all[lapp.html_id] = [] unless local_vul_lpids_of_all[lapp.html_id]
                        local_vul_lpids_of_all[lapp.html_id] << lapp.lpid
                      else #バージョンが「全て」でない場合
                        local_vul_lvids[lapp.html_id] = [] unless local_vul_lvids[lapp.html_id]
                        local_vul_lvids[lapp.html_id] << lapp.lvid
                      end
                      local_vul_lpids[lapp.html_id] = [] unless local_vul_lpids[lapp.html_id]
                      local_vul_lpids[lapp.html_id] << lapp.lpid
                      lpids << lapp.lpid
                    end 
      lprod_names = {}
      lver_names = {}
      SidfmdLproduct.where(:id => lpids).each{|lp| lprod_names[lp.id] = lp.name}
      SidfmdLversion.where(:sidfmd_lproduct_id => lpids).each{|lv| lver_names[lv.id] = lv.fullname}

      lproducts.each do |lapp|
        @issue_lproduct_names[lapp.issue_id] = [] unless @issue_lproduct_names[lapp.issue_id]
        if !lapp.host_lapp_lvid || lapp.host_lapp_lvid == 0 then #バージョンが「全て」の場合
          lprod_ids = local_vul_lpids[lapp.html_id]
          if lprod_ids && lprod_ids.include?(lapp.host_lapp_lpid) then
            @issue_lproduct_names[lapp.issue_id] << lprod_names[lapp.host_lapp_lpid] 
          end
        else #バージョンが「全て」でない場合
          lver_ids = local_vul_lvids[lapp.html_id]
          if lver_ids && lver_ids.include?(lapp.host_lapp_lvid) then
            @issue_lproduct_names[lapp.issue_id] << lver_names[lapp.host_lapp_lvid]   
          else
            lprod_ids = local_vul_lpids_of_all[lapp.html_id]
            if lprod_ids && lprod_ids.include?(lapp.host_lapp_lpid) then
              @issue_lproduct_names[lapp.issue_id] << lver_names[lapp.host_lapp_lvid]
            end
          end
        end
      end
    end
=end
   
    @cvss_v2_vectors = SidfmdIssueCve.get_cvss_v2_vectors
    @cvss_v3_vectors = SidfmdIssueCve.get_cvss_v3_vectors
  
    @deleted_comment = HISTORY_COMMENT_DELETED_NOTE  
    #ブラウザバックかどうか判断するための処理
    #if !session['chkbok_ids'] then
    #  session['chkbok_ids'] = {}
    #end
    #if !session['chkbok_ids']['host'] then
    #  session['chkbok_ids']['host'] = {}
    #end
    #session['chkbok_ids']['host']['is_browser_back'] = false
  end

  def update_status
=begin    f_params = params.fetch(:status, {}).permit(:htmlids, :prev_status, :new_status, :comment, :dump, :action)
    dump = {}
    begin
      dump = JSON.parse(f_params[:dump])
    rescue JSON::ParserError => e
      log_exception(e)
      dump[:action] = :index
      dump[:notice] = '入力データが不正です。'
      redirect_to dump and return
    end
    dump[:action] = :index
    if !f_params[:htmlids] or f_params[:htmlids].size <= 1 then
      dump[:notice] = '入力データが不正です。'
      redirect_to dump and return
    end
    htmlids = nil
    begin
      htmlids = JSON.parse(f_params[:htmlids])
    rescue JSON::ParserError => e
      log_exception(e)
      dump[:notice] = '入力データが不正です。'
      redirect_to dump and return
    end
    if !htmlids.respond_to?(:'[]') or !htmlids.respond_to?(:size) or htmlids.size == 0 then
      dump[:notice] = '入力データが不正です。'
      redirect_to dump and return
    end
    if !@role_admin then
      check = SidfmdIssue
        .enable_hosts
        .categorized
        .where(:sidfmd_host_id => @user_host_ids)
        .where(:html_id => htmlids)
        .count
      if check == 0 then
        dump[:notice] = 'アクセスを許可されておりません。'
        redirect_to dump and return
      end
    end
    prev_status = f_params[:prev_status].to_i
    if prev_status == 0 and f_params[:prev_status] != '0' then
      dump[:notice] = '設定前ステータスが不正です。'
      redirect_to dump and return
    end
    if !(SidfmdIssue::STATUS_ALL + [0]).include?(prev_status) then
      dump[:notice] = '設定前ステータスが不正です。'
      redirect_to dump and return
    end
    new_status = f_params[:new_status].to_i
    if new_status == 0 and f_params[:new_status] != '0' then
      dump[:notice] = '設定後ステータスが不正です。'
      redirect_to dump and return
    end
    if !(SidfmdIssue::STATUS_ALL + [0]).include?(new_status) then
      dump[:notice] = '設定後ステータスが不正です。'
      redirect_to dump and return
    end
    comment = f_params[:comment]
    comment = 'ステータス変更' if (comment.nil? or comment.size == 0)
    begin
      SidfmdIssue.transaction do
        issues = SidfmdIssue
          .joins(:issue)
          .where(:sidfmd_host_id => @id)
          .where(:html_id => htmlids)
        if prev_status > 0 then
          issues = issues.where(:issues => { :status_id => prev_status })
        end
        SidfmdIssue.change_statuses_with_journal(issues.map{|i|i.id}, nil, new_status, comment, User.current.id)
      end
      Rails.cache.clear
    rescue ActiveRecord::StatementInvalid => e
      log_exception(e)
      dump[:notice] = 'ステータスの変更に失敗しました。'
      redirect_to dump and return
    end

    action = f_params[:action]
    if action == 'single' then
      if !session['hostid_with_htmlid'] then
        session['hostid_with_htmlid'] = {}
      end
      if !session['hostid_with_htmlid']['host'] then
        session['hostid_with_htmlid']['host'] = {}
      end
      session['hostid_with_htmlid']['host'][@id.to_s] = [htmlids[0].to_s, true]
      dump[:'form[is_update_one_row]'] = true
    end
    
    redirect_to dump
=end
    f_params = params.permit(:issue_ids, :shift_num, :ids, :prev_status, :new_status, :comment, :action_name)
    err = false
    err_info = {}
    htmlids = nil
    shift_num = f_params[:shift_num]
    begin
      issue_ids = JSON.parse(f_params[:issue_ids])
      htmlids = JSON.parse(f_params[:ids])
    rescue JSON::ParserError => e
      err = true
    end
    if !htmlids.respond_to?(:'[]') or !htmlids.respond_to?(:size) or htmlids.size == 0 then
      err = true
    end
    prev_status = f_params[:prev_status].to_i
    if prev_status == 0 and f_params[:prev_status] != '0' then
      err = true
    end
    if !(SidfmdIssue::STATUS_ALL + [0]).include?(prev_status) then
      err = true
    end
    new_status = f_params[:new_status].to_i
    if new_status == 0 and f_params[:new_status] != '0' then
      err = true
    end
    if !(SidfmdIssue::STATUS_ALL + [0]).include?(new_status) then
      err = true
    end
    comment = f_params[:comment]
    comment = 'ステータス変更' if (comment.nil? or comment.size == 0)
    if err then
      render :layout => false, :json => { :err => 'invalid' } and return
    end
    return_value = {}
    p_issue_ids = []

    #target_html_ids = []
    update_data = []
    begin
      SidfmdIssue.transaction do
        old_status_id = nil
        if prev_status > 0 then
          old_status_id = prev_status
        end
        if issue_ids.size == 0 then
          issues = SidfmdIssue
            .joins(:issue)
            .where(:sidfmd_host_id => @id)
            .where(:html_id => htmlids)
          if prev_status > 0 then
            issues = issues.where(:issues => { :status_id => prev_status })
          end
          issue_ids = issues.map{|i| i.issue.id}
          return_value[:total_size] = issue_ids.size
          shift_num = 100
          return_value[:shift_num] = shift_num
        end

        p_issue_ids = issue_ids.shift(shift_num.to_i)
        SidfmdIssue.where(:issue_id => p_issue_ids).where(:sidfmd_host_id => @id).each do |i|
          update_data << {
            :sidfmd_issue_id => i.id,
            :html_id => i.html_id,
            :sidfmd_host_id => i.sidfmd_host_id,
            :status_id => new_status
          }
        end
        target_html_ids = SidfmdIssue.where(:issue_id => p_issue_ids).distinct.pluck(:html_id)
        SidfmdIssue.change_statuses_with_journal(p_issue_ids, old_status_id, new_status, comment, User.current.id)
      end
      Rails.cache.clear
    rescue ActiveRecord::StatementInvalid => e
      logger.info(e)
      err = true
    end
    return_value[:issue_ids] = issue_ids
    return_value[:executed_vuls_size] = p_issue_ids.size
    if err then
      return_value[:err] = 'db_error'
      return_value[:err_vuls_size] = p_issue_ids.size
    else
      #View更新
      SidfmdVulView.update_from_rule_action(update_data)
      SidfmdHostView.update_from_rule_action(update_data)
      #SidfmdVulView.sync_data(target_html_ids)
      #SidfmdHostView.sync_data([@id])
    end
  
    action = f_params[:action_name]
    if action == 'single' then
      if !session['hostid_with_htmlid'] then
        session['hostid_with_htmlid'] = {}
      end
      if !session['hostid_with_htmlid']['host'] then
        session['hostid_with_htmlid']['host'] = {}
      end
      session['hostid_with_htmlid']['host'][@id.to_s] = [htmlids[0].to_s, true]
    end
    render :layout => false, :json => return_value
  end

  def update_due_dates
=begin    f_params = params.fetch(:schedule, {}).permit(:htmlids, :force, :due_date, :comment, :dump, :action)
    dump = {}
    begin
      dump = JSON.parse(f_params[:dump])
    rescue JSON::ParserError => e
      log_exception(e)
      dump[:action] = :index
      dump[:notice] = '入力データが不正です。'
      redirect_to dump and return
    end
    dump[:action] = :index
    htmlids = JSON.parse(f_params[:htmlids])
    if htmlids.size == 0 or !htmlids.respond_to?(:'[]') then
      dump[:notice] = '入力データが不正です。'
      redirect_to dump and return
    end
    if !@role_admin then
      check = SidfmdIssue
        .enable_hosts
        .categorized
        .where(:sidfmd_host_id => @user_host_ids)
        .where(:html_id => htmlids)
        .count
      if check == 0 then
        dump[:notice] = 'アクセスを許可されておりません。'
        redirect_to dump and return
      end
    end
    force = !f_params[:force].nil?
    due_date = f_params[:due_date]
    unless due_date then
      dump[:notice] = '不正な日付です。'
      redirect_to dump and return
    end
    due_date = nil if (due_date and due_date.size == 0)
    if due_date then
      temp = nil
      begin
        temp = Date.strptime(due_date, '%Y/%m/%d')
      rescue Exception => e
        log_exception(e)
      end
      unless temp then
        begin
          temp = Date.strptime(due_date, '%m/%d/%Y')
        rescue Exception => e
          log_exception(e)
        end
      end
      if temp then
        due_date = temp
      else
        dump[:notice] = '不正な日付です。'
        redirect_to dump and return
      end
    end
    comment = f_params[:comment]
    comment = '対策予定日設定変更' if (comment.nil? or comment.size == 0)
    begin
      SidfmdIssue.transaction do
        issues = SidfmdIssue
          .joins(:issue)
          .where(:sidfmd_host_id => @id)
          .where(:html_id => htmlids)
          .where.not(:issues => { :status_id => SidfmdIssue::STATUS_FINISHED })
        if !force then
          issues = issues
            .where(:issues => { :due_date => nil })
        end
        #issues.each do |issue|
        #  issue.with_lock do
        #    issue.change_due_date_with_journal(User.current.id, due_date, comment)
        #  end
        #end
        issue_ids = issues.map{|i| i.issue.id}
        SidfmdIssue.change_due_dates_with_journal(issue_ids, force, due_date, comment, User.current.id)
      end
      Rails.cache.clear
    rescue ActiveRecord::StatementInvalid => e
      log_exception(e)
      dump[:notice] = '不正な日付です。'
      redirect_to dump and return
    end
    action = f_params[:action]
    if action == 'single' then
      if !session['hostid_with_htmlid'] then
        session['hostid_with_htmlid'] = {}
      end
      if !session['hostid_with_htmlid']['host'] then
        session['hostid_with_htmlid']['host'] = {}
      end
      session['hostid_with_htmlid']['host'][@id.to_s] = [htmlids[0].to_s, true]
      dump[:'form[is_update_one_row]'] = true
    end

    redirect_to dump
=end
    f_params = params.permit(:issue_ids, :shift_num, :ids, :force, :due_date, :comment, :action_name)
    err = false
    err_info = {}
    shift_num = f_params[:shift_num]
    begin 
      issue_ids = JSON.parse(f_params[:issue_ids])
      htmlids = JSON.parse(f_params[:ids])
    rescue JSON::ParserError => e
      logger.info(e)
      err = true
    end
    if htmlids.size == 0 or !htmlids.respond_to?(:'[]') then
      err = true
    end
    force = f_params[:force] == 'true' ? true : false
    due_date = f_params[:due_date]
    unless due_date then
      err = true
    end
    due_date = nil if (due_date and due_date.size == 0)
    if due_date then
      temp = nil
      begin
        temp = Date.strptime(due_date, '%Y/%m/%d')
      rescue Exception => e
        logger.info(e)
        err = true
      end
      unless temp then
        begin
          temp = Date.strptime(due_date, '%m/%d/%Y')
        rescue Exception => e
          logger.info(e)
          err = true
        end
      end
      if temp then
        due_date = temp
      else
        err = true
      end
    end
    comment = f_params[:comment]
    comment = '対策予定日設定変更' if (comment.nil? or comment.size == 0)
    if err then
      render :layout => false, :json => { :err => 'invalid' } and return
    end
    return_value = {}
    p_issue_ids = []

    #target_html_ids = []
    update_data = []
    begin
      SidfmdIssue.transaction do
        if issue_ids.size == 0 then
          issues = SidfmdIssue
            .joins(:issue)
            .where(:sidfmd_host_id => @id)
            .where(:html_id => htmlids)
            .where.not(:issues => { :status_id => SidfmdIssue::STATUS_FINISHED })
          if !force then
            issues = issues.where(:issues => { :due_date => nil })
          end
          issue_ids = issues.map{|i| i.issue.id}
          return_value[:total_size] = issue_ids.size
          shift_num = 100
          return_value[:shift_num] = shift_num
        end

        p_issue_ids = issue_ids.shift(shift_num.to_i)
        SidfmdIssue.where(:issue_id => p_issue_ids).where(:sidfmd_host_id => @id).each do |i|
          update_data << {
            :sidfmd_issue_id => i.id,
            :html_id => i.html_id,
            :sidfmd_host_id => i.sidfmd_host_id,
            :due_date => due_date
          }
        end
        #target_html_ids = SidfmdIssue.where(:issue_id => p_issue_ids).distinct.pluck(:html_id)
        SidfmdIssue.change_due_dates_with_journal(p_issue_ids, force, due_date, comment, User.current.id)
      end
      Rails.cache.clear
    rescue ActiveRecord::StatementInvalid => e
      logger.info(e)
      err = true
    end
    return_value[:issue_ids] = issue_ids
    return_value[:executed_vuls_size] = p_issue_ids.size
    if err then
      return_value[:err] = 'db_error'
      return_value[:err_vuls_size] = p_issue_ids.size
    else
      #View更新
      SidfmdVulView.update_from_rule_action(update_data)
      SidfmdHostView.update_from_rule_action(update_data)
      #SidfmdVulView.sync_data(target_html_ids)
      #SidfmdHostView.sync_data([@id])
    end
    action = f_params[:action_name]
    if action == 'single' then
      if !session['hostid_with_htmlid'] then
        session['hostid_with_htmlid'] = {}
      end
      if !session['hostid_with_htmlid']['host'] then
        session['hostid_with_htmlid']['host'] = {}
      end
      session['hostid_with_htmlid']['host'][@id.to_s] = [htmlids[0].to_s, true]
    end
    render :layout => false, :json => return_value
  end

  def update_ctags
    if !@sidfm_roles.include?(:use) then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    f_params = params.permit(:html_id, :ctag_ids)
    if !f_params[:html_id] or !f_params[:ctag_ids] then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    begin
      ctag_ids = JSON.parse(f_params[:ctag_ids])
    rescue JSON::ParserError => e
      logger.info(e)
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    html_id = f_params[:html_id].to_i
    begin
      issues = SidfmdIssue
        .enable_hosts
        .categorized
        .where(:sidfmd_host_id => @id)
        .where(:html_id => html_id)
      SidfmdIssueCtag.udpate_issue_ctags(
        ctag_ids: ctag_ids,
        sidfmd_issue_rows: issues,
        role_admin: @role_admin,
        user_category_ids: @user_category_ids,
        user_host_ids: @user_host_ids
      )
    rescue Exception => e
      logger.info(e)
      render :layout => false, :json => { :err_message => 'カスタムタグの更新に失敗しました。' } and return
    end

    render :layout => false, :json => { :err_message => '' }
  end

  CSV_BOM = "\xff\xfe".force_encoding('UTF-16LE').freeze

  MAX_CELL_LENGTH = 30000
  MAX_LINE_NUM = 254 # =最大改行数は253

  def read_download_data
    GC.start
    f_params = params.permit(:target, :executed_size, :process, :idx, :order, :filters, :ids)
    if !f_params[:ids] or !f_params[:target] or !f_params[:order] or !f_params[:filters] or
       !f_params[:executed_size] or !f_params[:process] or !f_params[:idx]
    then
      logger.info('params error')
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    format =  f_params[:target]
    if ![ 'json', 'tsv', 'tsv-package-info', 'playbook', 'tsv-affect-products' ].include?(format) then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    executed_size = f_params[:executed_size].to_i

    if format == 'tsv-package-info' then
      begin
        @vuls = {}
        @vuls[:pkgs] = JSON.parse(f_params[:ids] || '[]') #パッケージの時だけ、2回目以降はパッケージを受け取ることに注意
      rescue JSON::ParserError => e
        logger.error(e)
      end
    else
      @vuls = { :url => request.url }
      @vuls[:idx] = (f_params[:idx] || 'sri').to_sym
      @vuls[:order] = f_params[:order] || 'title_asc'
      @vuls[:filters] = JSON.parse((f_params[:filters] || '{}'), symbolize_names: true)
      @vuls[:ids] = []
      begin
        @vuls[:ids] = JSON.parse(f_params[:ids] || '[]') #processがinitの時(最初の時)はhtml_idが入り、2回目以降はSidfmdVulVeiwのidが入ることに注意
      rescue JSON::ParserError => e
        logger.error(e)
      end
      @corresp_required = Sidfm.corresp_required

      check_filters
      create_vul_view_content
      @vuls[:content_filtered] = apply_filters_to_vul_view_content

      @hostinfo = create_host_info
    end

    if format == 'json' then
      if f_params[:process] == 'init' then
        @vuls[:content_filtered] = @vuls[:content_filtered].where(:html_id => @vuls[:ids]) if @vuls[:ids].any?
        temp_order = @vuls[:order].to_s.split(/_/)
        temp_s_cat = @host.system_category
        init_json_summary_table_progress_download(
          :ids => @vuls[:content_filtered].pluck(:id),
          :order_col_name => temp_order[0],
          :order_type => temp_order[1],
          :filters => @vuls[:filters],
          :max_download_size => @vuls[:content_filtered].count,
          :idx => @vuls[:idx],
          :host_detail_info => {
            :id => @host.id,
            :hostname => @host.hostname,
            :os => (@hostinfo[:os] and @hostinfo[:os][:osname]) ? @hostinfo[:os][:osname] : '---',
            :last_checked_at =>  @host.last_checked_at ? @host.last_checked_at.strftime('%Y/%m/%d %H:%M:%S') : ( @hostinfo[:queue] ? @hostinfo[:queue].created_at.strftime('%Y/%m/%d %H:%M:%S') : '未同期' ),
            :is_agent => @host.is_agent,
            :system_category => (temp_s_cat ? { :id => temp_s_cat.id, :name => temp_s_cat.name } : { :id => 0, :name => '---' }),
            :no_system_categories => SidfmdCategory.where(
              :id => (@role_admin ? @host.no_system_categories : @host.no_system_categories(User.current)).select(:sidfmd_category_id)
            ).order('LOWER(name COLLATE"C")').collect {|x| { :id => x.id, :name => x.name } },
            :packages => (@hostinfo[:pkgs] ? @hostinfo[:pkgs].collect {|x| x.pkg } : []),
            :products => (@hostinfo[:apps] ? @hostinfo[:apps].collect {|x|
              (x.class == SidfmdQueueWinApp ? { :publisher => x.publisher, :name => x.name, :version => x.version } : { :name => x.name, :version => x.version, :path => x.path })
            } : []),
            :manual_products => (@hostinfo[:lapps] ? @hostinfo[:lapps].collect {|x| x.get_name }.compact : []), 
            :local_products => (@hostinfo[:lproducts] ? @hostinfo[:lproducts].collect {|x| x.get_name }.compact : []),
            :memo => @host.memo
          }
        )
      else
        create_vul_ctags_for_file(@vuls[:ids])
        create_htmlid_create_update_time(@vuls[:content_filtered].where(:id => @vuls[:ids]).pluck(:html_id))
        create_json_data = Proc.new{|id|
          vul = @vuls[:content_filtered].find_by(:id => id)
          next unless vul
          issue = SidfmdIssue.where(:sidfmd_host_id => @id, :html_id => vul.html_id).first
          host_vul_scsid = vul.sidfmd_vv_scsids.find_by(:sidfmd_host_id => @id)
          next if !issue or !host_vul_scsid
          item = {}
          ctags = (@vul_ctags_for_file[id.to_i] || [])
          item[:html_id] = vul.html_id > 0 ? "#{vul.html_id}" : "KB#{-vul.html_id}"
          item[:issue_id] = issue.issue_id
          item[:title] = SidfmdVulView.display_title(vul.title)
          item[:short_description] = (vul.short_description ? vul.short_description : '')
          item[:warning] = if SidfmdIssue::STATUS_UNFINISHED.include?(host_vul_scsid.status_id) then
                             if @vuls[:idx] == :sri then
                               if host_vul_scsid.sri >= @corresp_required[:sri] then
                                 true
                               else
                                 false
                               end
                             else
                               if host_vul_scsid.cvss.nil? || host_vul_scsid.cvss >= @corresp_required[:cvss] then
                                 true
                               else
                                 false
                               end
                             end
                           else
                             false
                           end
          item[:status] = SidfmdIssue.status_id_to_s(host_vul_scsid.status_id)
          item[:sri] = SidfmdIssueSri::RISKS[host_vul_scsid.sri]
          cvss_label = SidfmdIssueCve.labelling(host_vul_scsid.cvss).to_s
          item[:cvss] = (cvss_label == 'notreg' ? '' : sprintf('%.1f', (host_vul_scsid.cvss ? host_vul_scsid.cvss : 0).to_f))
          cves = vul.get_summary_cves
          cvss_2_cves = []
          cvss_3_cves = []
          cve_ids = []
          cves.each do |cve|
            cve_ids << cve.cve_id
            cvss_2_cves << { :id => cve.cve_id, :vector => cve.c2_vector_index }
            cvss_3_cves << { :id => cve.cve_id, :vector => cve.c3_vector_index }
          end
          item[:cve_ids] = cve_ids
          item[:cvss_v2] = cvss_2_cves
          item[:cvss_v3] = cvss_3_cves
          item[:area] = SidfmdArea.where(:area_id => vul.softek_area_id).first.name
          item[:exploitable] = (cvss_label != 'notreg' and SidfmdIssue::STATUS_UNFINISHED.include?(host_vul_scsid.status_id)) ? vul.exploitable : nil
          elapsed_created_on = host_vul_scsid.created_on ? host_vul_scsid.created_on : Time.current
          item[:elapsed_days] = (SidfmdIssue::STATUS_UNFINISHED.include?(host_vul_scsid.status_id) ? (((Time.current - elapsed_created_on) / 24.0 / 60.0 / 60.0).to_i) : '')
          item[:created_on] = elapsed_created_on.strftime('%Y/%m/%d')
          item[:due_date] = (host_vul_scsid.due_date ? host_vul_scsid.due_date.strftime('%Y/%m/%d') : '')
          item[:journals] = []  
          Journal.where(:journalized_id => issue.issue_id).order(:created_on => :desc).each do |journal|
            j_item = {}
            j_item[:id] = journal[:id]
            j_item[:created_on] = journal[:created_on] ? journal[:created_on].strftime('%Y/%m/%d %H:%M:%S') : '---'
            user = User.find(journal[:user_id])
            j_item[:user] = { :id => journal[:user_id], :name => user.anonymous? ? '削除済みユーザ' : view_context.display_user_name(user) }
            j_item[:notes] = Journal.find(journal[:id]).notes
            j_item[:details] = []
            JournalDetail.where(:journal_id => journal[:id]).order(:id).each do |detail|
              jd_item = {}
              jd_item[:id] = detail[:id]
              case detail[:prop_key]
              when 'status_id' then
                jd_item[:action] = 'ステータス変更'
                jd_item[:from] = SidfmdIssue.status_id_to_s(detail[:old_value].to_i)
                jd_item[:to] = SidfmdIssue.status_id_to_s(detail[:value].to_i)
              when 'due_date' then
                jd_item[:action] = '対策予定日の変更'
                jd_item[:from] = SidfmdIssue.due_date_to_s(detail[:old_value])
                jd_item[:to] = SidfmdIssue.due_date_to_s(detail[:value])
              else
                next
              end
              j_item[:details] << jd_item
            end
            item[:journals] << j_item
          end
          item[:affect_packages] = issue.affect_packages('')
          item[:affect_products] = issue.affect_products + issue.affect_lproducts
          item[:ctags] = ctags
          item[:create_date] = (@htmlid_dates[vul.html_id] && @htmlid_dates[vul.html_id][:create_datetime] ? @htmlid_dates[vul.html_id][:create_datetime].strftime('%Y/%m/%d') : nil)
          item[:update_date] = (@htmlid_dates[vul.html_id] && @htmlid_dates[vul.html_id][:update_datetime] ? @htmlid_dates[vul.html_id][:update_datetime].strftime('%Y/%m/%d') : nil)
          item
        }
        json_tsv_summary_table_progress_download(@vuls[:ids], create_json_data, executed_size, 200)
      end
    elsif format == 'playbook' then
      if f_params[:process] == 'init' then
        @vuls[:content_filtered] = @vuls[:content_filtered].where(:html_id => @vuls[:ids]) if @vuls[:ids].any?
        init_json_summary_table_progress_download(
          :ids => @vuls[:content_filtered].pluck(:id),
          :host_detail_info => {
            :hostname => @host.hostname,
            :os => (@hostinfo[:os] and @hostinfo[:os][:osname]) ? @hostinfo[:os][:osname] : '---'
          },
          :max_download_size => @vuls[:content_filtered].count
        )
      else
        create_json_data = Proc.new{|id|
          vul = @vuls[:content_filtered].find_by(:id => id)
          next unless vul
          issue = SidfmdIssue.where(:sidfmd_host_id => @id, :html_id => vul.html_id).first
          host_vul_scsid = vul.sidfmd_vv_scsids.find_by(:sidfmd_host_id => @id)
          next if !issue or !host_vul_scsid
          item = {}
          item[:affect_packages] = issue.affect_packages('')
          item
        }
        json_tsv_summary_table_progress_download(@vuls[:ids], create_json_data, executed_size, 200)
      end 
    elsif format == 'tsv' then
      convert_chars = ->(text){
        text.gsub('"', '”').gsub("'", "‘").gsub(',', '，').gsub("\n", ' ')
      }
      if f_params[:process] == 'init' then
        header = [ 'HTML_ID', '要対応', '対応状況', 'SRI', 'CVSS', 'CVE ID', 'CVSS V2', 'CVSS V3', '影響範囲', '攻撃コード', '経過日数', '検出日', '脆弱性タイトル', '概要', '履歴数', '履歴(日時,ユーザ名,メールアドレス,操作,操作内容,コメント)', '対策予定日', 'カスタムタグ', '登録日', '最終更新日' ]
        @vuls[:content_filtered] = @vuls[:content_filtered].where(:html_id => @vuls[:ids]) if @vuls[:ids].any?
        init_tsv_summary_table_progress_download(@vuls[:content_filtered].pluck(:id), header, @vuls[:content_filtered].count)
      else
        create_vul_ctags_for_file(@vuls[:ids])
        create_htmlid_create_update_time(@vuls[:content_filtered].where(:id => @vuls[:ids]).pluck(:html_id))
        create_tsv_data = Proc.new{|id|
          vul = @vuls[:content_filtered].find_by(:id => id)
          next unless vul
          issue = SidfmdIssue.where(:sidfmd_host_id => @id, :html_id => vul.html_id).first
          host_vul_scsid = vul.sidfmd_vv_scsids.find_by(:sidfmd_host_id => @id)
          next if !issue or !host_vul_scsid
          item = []
          ctags = (@vul_ctags_for_file[id.to_i] || [])
          item << (vul[:html_id] > 0 ? "#{vul[:html_id]}" : "KB#{-vul[:html_id]}")
          is_warning = if SidfmdIssue::STATUS_UNFINISHED.include?(host_vul_scsid.status_id) then
                         if @vuls[:idx] == :sri then
                           if host_vul_scsid.sri >= @corresp_required[:sri] then
                             true
                           else
                             false
                           end
                         else
                           if host_vul_scsid.cvss.nil? || host_vul_scsid.cvss >= @corresp_required[:cvss] then
                             true
                           else
                             false
                           end
                         end
                       else
                         false
                       end
          item << is_warning.to_s
          item << SidfmdIssue.status_id_to_s(host_vul_scsid[:status_id])
          item << SidfmdIssueSri::RISKS[host_vul_scsid[:sri]]
          cvss_label = SidfmdIssueCve.labelling(host_vul_scsid.cvss).to_s
          item << (cvss_label == 'notreg' ? '' : sprintf('%.1f', (host_vul_scsid.cvss ? host_vul_scsid.cvss : 0).to_f))
          cves = vul.get_summary_cves
          cvss_2_cves = []
          cvss_3_cves = []
          cve_ids = []
          cves.each do |cve|
            cve_ids << cve.cve_id
            cvss_2_cves << "#{cve.cve_id}|#{cve.c2_base_score}|#{cve.c2_vector_index}" if cve.c2_av
            cvss_3_cves << "#{cve.cve_id}|#{cve.c3_base_score}|#{cve.c3_vector_index}" if cve.c3_av
          end
          item << cve_ids.join(',')
          item << cvss_2_cves.join(',')
          item << cvss_3_cves.join(',')
          item << SidfmdArea.where(:area_id => vul[:softek_area_id]).first.name
          item << ((cvss_label != 'notreg' and SidfmdIssue::STATUS_UNFINISHED.include?(host_vul_scsid.status_id)) ? vul.exploitable : nil).to_s
          elapsed_created_on = host_vul_scsid.created_on ? host_vul_scsid.created_on : Time.current
          item << (SidfmdIssue::STATUS_UNFINISHED.include?(host_vul_scsid.status_id) ? (((Time.current - elapsed_created_on) / 24.0 / 60.0 / 60.0).to_i.to_s) : '')
          item << elapsed_created_on.strftime('%Y/%m/%d')
          item << SidfmdVulView.display_title(vul.title)
          item << (vul[:short_description] ? vul[:short_description] : '')
          vul_journals = Journal.where(:journalized_id => issue.issue_id).order(:created_on)
          vul_journals_size = vul_journals.count 
          item << vul_journals_size.to_s
          count_journal_lines = 0
          journal_text_size = 0
          journal_text_items = []
          vul_journals.each_with_index do |journal, idx|
            j_text = ''
            if journal[:created_on] then
              j_text += journal[:created_on].strftime('%Y/%m/%d %H:%M:%S') + ','
            end
            user = User.find(journal[:user_id])
            j_text += user.anonymous? ?  '削除済みユーザ,' : convert_chars.(view_context.display_user_name(user)) + ','
            j_text += user.mail ? "#{user.mail}," : ','
            detail_flag = false
            JournalDetail.where(:journal_id => journal[:id]).order(:id).each do |detail|
              case detail[:prop_key]
              when 'status_id' then
                detail_flag = true
                j_text += 'ステータス変更' + ','
                j_text += "[#{SidfmdIssue.status_id_to_s(detail[:old_value].to_i)}]から[#{SidfmdIssue.status_id_to_s(detail[:value].to_i)}]へ,"
              when 'due_date' then
                detail_flag = true
                j_text += '対策予定日の変更' + ','
                j_text += "[#{SidfmdIssue.due_date_to_s(detail[:old_value])}]から[#{SidfmdIssue.due_date_to_s(detail[:value])}]へ,"
              end
            end
            if !detail_flag then
              j_text += ',,'
            end
            j_text += convert_chars.(Journal.find(journal[:id]).notes)
            j_text_size = j_text.size
            if vul_journals_size != 1 then
              if idx != 0 then
                j_text_size += 1 #改行文字をカウント
              end
            end
            total_text_size = journal_text_size + j_text_size
            if journal_text_items.any? then
              while total_text_size > MAX_CELL_LENGTH do
                oldest_journal = journal_text_items.shift
                journal_text_size -= oldest_journal.size + 1 #改行文字分も引く
                total_text_size -= oldest_journal.size + 1 #改行文字分も引く 
                if journal_text_items.size == 0 then
                  if j_text_size > MAX_CELL_LENGTH then
                    #最大文字数までは出力する
                    j_text = j_text.slice(0..MAX_CELL_LENGTH - 1)
                    j_text_size = MAX_CELL_LENGTH
                    total_text_size = MAX_CELL_LENGTH
                  end
                  break
                end
              end
            elsif vul_journals_size == 1 and total_text_size > MAX_CELL_LENGTH then
              #最大文字数までは出力する
              j_text = j_text.slice(0..MAX_CELL_LENGTH - 1)
              j_text_size = MAX_CELL_LENGTH
            end
            while journal_text_items.size > MAX_LINE_NUM do
              oldest_journal = journal_text_items.shift
              journal_text_size -= oldest_journal.size
            end
            journal_text_size += j_text_size
            journal_text_items << j_text
          end
          item << journal_text_items.reverse.join("\n")
          item << (host_vul_scsid[:due_date] ? host_vul_scsid[:due_date].strftime('%Y/%m/%d') : '')
          item << ctags.collect { |tag| escape_category_tsv(tag[:keyword]) }.join(',')
          item << (@htmlid_dates[vul.html_id] && @htmlid_dates[vul.html_id][:create_datetime] ? @htmlid_dates[vul.html_id][:create_datetime].strftime('%Y/%m/%d') : '')
          item << (@htmlid_dates[vul.html_id] && @htmlid_dates[vul.html_id][:update_datetime] ? @htmlid_dates[vul.html_id][:update_datetime].strftime('%Y/%m/%d') : '')
          item
        }
        json_tsv_summary_table_progress_download(@vuls[:ids], create_tsv_data, executed_size, 200)
      end
    elsif format == 'tsv-package-info' then
      if f_params[:process] == 'init' then
        header = [ 'ホストID', 'ホスト名', 'パッケージ名' ]
        pkgs = []
        last_queue = @host.get_last_queue
        if last_queue then
          pkgs = last_queue.sidfmd_queue_pkgs.map{|x| x.pkg }
        end
        init_tsv_summary_table_progress_download(pkgs, header, pkgs.size)
      else
        create_tsv_data = Proc.new{|pkg|  
          item = []
          item << @host.id.to_s
          item << @host.hostname
          item << pkg
          item
        }
        json_tsv_summary_table_progress_download(@vuls[:pkgs], create_tsv_data, executed_size, 200)
      end
    elsif format == 'tsv-affect-products' then
      if f_params[:process] == 'init' then
        header = [ 'HTML_ID', '種類', 'パッケージ名/製品名' ]
        @vuls[:content_filtered] = @vuls[:content_filtered].where(:html_id => @vuls[:ids]) if @vuls[:ids].any?
        init_tsv_summary_table_progress_download(@vuls[:content_filtered].pluck(:id), header, @vuls[:content_filtered].count)
      else
        prod_items = Set.new
        lprod_items = Set.new
        pkg_items = Set.new
        create_tsv_data = Proc.new{|id|
          vul = @vuls[:content_filtered].find_by(:id => id)
          next unless vul
          issue = SidfmdIssue.where(:sidfmd_host_id => @id, :html_id => vul.html_id).first
          next unless issue
          issue.affect_products.each do |prod|
            item = []
            item << (issue.html_id > 0 ? "#{issue.html_id}" : "KB#{-issue.html_id}")
            item << 'manual_product'
            item << prod[:display_name]
            session[:export_data][:content] << item
          end

          issue.affect_lproducts.each do |lprod|
            item = []
            item << (issue.html_id > 0 ? "#{issue.html_id}" : "KB#{-issue.html_id}")
            item << 'local_product'
            item << lprod[:display_name]
            session[:export_data][:content] << item
          end

          issue.affect_packages('').each do |pkg|
            item = []
            item << (issue.html_id > 0 ? "#{issue.html_id}" : "KB#{-issue.html_id}")
            item << 'package'
            item << pkg[:fullname]
            session[:export_data][:content] << item
          end
          item = nil
        }
        json_tsv_summary_table_progress_download(@vuls[:ids], create_tsv_data, executed_size, 200)
      end
    end
  end

  def download
    f_params = params.fetch(:download, {}).permit(:format)
    case f_params[:format]
    when 'json' then
      send_summary_table_json_file("vmlists-#{@host.hostname}-json-#{Time.current.strftime('%Y%m%d%H%M%S')}.json")
    when 'playbook' then
      send_summary_table_playbook_file("vmplaybook-#{@host.hostname}-yml-#{Time.current.strftime('%Y%m%d%H%M%S')}.yml")
    when 'tsv' then
      send_summary_table_tsv_file("vmlists-#{@host.hostname}-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv")
    when 'tsv-package-info' then
      send_summary_table_tsv_file("#{@host.hostname}-packages-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv")
    when 'tsv-affect-products' then
      get_tsv_affect_products_sort_value = ->(x){
        r = case x[1]
            when 'package' then
              1
            when 'manual_product' then
              2
            when 'local_product' then
              3
            else
              4 #invalid
            end
      }
      header = session[:export_data][:content].shift
      session[:export_data][:content] = session[:export_data][:content].sort{|a, b|
        r = get_tsv_affect_products_sort_value.(a) <=> get_tsv_affect_products_sort_value.(b)
        if r == 0 then
          r = a[2] <=> b[2]
          if r == 0 then
            r = a[0] <=> b[0]
          end
        end
        r
      }
      session[:export_data][:content].unshift(header)
      send_summary_table_tsv_file("#{@host.hostname}-affect-products-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv")
    end
  end

=begin
  #旧ダウンロード
  def download
    pages_params = params.fetch(:download, {}).permit(:format, :idx, :order, :filters, :htmlids)
    format =  pages_params[:format]
    if ![ 'json', 'tsv', 'tsv-package-info', 'playbook', 'tsv-affect-products' ].include?(format) then
      redirect_to :action => :index , :notice => '不正な入力です。' and return
    end
    @vuls = { :url => request.url }
    @vuls[:idx] = (pages_params[:idx] || 'sri').to_sym
    @vuls[:order] = pages_params[:order] || 'title_asc'
    @vuls[:filters] = JSON.parse((pages_params[:filters] || '{}'), symbolize_names: true)
    @vuls[:htmlids] = []
    begin
      @vuls[:htmlids] = JSON.parse(pages_params[:htmlids] || '[]')
    rescue JSON::ParserError => e
      logger.error(e)
    end
    check_filters
    @corresp_required = Sidfm.corresp_required
    create_all_content
    @vuls[:corresp_required_vul_size] =
      @vuls[:all_content].sum {|x| x[:warning][@vuls[:idx]] ? 1 : 0 }
    @vuls[:fixed_corresp_required_vul_size] =
      @vuls[:all_content].sum {|x| x[:fixed_warning][@vuls[:idx]] ? 1 : 0 }
    @vuls[:not_fixed_corresp_required_vul_size] =
      @vuls[:corresp_required_vul_size] - @vuls[:fixed_corresp_required_vul_size]
    @vuls[:all_vul_size] = @vuls[:all_content].size
    filter_content
    @hostinfo = create_host_info
   if format == 'json' || format == 'playbook' then
      json = {}
      temp_order = @vuls[:order].to_s.split(/_/)
      json[:order] = { :by => temp_order[0], :type => temp_order[1], }
      json[:filters] = @vuls[:filters]
      json[:idx] = @vuls[:idx]
      json[:host] = {
        :id => @host.id,
        :hostname => @host.hostname,
        :os => (@hostinfo[:os] and @hostinfo[:os][:osname]) ? @hostinfo[:os][:osname] : '---',
        :last_checked_at =>  @host.last_checked_at ? @host.last_checked_at.strftime('%Y/%m/%d %H:%M:%S') : ( @hostinfo[:queue] ? @hostinfo[:queue].created_at.strftime('%Y/%m/%d %H:%M:%S') : '未同期' ),
        :is_agent => @host.is_agent,
      }
      temp_s_cat = @host.system_category
      json[:host][:system_category] = temp_s_cat ? { :id => temp_s_cat.id, :name => temp_s_cat.name } : { :id => 0, :name => '---' }
      json[:host][:no_system_categories] = SidfmdCategory.where(
        :id => (@role_admin ? @host.no_system_categories : @host.no_system_categories(User.current)).select(:sidfmd_category_id)
      ).order('LOWER(name COLLATE"C")')
       .collect {|x| { :id => x.id, :name => x.name } }
      temp_pkgs = @hostinfo[:pkgs]
      json[:host][:packages] = temp_pkgs ? temp_pkgs.collect {|x| x.pkg } : []
      temp_apps = @hostinfo[:apps]
      json[:host][:products] = temp_apps ? temp_apps.collect {|x| { :name => x.name, :version => x.version, :path => x.path } } : []
      temp_lapps = @hostinfo[:lapps]
      json[:host][:manual_products] = temp_lapps ? temp_lapps.collect {|x| x.get_name }.compact : []
      temp_lproducts = @hostinfo[:lproducts]
      json[:host][:local_products] = temp_lproducts ? temp_lproducts.collect {|x| x.get_name }.compact : []
      json[:host][:memo] = @host.memo
      json[:content] = []
      @vuls[:content_filtered].each do |vul|
        if @vuls[:htmlids].size != 0 and !@vuls[:htmlids].include?(vul[:html_id]) then
          next
        end
        item = {}
        if format == 'json' then 
          item[:html_id] = vul[:html_id] > 0 ? "#{vul[:html_id]}" : "KB#{-vul[:html_id]}"
          item[:issue_id] = vul[:issue_id]
          item[:title] = vul[:title]
          item[:short_description] = (vul[:short_description] ? vul[:short_description] : '')
          item[:warning] = vul[:warning][@vuls[:idx]]
          item[:status] = SidfmdIssue.status_id_to_s(vul[:status_id])
          item[:sri] = SidfmdIssueSri::RISKS[vul[:sri]]
          t_cvss = vul[:cvss]
          item[:cvss] = (t_cvss[:label] == :notreg ? '' : sprintf('%.1f', (t_cvss[:score] ? t_cvss[:score] : 0).to_f))
          cves = get_filtered_cves(vul[:cves], false)
          cvss_2_cves = []
          cvss_3_cves = []
          cve_ids = []
          cves.each do |cve|
            cve_ids << cve[:cve_id]
            cve[:c2_vector] = nil if cve[:c2_vector] && cve[:c2_vector].size == 0
            cve[:c3_vector] = nil if cve[:c3_vector] && cve[:c3_vector].size == 0
            cvss_2_cves << { :id => cve[:cve_id], :vector => cve[:c2_vector] }
            cvss_3_cves << { :id => cve[:cve_id], :vector => cve[:c3_vector] }
          end
          item[:cve_ids] = cve_ids
          item[:cvss_v2] = cvss_2_cves
          item[:cvss_v3] = cvss_3_cves
          item[:area] = SidfmdArea.where(:area_id => vul[:softek][:area_id]).first.name
          item[:exploitable] = (t_cvss[:label] != :notreg ? vul[:exploitable] : nil)
          item[:elapsed_days] = (vul[:created_on][:elapsed_days] >= 0 ? vul[:created_on][:elapsed_days] : '')
          item[:created_on] = vul[:created_on][:display]
          item[:due_date] = (vul[:due_date] ? vul[:due_date].strftime('%Y/%m/%d') : '')
          item[:journals] = []
          vul[:journals].sort_by{|j| j[:created_on]}.each do |journal|
            j_item = {}
            j_item[:id] = journal[:id]
            j_item[:created_on] = journal[:created_on] ? journal[:created_on].strftime('%Y/%m/%d %H:%M:%S') : '---'
            user = User.find(journal[:user_id])
            j_item[:user] = { :id => journal[:user_id], :name => user.anonymous? ? '削除済みユーザ' : view_context.display_user_name(user) }
            j_item[:notes] = Journal.find(journal[:id]).notes
            j_item[:details] = []
            JournalDetail.where(:journal_id => journal[:id]).order(:id).each do |detail|
              jd_item = {}
              jd_item[:id] = detail[:id]
              case detail[:prop_key]
              when 'status_id' then
                jd_item[:action] = 'ステータス変更'
                jd_item[:from] = SidfmdIssue.status_id_to_s(detail[:old_value].to_i)
                jd_item[:to] = SidfmdIssue.status_id_to_s(detail[:value].to_i)
              when 'due_date' then
                jd_item[:action] = '対策予定日の変更'
                jd_item[:from] = SidfmdIssue.due_date_to_s(detail[:old_value])
                jd_item[:to] = SidfmdIssue.due_date_to_s(detail[:value])
              else
                next
              end
              j_item[:details] << jd_item
            end
            item[:journals] << j_item
          end
          item[:journals].reverse!
        end
        item[:affect_packages] = []
        item[:affect_products] = []
        issue = SidfmdIssue.find(vul[:sidfmd_issue_id])
        if issue then
          item[:affect_packages] = issue.affect_packages('')
          item[:affect_products] = issue.affect_products + issue.affect_lproducts
        end
        json[:content] << item
      end
      
      if format == 'playbook' then
        a = GenerateAnsiblePlaybook.new(json.to_json)
        playbook = a.generate
        if playbook != 'error' && playbook != 'no_packages' then 
          send_data(playbook, :filename => "vmplaybook-#{@host.hostname}-yml-#{Time.current.strftime('%Y%m%d%H%M%S')}.yml", :disposition => 'attachment')
        end
      else
        send_data(json.to_json, :filename => "vmlists-#{@host.hostname}-json-#{Time.current.strftime('%Y%m%d%H%M%S')}.json", :disposition => 'attachment')
      end
    elsif format == 'tsv' then
      convert_chars = ->(text){
        text.gsub('"', '”').gsub("'", "‘").gsub(',', '，').gsub("\n", ' ')
      }
      header = [ 'HTML_ID', '要対応', '対応状況', 'SRI', 'CVSS', 'CVE ID', 'CVSS V2', 'CVSS V3', '影響範囲', '攻撃コード', '経過日数', '検出日', '脆弱性タイトル', '概要', '履歴数', '履歴(日時,ユーザ名,メールアドレス,操作,操作内容,コメント)', '対策予定日' ]
      header = header.collect {|x| x.encode('UTF-16LE') }
      csv = CSV.generate("#{CSV_BOM}".force_encoding('UTF-16LE'), :col_sep => "\t".encode('UTF-16LE')) do |o|
        o << header
        item = []
        @vuls[:content_filtered].each do |vul|
          if @vuls[:htmlids].size != 0 and !@vuls[:htmlids].include?(vul[:html_id]) then
            next
          end
          item.clear
          item << (vul[:html_id] > 0 ? "#{vul[:html_id]}" : "KB#{-vul[:html_id]}").encode('UTF-16LE')
          item << vul[:warning][@vuls[:idx]].to_s.encode('UTF-16LE')
          item << SidfmdIssue.status_id_to_s(vul[:status_id]).encode('UTF-16LE')
          item << SidfmdIssueSri::RISKS[vul[:sri]].encode('UTF-16LE')
          t_cvss = vul[:cvss]
          item << (t_cvss[:label] == :notreg ? '' : sprintf('%.1f', (t_cvss[:score] ? t_cvss[:score] : 0).to_f)).encode('UTF-16LE')
          cves = get_filtered_cves(vul[:cves], false)
          cvss_2_cves = []
          cvss_3_cves = []
          cve_ids = []
          cves.each do |cve|
            cve_ids << cve[:cve_id]
            cvss_2_cves << "#{cve[:cve_id]}|#{cve[:c2_vector]}" if cve[:c2_vector] && cve[:c2_vector].size > 0
            cvss_3_cves << "#{cve[:cve_id]}|#{cve[:c3_vector]}" if cve[:c3_vector] && cve[:c3_vector].size > 0
          end
          item << cve_ids.join(',').encode('UTF-16LE')
          item << cvss_2_cves.join(',').encode('UTF-16LE')
          item << cvss_3_cves.join(',').encode('UTF-16LE')
          item << SidfmdArea.where(:area_id => vul[:softek][:area_id]).first.name.encode('UTF-16LE')
          item << (t_cvss[:label] != :notreg ? vul[:exploitable].to_s : '').encode('UTF-16LE')
          item << (vul[:created_on][:elapsed_days] >= 0 ? vul[:created_on][:elapsed_days].to_s : '').encode('UTF-16LE')
          item << vul[:created_on][:display].encode('UTF-16LE')
          item << vul[:title].encode('UTF-16LE')
          item << (vul[:short_description] ? vul[:short_description] : '').encode('UTF-16LE')
          item << vul[:journals].size.to_s.encode('UTF-16LE')
          count_journal_lines = 0
          journal_text_size = 0
          journal_text_items = []
          vul[:journals].sort_by{|j| j[:created_on]}.each_with_index do |journal, idx|
            j_text = ''
            if journal[:created_on] then
              j_text += journal[:created_on].strftime('%Y/%m/%d %H:%M:%S') + ','
            end
            user = User.find(journal[:user_id])
            j_text += user.anonymous? ?  '削除済みユーザ,' : convert_chars.(view_context.display_user_name(user)) + ','
            j_text += user.mail ? "#{user.mail}," : ','
            detail_flag = false
            JournalDetail.where(:journal_id => journal[:id]).order(:id).each do |detail|
              case detail[:prop_key]
              when 'status_id' then
                detail_flag = true
                j_text += 'ステータス変更' + ','
                j_text += "[#{SidfmdIssue.status_id_to_s(detail[:old_value].to_i)}]から[#{SidfmdIssue.status_id_to_s(detail[:value].to_i)}]へ,"
              when 'due_date' then
                detail_flag = true
                j_text += '対策予定日の変更' + ','
                j_text += "[#{SidfmdIssue.due_date_to_s(detail[:old_value])}]から[#{SidfmdIssue.due_date_to_s(detail[:value])}]へ,"
              end
            end
            if !detail_flag then
              j_text += ',,'
            end
            j_text += convert_chars.(Journal.find(journal[:id]).notes)
            j_text_size = j_text.size
            if vul[:journals].size != 1 then
              if idx != 0 then
                j_text_size += 1 #改行文字をカウント
              end
            end
            total_text_size = journal_text_size + j_text_size
            if journal_text_items.any? then
              while total_text_size > MAX_CELL_LENGTH do
                oldest_journal = journal_text_items.shift
                journal_text_size -= oldest_journal.size + 1 #改行文字分も引く
                total_text_size -= oldest_journal.size + 1 #改行文字分も引く 
                if journal_text_items.size == 0 then
                  if j_text_size > MAX_CELL_LENGTH then
                    #最大文字数までは出力する
                    j_text = j_text.slice(0..MAX_CELL_LENGTH - 1)
                    j_text_size = MAX_CELL_LENGTH
                    total_text_size = MAX_CELL_LENGTH
                  end
                  break
                end
              end
            elsif vul[:journals].size == 1 and total_text_size > MAX_CELL_LENGTH then
              #最大文字数までは出力する
              j_text = j_text.slice(0..MAX_CELL_LENGTH - 1)
              j_text_size = MAX_CELL_LENGTH
            end
            while journal_text_items.size > MAX_LINE_NUM do
              oldest_journal = journal_text_items.shift
              journal_text_size -= oldest_journal.size
            end
            journal_text_size += j_text_size
            journal_text_items << j_text
          end
          item << journal_text_items.reverse.join("\n").encode('UTF-16LE')
          item << (vul[:due_date] ? vul[:due_date].strftime('%Y/%m/%d') : '').encode('UTF-16LE')
          o << item
        end
      end
      send_data(csv, :filename => "vmlists-#{@host.hostname}-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv", :disposition => 'attachment')
    elsif format == 'tsv-package-info' then
      header = [ 'ホストID', 'ホスト名', 'パッケージ名' ]
      header = header.collect {|x| x.encode('UTF-16LE') }
      csv = CSV.generate("#{CSV_BOM}".force_encoding('UTF-16LE'), :col_sep => "\t".encode('UTF-16LE')) do |o|
        o << header
        item = []
        last_queue = SidfmdHost.find_by_hostname(@host.hostname).get_last_queue
        if last_queue then
          pkgs = last_queue.sidfmd_queue_pkgs
          if pkgs.size > 0 then
            pkgs.each do |x|
              item.clear
              item << @host.id.to_s.encode('UTF-16LE')
              item << @host.hostname.encode('UTF-16LE')
              item << x.pkg.encode('UTF-16LE')
              o << item
            end
          end
        end
      end
      send_data(csv, :filename => "#{@host.hostname}-packages-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv", :disposition => 'attachment')
    elsif
      format == 'tsv-affect-products' then
      header = [ 'HTML_ID', '種類', 'パッケージ名/製品名' ]
      header = header.collect {|x| x.encode('UTF-16LE') }
      prod_items = Set.new
      lprod_items = Set.new
      pkg_items = Set.new
      @vuls[:content_filtered].each do |vul|
        if @vuls[:htmlids].size != 0 and !@vuls[:htmlids].include?(vul[:html_id]) then
          next
        end
        issue = SidfmdIssue.find(vul[:sidfmd_issue_id])
        issue.affect_products.each do |prod|
          item = []
          item << (issue.html_id > 0 ? "#{issue.html_id}" : "KB#{-issue.html_id}").encode('UTF-16LE')
          item << 'manual_product'.encode('UTF-16LE')
          item << prod[:display_name].encode('UTF-16LE')
          prod_items.add item
        end

        issue.affect_lproducts.each do |lprod|
          item = []
          item << (issue.html_id > 0 ? "#{issue.html_id}" : "KB#{-issue.html_id}").encode('UTF-16LE')
          item << 'local_product'.encode('UTF-16LE')
          item << lprod[:display_name].encode('UTF-16LE')
          lprod_items.add item
        end
    
        issue.affect_packages('').each do |pkg|
          item = []
          item << (issue.html_id > 0 ? "#{issue.html_id}" : "KB#{-issue.html_id}").encode('UTF-16LE')
          item << 'package'.encode('UTF-16LE')
          item << pkg[:fullname].encode('UTF-16LE')
          pkg_items.add item
        end
      end
      csv = CSV.generate("#{CSV_BOM}".force_encoding('UTF-16LE'), :col_sep => "\t".encode('UTF-16LE')) do |o|
        o << header
        #package
        pkg_items.sort_by{|item| item[2]}.each do |item|
          o << item
        end
        #Sidfmプロダクトから先に表示する
        prod_items.sort_by{|item| item[2]}.each do |item|
          o << item
        end
        #ローカルプロダクト
        lprod_items.sort_by{|item| item[2]}.each do |item|
          o << item
        end
      end
      send_data(csv, :filename => "#{@host.hostname}-affect-products-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv", :disposition => 'attachment')
    end

    return
    redirect_to :action => :index, :notice => 'ファイルの出力に失敗しました。'
    
  end
=end

  def send_mail
    filtered_params = params.permit(:recipients, :subject, :message)
  begin
    mail = {}
    mail[:from] = User.current.mail
    email_with_name = []
    recipients = JSON.parse(filtered_params[:recipients])
    recipients.each do |address, name|
      email_name = %("#{name}" <#{address}>)
      email_with_name << email_name
    end
    #mail[:to] = email_with_name
    mail[:subject] = filtered_params[:subject]
    mail[:message] = filtered_params[:message]
    send_mail_num = SidfmdMailer::SEND_MAIL_NUM
    time_out_second = SidfmdMailer::SEND_MAIL_TIMEOUT_SECOND
    while (send_mail_targets = email_with_name.shift(send_mail_num)).size > 0 do
      threads = send_mail_targets.map{|fullname_and_address|
        Thread.new(mail.deep_dup, fullname_and_address){|cp_mail, data|
          ret = {}
          cp_mail[:to] = data
          begin
            SidfmdMailer.send_mail(cp_mail).deliver_now
          rescue Exception => e
            ret[:exception] = e
          end
          ret
        }
      }
      threads.each do |result|
        result = result.join(time_out_second)
        if result then
          result = result.value
          if result[:exception] then logger.error("failed send mail. #{result[:exception]}") end
        else
          logger.error("time-out of sending a mail")
        end
      end
    end
    #SidfmdMailer.send_mail(mail).deliver_now
  rescue Exception => e
    log_exception(e)
    render :layout => false, :json => { :err_message => 'メールの送信に失敗しました。' } and return
    #redirect_to :action => :index, :notice => e and return
  end
    render :layout => false, :json => { :err_message => '' }
  end

  def get_journal
    filtered_params = params.fetch(:data, {}).permit(:html_id)
    data = []
    html_id = filtered_params[:html_id] 
    issue_id = SidfmdIssue.find_by(:html_id => html_id).id
    journal = Journal.where(:journalized_id => issue_id).map{|j| [j.user_id, j.notes, j.created_on]}
    users = Hash[User.where(:id => journal.map{|j| j[0]}).map{|user| [user.id, user.login]}]
    journal.each  do |j|
      j.push(users[j[0]])
      data.push(j)
    end
    render :layout => false, :json => data.to_json
  end

=begin
  def api_get_display_summary_rows
    GC.start
    f_params = params.permit(:ids, :idx, :checked, :filters)
    html_ids = JSON.parse(f_params[:ids])
    filters = JSON.parse(f_params[:filters], :symbolize_names => true)
    checked = JSON.parse(f_params[:checked]).map(&:to_i)
    @vuls = {}
    create_vul_view_content
    idx = f_params[:idx].to_sym
    corresp_required = Sidfm.corresp_required
    user_id = User.current.id
    ret = {}
    @vuls[:all_content].where(:html_id => html_ids).each do |vul_view|
      ret[vul_view.html_id] = create_summary_vul_html(
        vul_view,
        {
          :idx => idx,
          :corresp_required => corresp_required,
          :checked =>  checked,
          :scsid => vul_view.sidfmd_vv_scsids.find_by(:sidfmd_host_id => @id),
          :user_id => user_id
        }
      )
    end

    render :layout => false, :json => ret
  end
=end

  def api_get_display_summary_content
    GC.start
    f_params = params.fetch(:pages, {}).permit(:per_page, :history_filter_vul)
    parse_params
    @vuls[:per_page] = f_params[:per_page].to_i
    @corresp_required = Sidfm.corresp_required
    @vuls[:content_filtered] = apply_filters_to_vul_view_content
    user_id = User.current.id
    history_filter_vul = JSON.parse((f_params[:history_filter_vul] || '{}'), symbolize_names: true)
    is_history_filter_vul = history_filter_vul.size > 0
    @vuls[:content] = is_history_filter_vul ?
      @vuls[:content_filtered].where(:html_id => history_filter_vul[:html_id]).limit(1) :
      @vuls[:content_filtered].limit(@vuls[:per_page]).offset(@vuls[:offset])
    create_vul_ctags(@vuls[:content].pluck(:id))
    all_html_ids = (is_history_filter_vul ? @vuls[:content] : @vuls[:content_filtered]).pluck(:html_id)
    create_htmlid_create_update_time(all_html_ids)
    ret = {
      :rows => @vuls[:content].map{|vul_view|
        create_summary_vul_html(
          vul_view,
          {
            :idx => @vuls[:idx],
            :corresp_required => @corresp_required,
            :checked =>  @vuls[:checked],
            :scsid => vul_view.sidfmd_vv_scsids.find_by(:sidfmd_host_id => @id),
            :user_id => user_id
          }
        )
      },
      :total => (is_history_filter_vul ? @vuls[:content] : @vuls[:content_filtered]).count,
      :all_ids => all_html_ids,
      :sidfm_roles => @sidfm_roles,
      :user_cats => @user_category_ids
    }
    render :layout => false, :json => ret
  end

  def api_establish_filters_consistency
    f_params = params.permit(:filters)
    filters = JSON.parse(f_params[:filters], :symbolize_names => true)
    init_filter_keys if FILTER_KEYS.size == 0
    filters = establish_filters_consistency(filters, FILTER_KEYS)
    filters = filter_filters(
      :filters => filters,
      :role_admin => @role_admin,
      :user_category_ids => @user_category_ids
    )
    render :layout => false, :json => filters
  end

end
