# -*- coding: utf-8 -*-

require 'sidfmdlm_api'
require 'sidfm_controller_support'

class SidfmInformationsController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  MAX_CELL_LENGTH = 30000
  MAX_LINE_NUM = 512 # =最大改行数は511

  private

  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles
  
  before_action :get_vm_version
  before_action :clear_import_session
  before_action :clear_export_session, only: [ :index, :vuln, :vuln_list ]

  # 権限チェック
  def check_roles
    redirect_to home_path, :notice => '許可されていない操作です。' unless @sidfm_user
    unless @sidfm_roles.include?(:view) then
      redirect_to user_path(User.current), :notice => af_log_user('アクセス権限がありません。') and return
    end
    @role_admin = @sidfm_roles.include?(:admin)
    @user_category_ids = if @role_admin then
                           nil
                         else
                           User.current.categories.order(:id).pluck(:id)
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => @user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
                     end
    @cache_prefix = @role_admin ? '/infos/admin/0' : "/infos/user/#{@user_category_ids.join(",")}"
  end

  def get_is_date(str)
    is_date = true
    begin
      Date.parse(str)
    rescue Exception => e
      is_date = false
    end
    is_date
  end

  def replace_br_tag(str)
    str.gsub(/<br>|<br\s*\/>/, '')
  end

  def escape_special_chars(text)
    text = text.gsub(/&/,'&amp;')
    text = text.gsub(/<|>|"|'/, '<' => '&lt;', '>' => '&gt;', '"' => '&quot;', "'" => '&#39;', '`' => '&#x60;')
  end

  def get_cell_content(content)
    temp = content.split("\n")
    if temp.size > MAX_LINE_NUM then
      content = temp.slice(0..(MAX_LINE_NUM - 1)).join("\n")
    end
    #文字列の先頭と末尾はダブルクオーテーションであることに注意
    if content.size > MAX_CELL_LENGTH then
      temp = content.slice(1..(content.size - 2))
      content = '"' + temp.slice(0..(MAX_CELL_LENGTH - 3)) + '"'
    end
    content
  end  

  def get_local_vuln_data(local_vuln)
    local_vuln.information_content
  end

  def get_local_prod
    lprod = SidfmdLproduct.find_by_id(@id)
    if !lprod || !lprod.flg_enable then
      return nil
    end
    lprod_info = {}
    lprod_info[:id] = lprod.id
    lprod_info[:name] = lprod.name
    lprod_info[:is_support_end] = lprod.is_support_end
    lprod_info[:category] = 'ローカルプロダクト'
    lprod_info[:support_date] = lprod.support_date
    lprod_info[:create_datetime] = lprod.created_at
    lprod_info[:update_datetime] = lprod.updated_at
    versions = lprod.sidfmd_lversions.order(:version)
    formated_versions = []
    eols = []
    versions_info = {}
    versions.each_with_index do |v, i|
      formated_versions << {
        :id => v.id,
        :fullname => v.fullname,
        :version => v.version,
        :is_support_end => v.is_support_end,
        :position => i,
        :release_date => v.release_date,
        :support_date => v.support_date,
        :create_datetime => v.created_at,
        :update_datetime => v.updated_at
      }
      eols << {
         :name => v.fullname,
         :date => v.eol_date,
         :is_support_end => v.is_support_end,
         :url => nil
      }
      versions_info[v.id] = v
    end

    if versions.size == 0 then
      formated_versions << {
        :id => 0, #ローカルプロダクトでバージョンが無い場合、DBに「ALL」のバージョンはない
        :fullname => 'ALL',
        :version => 'ALL',
        :is_support_end => lprod_info[:is_support_end],
        :position => 0,
        :release_date => nil,
        :support_date => lprod_info[:support_date],
        :create_datetime => lprod_info[:create_datetime],
        :update_datetime => lprod_info[:update_datetime]
      }
    end

    lprod_info[:versions] = formated_versions
    vulns = get_local_prod_vulns(lprod)
    affect_hosts = get_local_prod_hosts(lprod, versions_info)
    return {:lprod_info => lprod_info, :eols => eols, :vulns => vulns, :affect_hosts => affect_hosts}
  end

  def get_local_prod_vulns(lproduct)
    local_vul_lapps = SidfmdLocalVulLapp.where(:sidfmd_lproduct_id => @id)
    vulns = SidfmdLocalVul.where(:id => local_vul_lapps.map(&:sidfmd_local_vul_id)).where(:flg_enable => true)
    ret = []
    vulns.each do |vul|
      r = {}
      r[:id] = vul.html_id
      r[:title] = vul.title
      r[:area_id] = vul.area_id
      r[:create_datetime] = vul.created_at
      r[:update_datetime] = vul.updated_at
      r[:risk_id] = vul.risk_id
      r[:affect_products] = [
        {
          :product => { :id => lproduct.id, :name => lproduct.name },
          :create_datetime => lproduct.created_at,
          :update_datetime => lproduct.updated_at
        }
      ]
      cves = vul.sidfmd_local_vul_cves
      r[:cves] = cves.map{|cve|
        {
          :cve_id => cve.cve_id,
          :cvss3 => {
            :vector => cve.c3_vector,
            :base_score => cve.c3_base_score,
            :create_datetime => cve.created_at,
            :update_datetime => cve.updated_at,
            :exploitable => cve.c3_exploitable
          }
        }
      }
      ret << r
    end
    return ret
  end

  def get_local_prod_hosts(lproduct, versions) #local productのレコードと、keyがidでvalueがversionのハッシュが引数
    user_host_ids_role = if @role_admin then
                           SidfmdHost
                             .joins(:sidfmd_categories_hosts)
                             .joins(:sidfmd_categories)
                             .where(:flg_delete => false)
                             .where(:sidfmd_categories => { :is_system => true })
                             .select(:id)
                             .distinct
                         else
                           @user_host_ids
                         end
    ret = []
    hostlapps = SidfmdHostLapp
      .joins(:sidfmd_host)
      .includes(:sidfmd_host)
      .where(:sidfmd_host_id => user_host_ids_role)
      .where("(sidfmd_host_lapps.sidfmd_lproduct_id = ? OR sidfmd_host_lapps.sidfmd_lversion_id IN (?))", @id, versions.keys)
      .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
      .order('display_name')
    hostlapps.each do |hostlapp|
      temp_ver = versions[hostlapp.sidfmd_lversion_id]
      if temp_ver then
        temp_ret = {
          :type => :version,
          #:sidfmd_hostapp_id => hostlapp.id,
          :sidfmd_host_id => hostlapp.sidfmd_host_id,
          :hostname => hostlapp.sidfmd_host.hostname,
          :product_id => @id,
          :name => lproduct.name,
          :version_id => hostlapp.sidfmd_lversion_id,
          :version => temp_ver.version,
          :full_name => temp_ver.fullname,
          :is_support_end => true,
          :release_date =>  nil,
          :support_date => nil,
          :eol_date => nil,
          :ext_name => hostlapp.sidfmd_host.ext_name,
          :display_name => (hostlapp.sidfmd_host.ext_name ? hostlapp.sidfmd_host.ext_name : hostlapp.sidfmd_host.hostname)
        }
        ret << temp_ret
      else
        temp_ret = {
          :type => :product,
          #:sidfmd_hostapp_id => hostapp.id,
          :sidfmd_host_id => hostlapp.sidfmd_host_id,
          :hostname => hostlapp.sidfmd_host.hostname,
          :product_id => @id,
          :name => lproduct.name,
          :is_support_end => true,
          :support_date => nil,
          :ext_name => hostlapp.sidfmd_host.ext_name,
          :display_name => (hostlapp.sidfmd_host.ext_name ? hostlapp.sidfmd_host.ext_name : hostlapp.sidfmd_host.hostname)
        }
        ret << temp_ret 
      end
    end
    return ret
  end

  public

  def index
    redirect_to :action => :vuln_list
  end

  def vuln_list
  end

  def vuln
    @id = params[:id].to_i
    if @id <= 0 then
      redirect_to :action => :vuln_list, :notice => 'idが不正です。' and return
    end
    local_vuln = SidfmdLocalVul.find_by_html_id(@id)
    if local_vuln then
      @local_vuln = get_local_vuln_data(local_vuln)
      @areas = SidfmdArea.order(:area_id).map{|area| {:id => area.area_id, :name => area.name} } 
      
      @risks = SidfmdRisk.order(:risk_id).map{|risk| {:id => risk.risk_id, :name => risk.name, :name_en => risk.name_en} }
      
      cat_labels = SidfmdCategory::LABELS
      @sri_categories = SidfmdCategory.where(:is_system => true).order(:id).map.with_index{|c, idx|
        {:id => c.id, :name => c.name, :key => cat_labels[idx] }
      }
    else
      @local_vuln = nil
    end
  end

  def product_index
  end

  def product
    id = params[:id]
    if id.index('lproduct') then
      @id = id.slice(/[0-9]+/).to_i
      if @id <= 0 then
        redirect_to :action => :product_index, :notice => 'idが不正です。' and return
      end
      data = get_local_prod
      if !data then
        redirect_to :action => :product_index, :notice => 'プロダクトが見つかりませんでした。' and return
      end
      @local_prod = data[:lprod_info]
      @local_prod_eols = data[:eols]
      @local_prod_vulns = data[:vulns]
      @local_prod_affect_hosts = data[:affect_hosts]
      @areas = SidfmdArea.order(:area_id).map{|area| {:id => area.area_id, :name => area.name} }
      @risks = SidfmdRisk.order(:risk_id).map{|risk| {:id => risk.risk_id, :name => risk.name, :name_en => risk.name_en} }
    else 
      @id = id.to_i
      if @id <= 0 then
        redirect_to :action => :product_index, :notice => 'idが不正です。' and return
      end
      @local_prod = nil
    end
  end

  def news
  end

  # プロダクトIDから登録済ホストを取得する
  # input: product_id (GET)
  # output: [ { :sidfmd_host_id => 123, :hostname => 'xxx' }, ... ]
  def api_prd2hosts
    product_id = params[:id].to_i
    if product_id <= 0 then
      render :layout => false, :json => [], :status => 400
      return
    end
    product = SidfmdProduct.find_by_pid(product_id)
    unless product then
      render :layout => false, :json => [], :status => 400
      return
    end
    versions = product.sidfmd_versions
    version_ids = versions.collect {|x| x.vid }
    user_host_ids_role = if @role_admin then
                           SidfmdHost
                             .joins(:sidfmd_categories_hosts)
                             .joins(:sidfmd_categories)
                             .where(:flg_delete => false)
                             .where(:sidfmd_categories => { :is_system => true })
                             .select(:id)
                             .distinct
                         else
                           @user_host_ids
                         end
    ret = []
    hostapps = SidfmdHostApp
      .joins(:sidfmd_host)
      .includes(:sidfmd_host)
      .where(:sidfmd_host_id => user_host_ids_role)
      .where("(sidfmd_host_apps.product_id = ? OR sidfmd_host_apps.version_id IN (?))", product_id, version_ids)
      .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
      .order('display_name')
      #.order("sidfmd_hosts.hostname")
    hostapps.each do |hostapp|
      temp_prod = SidfmdProduct.find_by_pid(hostapp.product_id)
      temp_ver = SidfmdVersion.find_by_vid(hostapp.version_id)
      if temp_ver and temp_ver.sidfmd_product then
        temp_ret = {
          :type => :version,
          :sidfmd_hostapp_id => hostapp.id,
          :sidfmd_host_id => hostapp.sidfmd_host_id,
          :hostname => hostapp.sidfmd_host.hostname,
          :product_id => temp_ver.sidfmd_product_id,
          :name => temp_ver.sidfmd_product.name,
          :version_id => hostapp.version_id,
          :version => temp_ver.version,
          :full_name => temp_ver.fullname,
          :is_support_end => temp_ver.is_support_end,
          :release_date => temp_ver.release_date,
          :support_date => temp_ver.support_date,
          :eol_date => temp_ver.eol_date,
          :ext_name => hostapp.sidfmd_host.ext_name,
          :display_name => (hostapp.sidfmd_host.ext_name ? hostapp.sidfmd_host.ext_name : hostapp.sidfmd_host.hostname)
        }
        ret << temp_ret
      elsif temp_prod then
        temp_ret = {
          :type => :product,
          :sidfmd_hostapp_id => hostapp.id,
          :sidfmd_host_id => hostapp.sidfmd_host_id,
          :hostname => hostapp.sidfmd_host.hostname,
          :product_id => hostapp.product_id,
          :name => temp_prod.name,
          :is_support_end => temp_prod.is_support_end,
          :support_date => temp_prod.support_date,
          :ext_name => hostapp.sidfmd_host.ext_name,
          :display_name => (hostapp.sidfmd_host.ext_name ? hostapp.sidfmd_host.ext_name : hostapp.sidfmd_host.hostname)
        }
        ret << temp_ret
      else
        # skip: invalid data
      end
    end
    # see vm_daemon
    queue_ids = SidfmdHost
      .where(:flg_delete => false)
      .where(:id => user_host_ids_role)
      .select("(SELECT id FROM sidfmd_queues WHERE sidfmd_hosts.id=sidfmd_queues.sidfmd_host_id ORDER BY created_at DESC LIMIT 1) AS lastest_queue_id")
    case product.name
    when 'Apache Struts' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'Apache Struts')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'Tomcat' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'Apache Tomcat')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'PHP' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'PHP')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'Ruby' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'ruby')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        unless temp_version then
          if queue_app.version =~ /(\d+\.\d+\.\d+)/ then
            temp_version = product.sidfmd_versions.where(:version => Regexp.last_match(1)).first
          end
        end
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path,
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    when 'Oracle Java SE SDK', 'Oracle Java SE JDK', 'Oracle Java SE JRE' then
      queue_apps = SidfmdQueueApp
        .joins(:sidfmd_queue)
        .joins(:sidfmd_queue => [ :sidfmd_host ])
        .includes(:sidfmd_queue => [ :sidfmd_host ])
        .where(:sidfmd_hosts => { :flg_delete => false })
        .where(:sidfmd_queues => { :sidfmd_host_id => user_host_ids_role })
        .where(:sidfmd_queue_id => queue_ids)
        .where(:name => 'Oracle Java')
        .select('COALESCE(sidfmd_hosts.ext_name, sidfmd_hosts.hostname) AS display_name')
        .order('display_name')
        #.order("sidfmd_hosts.hostname")
      queue_apps.each do |queue_app|
        temp_version = product.sidfmd_versions.where(:version => queue_app.version).first
        unless temp_version then
          if queue_app.version =~ /(\d+\.\d+\.\d+)/ then
            temp_version = product.sidfmd_version.where(:version => Regexp.last_match(1)).first
          end
        end
        queue_host = queue_app.sidfmd_queue.sidfmd_host
        ret << {
          :type => :queue_app,
          :sidfmd_queue_app_id => queue_app.id,
          :sidfmd_host_id => queue_host.id,
          :hostname => queue_host.hostname,
          #:sidfmd_host_id => queue_app.sidfmd_queue.sidfmd_host.id,
          #:hostname => queue_app.sidfmd_queue.sidfmd_host.hostname,
          :product_id => product_id,
          :name => queue_app.name,
          :version_id => temp_version ? temp_version.id : nil,
          :version => queue_app.version,
          :path => queue_app.path, 
          :ext_name => queue_host.ext_name,
          :display_name => (queue_host.ext_name ? queue_host.ext_name : queue_host.hostname)
        }
      end
    end
    ret.sort! {|x, y| x[:display_name] <=> y[:display_name] }
    render :layout => false, :json => ret
  end

  # 指定したhtml_idの配列から脆弱性詳細の有無を返す
  # input:  html_ids=JSON.stringify([ 1, 2, 3, ... ]) (POST)
  # output: { "1" => true, "2" => false, }
  # FIXME: under constructions
  def api_htmlids2issue
    htmlids = params[:html_ids]
    ret = {}
    unless htmlids then
      htmlids = []
    end
    unless htmlids.respond_to?(:collect) then
      render :layout => false, :json => ret.to_json, :status => 400
      return
    end
    begin
      htmlids = htmlids.collect {|x| x.to_i }.sort.uniq
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => ret.to_json, :status => 400
      return
    end
    if !htmlids.respond_to?(:each) or !htmlids.respond_to?(:'[]') then
      render :layout => false, :json => ret.to_json, :status => 400
      return
    end
    ret = SidfmdIssue
      .joins(:issue)
      .where(:sidfmd_host_id => if @role_admin then
                                  SidfmdHost
                                    .joins(:sidfmd_categories_hosts)
                                    .joins(:sidfmd_categories)
                                    .where(:flg_delete => false)
                                    .where(:sidfmd_categories => { :is_system => true })
                                    .select(:id)
                                    .distinct
                                else
                                  @user_host_ids
                                end)
      .where(:html_id => htmlids)
      .select(:html_id)
      .group(:html_id)
      .count
    render :layout => false, :json => ret.to_json
  end

  def read_download_data
    clear_export_session    
    GC.start
    f_params = params.permit(:vuln_info)
    begin 
      vuln_info = JSON.parse(f_params[:vuln_info], symbolize_names: true)
    rescue JSON::ParserError => e
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return  
    end
  
    err_message = ''
    session[:export_data] = {}
    session[:export_data][:content] = []

    header = [ 
               'SIDfmID', 'タイトル', 'CVSSスコア', '影響範囲', '危険度', 
               'CVE ID', 'CVSS V2', 'CVSS V3', '攻撃コード', '影響製品', 'システムカテゴリ別のリスクの影響度(SRI)', 'SRI補足情報', 
               '概要', '影響を受ける製品', '影響などの確認方法', '対処方法', '関連情報', '更新履歴',
               '登録日', '最終更新日'
             ]
    begin
      session[:export_data][:content] << header
      item = []
      item << vuln_info[:id].to_s
      item << vuln_info[:title].to_s
      item << (vuln_info[:max_cvss][:base_score] > -1 ? vuln_info[:max_cvss][:base_score] : "未評価").to_s
      item << vuln_info[:area][:name].to_s
      item << vuln_info[:risk][:name].to_s
      cve_ids = []
      cvss_2_cves = []
      cvss_3_cves = []
      vuln_info[:cves].each do |cve|
        cve_ids << cve[:cve_id].to_s
        if cve[:cvss2][:vector] then
          cvss_2_cves << "#{cve[:cve_id]}|#{((cve[:cvss2] and cve[:cvss2][:base_score] and cve[:base_score].to_i >= 0) ? cve[:cvss2][:base_score] : 0.0)}|#{cve[:cvss2][:vector]}" 
        end
        if cve[:cvss3][:vector] then
          cvss_3_cves << "#{cve[:cve_id]}|#{((cve[:cvss3] and cve[:cvss3][:base_score] and cve[:base_score].to_i >= 0) ? cve[:cvss3][:base_score] : 0.0)}|#{cve[:cvss3][:vector]}"
        end
      end
      item << cve_ids.join(',')
      item << cvss_2_cves.join(',')
      item << cvss_3_cves.join(',')
      item << (vuln_info[:exploitable].to_s == 'true').to_s

      products = vuln_info[:affect_products].map{|prod| escape_category_tsv("#{prod[:product][:id] ? prod[:product][:id] : prod[:product][:lproduct_id]}|#{prod[:product][:name]}")}
      while (products.join(',').length > MAX_CELL_LENGTH) do
        products.pop
      end      
      item << products.join(',')
      item << vuln_info[:sris].sort_by{|sri| sri[:category_id]}
                              .map{|sri| escape_category_tsv("#{sri[:category][:name]}:#{sri[:risk][:name]}")}
                              .join(',')
    
      get_escaped_content = ->(str){
        str = (str.size > 0 ? escape_category_tsv(str) : '') 
      }

      sri_memo = get_escaped_content.(vuln_info[:sri_memos].any? ? vuln_info[:sri_memos].first[:memo] : '')
      item << sri_memo

      description = get_escaped_content.(vuln_info[:content][:description])
      description = replace_br_tag(description)
      item << get_cell_content(description)  

      affect = get_escaped_content.(vuln_info[:content][:affect])
      affect = replace_br_tag(affect)
      item << get_cell_content(affect)

      verification = get_escaped_content.(vuln_info[:content][:verification])
      verification = replace_br_tag(verification)
      item << get_cell_content(verification)     

      solution = get_escaped_content.(vuln_info[:content][:solution])
      solution = replace_br_tag(solution)
      item << get_cell_content(solution)

      link = get_escaped_content.(vuln_info[:content][:link])
      link = replace_br_tag(link)
      item << get_cell_content(link)

      history = get_escaped_content.(vuln_info[:content][:history])   
      history = replace_br_tag(history)
      item << get_cell_content(history)

      item << (get_is_date(vuln_info[:create_date]) ? vuln_info[:create_date] : '').to_s
      item << (get_is_date(vuln_info[:update_date]) ? vuln_info[:update_date] : '').to_s
      session[:export_data][:content] << item
    rescue Exception => e
      logger.info(e)
      err_message = 'ダウンロードに失敗しました。'
    end

    render :layout => false, :json => { :err_message => err_message }
  end

  def download
    f_params = params.permit(:id, :download)
    case f_params[:download]
    when 'tsv' then
      send_summary_table_tsv_file("sidfm_content-#{f_params[:id].to_i}-tsv-#{Time.current.strftime('%Y%m%d%H%M%S')}.csv")
    end
  end

end
