# -*- coding: utf-8 -*-

require 'zlib'
require 'uri'
require 'base64'
require 'sidfm_controller_support'

# SIDfm Probe API
class SidfmProbeApiController < ApplicationController
  unloadable

  layout false

  skip_before_action :check_if_login_required
  skip_before_action :verify_authenticity_token

  private

  include SidfmControllerSupport

  before_action :authorize, :except => [ :index ]

  # 応答の雛形を返す。
  def create_response
    { 'success' => false, 'response' => {} }
  end

  # zlib展開
  def inflate(data)
    ret = nil
    z = nil
    begin
      z = Zlib::Inflate.new
      ret = z.inflate(data)
      z.finish
    ensure
      z.close
    end
    ret
  end

  # ※ Redmineの認証処理を上書きする
  # 認証処理
  # 認証成功時は、@auth=true、@apikeyにSidfmdProbeApikeyのインスタンスが設定される。
  def authorize
    params_fetched = params.fetch(:q, {})
    @api_params = params_fetched.permit(:papi_key, :data, :type, :datekey)
    header_apikey = request.headers['X-PAPI-Key']
    params_apikey = @api_params[:papi_key]
    @auth = false
    @apikey = nil
    if header_apikey then
      # X-PAPI-Keyヘッダの認証
      @apikey = SidfmdProbeApikey.auth(header_apikey)
      @auth = true if @apikey
    end
    if !@auth and params_apikey then
      # q[papi_key]パラメータでの認証
      @apikey = SidfmdProbeApikey.auth(params_apikey)
      @auth = true if @apikey
    end
    unless @auth then
      res = create_response
      res['response']['message'] = 'invalid probe apikey'
      render :json => res and return
    end
    logger.info("  Probe Apikey ID/Name: #{@apikey.id}/#{@apikey.name}")
    @type = @api_params[:type]
    if @type then
      begin
        @type = JSON.parse(@type)
        unless [ 'ssh' ].include?(@type['name']) then
          res = create_response
          res['response']['message'] = 'invalid type'
          render :json => res and return
        end
      rescue JSON::ParserError
        res = create_response
        res['response']['message'] = 'broken type'
        render :json => res and return
      end
    end
    @datekey = @api_params[:datekey]
    if @datekey then
      @datekey = @datekey.to_s
    end
    @data = @api_params[:data]
    if @data then
      begin
        @data = JSON.parse(inflate(Base64.strict_decode64(@data)))
      rescue Exception => e
        res = create_response
        res['response']['message'] = 'brocken data'
        render :json => res and return
      end
    end
  end

  public

  # 導通確認
  # GET/POST
  # 認証不要で、常に success: true を返す。
  def index
    res = create_response
    res['success'] = true
    render :json => res
  end

  # アクセスキー認証チェック
  # GET/POST
  def auth
    res = create_response
    res['success'] = true
    res['name'] = @apikey&.name
    render :json => res
  end

  # データ受信
  # POST only
  # FIXME:: under constructions
  def send_data
    unless @type then
      res = create_response
      res['response']['message'] = 'lack of type'
      render :json => res and return
    end
    unless @data then
      res = create_response
      res['response']['message'] = 'lack of data'
      render :json => res and return
    end
    ###
    logger.info("send_data begin: #{@apikey.id}")
    logger.info("type=#{@type.inspect}")
    logger.info("datekey=#{@datekey}")
    logger.info("nmap=#{@data['nmap'].inspect}")
    logger.info("ping=#{@data['ping'].inspect}")
    logger.info("ssh=#{@data['ssh'].inspect}")
    logger.info("hostinfo=#{@data['hostinfo'].inspect}")
    probe_id = nil
    begin
      SidfmdProbe.transaction do
        probe = SidfmdProbe.create(sidfmd_probe_apikey_id: @apikey.id,
                                   kind: @type['name'], version: @type['version'])
        probe_id = probe.id
        raise StandardError, 'unknwon failure: probe is nil' unless probe
        probe_ssh = SidfmdProbeSsh.create(sidfmd_probe_id: probe_id,
                                          datekey: @datekey)
        raise StandardError, 'unknown faulure: probe_ssh is nil' unless probe_ssh
        ips = @data['ping'] + @data['nmap'].collect {|x| x[0] }
        ips.uniq!
        tuple = {}
        opts = {}
        ips.each do |ip|
          tuple.clear
          opts.clear
          tuple[:is_ping] = @data['ping'].include?(ip)
          tuple[:nmap] = @data['nmap'].collect {|x| ip == x[0] ? x[1] : nil }.compact
          tuple[:ssh] = @data['ssh'].collect {|x| ip == x[0] ? { :port => x[1], :authtype => x[2], :username => x[3] } : nil }.compact
          tuple[:hostinfo] = @data['hostinfo'].find {|x| ip == x[0] }
          opts[:sidfmd_probe_ssh_id] = probe_ssh.id
          opts[:is_ping] = tuple[:is_ping]
          opts[:ipaddress] = ip
          pkgs = []
          if tuple[:hostinfo] and tuple[:hostinfo].kind_of?(Array) then
            # hostinfo exists
            opts[:hostname_f] = tuple[:hostinfo][1]
            opts[:hostname_s] = tuple[:hostinfo][2]
            dist = tuple[:hostinfo][3]
            opts[:ident] = dist['id']
            opts[:version] = dist['version']
            opts[:name] = dist['os']
            pkgs = tuple[:hostinfo][4]
          end
          probe_ssh_ip = SidfmdProbeSshIp.create(opts)
          tuple[:nmap].each do |i|
            SidfmdProbeSshIpNmap.create(sidfmd_probe_ssh_ip_id: probe_ssh_ip.id,
                                        open_port: i)
          end
          tuple[:ssh].each do |i|
            SidfmdProbeSshIpSsh.create(sidfmd_probe_ssh_ip_id: probe_ssh_ip.id,
                                       port: i[:port]&.to_i,
                                       authtype: i[:authtype]&.to_s,
                                       username: i[:username]&.to_s,)
          end
          pkgs.each do |i|
            SidfmdProbeSshIpPkg.create(sidfmd_probe_ssh_ip_id: probe_ssh_ip.id,
                                       pkg: i)
          end
        end
      end
    rescue Exception => e
      logger.error(e.message)
      e.backtrace.each do |i|
        logger.error(i)
      end
      res = create_response
      res['response']['message'] = 'unknown error'
      render :json => res and return
    end
    logger.info("send_data end: #{@apikey.id}")
    probe = SidfmdProbe.find(probe_id)
    if probe then
      probe.update_attribute :flg_enable, true
    end
    res = create_response
    res['success'] = true
    render :json => res
  end

end
