# -*- coding: utf-8 -*-

require 'date'
require 'active_record'
require 'sidfm_controller_support'

class SidfmProbeApikeysController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private

  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles

  before_action :get_vm_version
  before_action -> { get_summary_table_lines('probe_apikey') }
  def check_roles
     unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    @role_admin = @sidfm_roles.include?(:admin)
    @user_category_ids = if @role_admin then
                           nil
                         else
                           User.current.categories.order(:id).pluck(:id)
                         end

    unless @role_admin then
      if !@sidfm_roles.include?(:host_admin) then
        redirect_to home_path, :notice => '許可されていない操作です。' and return
      end
    end
  end

  def create_all_content
    @apikeys[:all_content] = SidfmdProbeApikey.joins(:sidfmd_category).select("sidfmd_probe_apikeys.*, sidfmd_categories.name as category_name")

    unless @role_admin then
        # get only belongs categories
        @apikeys[:all_content] = @apikeys[:all_content].where(
          :sidfmd_category_id => @user_category_ids
        )
    end

    @apikeys[:all_content] = @apikeys[:all_content].select(
      :id,
      :sidfmd_category_id,
      :name,
      :api_key,
      :begin_date,
      :end_date
    )
  end

  FILTER_KEYS = {}

  def init_filter_keys
    if FILTER_KEYS.size == 0 then
      FILTER_KEYS[:name] = ''
      FILTER_KEYS[:period] = '::'
      FILTER_KEYS[:category_id] = '0'
    end
  end

  def check_filters
    init_filter_keys
    FILTER_KEYS.each do |key, val|
      unless @apikeys[:filters].has_key?(key) then
        @apikeys[:filters][key] = val
      end
    end
    @default_filters = FILTER_KEYS
  end

  def filter_content
    if @apikeys[:filters][:name].size > 0 then
      @apikeys[:all_content] = @apikeys[:all_content].where(
        'sidfmd_probe_apikeys.name ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, @apikeys[:filters][:name])}%"
      )
    end

    if @apikeys[:filters][:category_id].to_i != 0 then
      @apikeys[:all_content] = @apikeys[:filters][:category_id].to_i == -1 ?
        @apikeys[:all_content].where("sidfmd_categories.id IS NULL") :
        @apikeys[:all_content].where("sidfmd_categories.id = ?", @apikeys[:filters][:category_id].to_i)
    end

    if @apikeys[:filters][:period] != '::' then
      if /\A(|notset):(|\d{4}\/\d{2}\/\d{2}):(|\d{4}\/\d{2}\/\d{2})\z/ =~ @apikeys[:filters][:period] then
        notset = ('notset' == Regexp.last_match(1))
        begindate = Regexp.last_match(2)
        enddate = Regexp.last_match(3)
        if notset then
          #無期限
          @apikeys[:all_content] = @apikeys[:all_content].where(:begin_date => nil).where(:end_date => nil)
        else
          begin
            begindate = Date.parse(begindate)
          rescue Exception => e
            begindate = nil
          end
          begin
            enddate = Date.parse(enddate)
          rescue Exception => e
            enddate = nil
          end
          if begindate and enddate then
            @apikeys[:all_content] = @apikeys[:all_content].where(%| begin_date >= ? AND end_date <= ?|, begindate,enddate)
          elsif begindate then
            @apikeys[:all_content] = @apikeys[:all_content].where(%| begin_date >= ?|, begindate)
          elsif enddate then
            @apikeys[:all_content] = @apikeys[:all_content].where(%| end_date <= ?|, enddate)
          else
            #invalid
            @apikeys[:filters][:period] = '::'
          end
        end
      end
    end
  end

  def sort_content
    order_param = @apikeys[:order].split('_')
    col = order_param[0]
    order = SidfmControllerSupport.check_order(order_param[1])

    case col
    when 'name' then
      @apikeys[:all_content] = @apikeys[:all_content].order(%|
        LOWER(sidfmd_probe_apikeys.name COLLATE "C") #{order.upcase}
      |)
    when 'category' then
      @apikeys[:all_content] = @apikeys[:all_content].order(%|
        LOWER(sidfmd_categories.name COLLATE "C") #{order.upcase}
      |)
    when 'period' then
      @apikeys[:all_content] = @apikeys[:all_content].order(:begin_date => :"#{order}")
        .order(:end_date => :"#{order}")
        .order(%|
          LOWER(sidfmd_probe_apikeys.name COLLATE "C") #{order.upcase}
        |)
    end
  end

  def parse_params
    f_params = params.fetch(:pages, {}).permit(:order, :offset, :per_page, :filters, :checked, :all_checked)
    @apikeys = {}
    @apikeys[:per_page] = (@summary_table_lines || f_params[:per_page] || 10).to_i
    @apikeys[:offset] = (f_params[:offset] || 0).to_i
    @apikeys[:order] = f_params[:order] || 'name_asc'
    @apikeys[:filters] = JSON.parse((f_params[:filters] || '{}'), symbolize_names: true)
    @apikeys[:checked] = JSON.parse(f_params[:checked] || '[]')
    @apikeys[:all_checked] = f_params[:all_checked] || false
  end

  def create_summary_apikey_html(apikey, **opts)
    data = {
      :chkbox => view_context.create_summary_probe_apikey_chkbox(apikey.id, opts[:checked]),
      :content => view_context.create_summary_probe_apikey_content(
        apikey,
        @item_names,
        @sidfm_roles,
      ),
      :operation => view_context.create_summary_probe_apikey_operation(apikey)
    }
  end

  def get_item_names
    {
      :apikey => { :name => SidfmdProbeApikey::APIKEY_LABEL },
      :name => { :name => SidfmdProbeApikey::NAME_LABEL },
      :category => { :name => SidfmdProbeApikey::CATEGORY_LABEL },
      :period => { :name => SidfmdProbeApikey::PERIOD_LABEL },
    }
  end

  public

  def index
    parse_params
    check_filters
    @page_name = 'VM 探索APIキー設定'
    @item_names = get_item_names
  end

  def create
    f_params = params.permit(:name, :category_id)
    if !f_params[:name] || f_params[:name].length == 0 then 
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    name = f_params[:name]
    category_id = f_params[:category_id].to_i
    user_id = User.current.id
    apikey = SidfmdProbeApikey.new(:name => name, :sidfmd_category_id => category_id)

    is_available_user = apikey.is_available_user?(
      :user_id => user_id,
      :user_category_ids => @user_category_ids,
      :sidfm_roles => @sidfm_roles
    )
    unless is_available_user then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    #権限がないユーザは設定不可
    if !@sidfm_roles.include?(:host_admin) then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    begin
      SidfmdProbeApikey.create_apikey(name, category_id)
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'APIキーの作成に失敗しました。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def modify
    f_params = params.permit(
      :id,
      :target,
      :name,
      :update_api_key,
      :remove_api_key,
      :category_id,
      :begin_date,
      :end_date,
      :is_unlimit_period
    )
    if !f_params[:id] || !f_params[:target] then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    begin
      apikey = SidfmdProbeApikey.find(f_params[:id].to_i)
    rescue ActiveRecord::RecordNotFound => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'APIキーが削除されています。' } and return
    end

    is_available_user = apikey.is_available_user?(
      :user_id => User.current.id,
      :user_category_ids => @user_category_ids,
      :sidfm_roles => @sidfm_roles
    )
    unless is_available_user then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    begin
      SidfmdProbeApikey.transaction do
        case f_params[:target]
        when 'api_key' then
          if !f_params[:update_api_key] and !f_params[:remove_api_key] then
            render :layout => false, :json => { :err_message => '不正な操作です。' } and return
          end
          if f_params[:update_api_key] then
            apikey.lock!.generate_apikey
          else
            apikey.lock!.remove_apikey
          end
        when 'name' then
          if !f_params[:name] || f_params[:name].length == 0 then
            render :layout => false, :json => { :err_message => '不正な操作です。' } and return
          end
          apikey.lock!.update_name f_params[:name] 
        when 'period' then
          if f_params[:is_unlimit_period].to_s == 'true' then
            apikey.lock!.update_period nil, nil
          else
            if !f_params[:begin_date] or !f_params[:end_date] then
              render :layout => false, :json => { :err_message => '不正な操作です。' } and return
            end
            begin_date = f_params[:begin_date].size > 0 ? Date.parse(f_params[:begin_date]) : nil
            end_date = f_params[:end_date].size > 0 ? Date.parse(f_params[:end_date]) : nil
            if begin_date.is_a?(Date) and end_date.is_a?(Date) and begin_date > end_date then
              render :layout => false, :json => { :err_message => '不正な操作です。' } and return
            end
            apikey.lock!.update_period begin_date, end_date
          end
        when 'category' then
          if !f_params[:category_id] then
            render :layout => false, :json => { :err_message => '不正な操作です。' } and return
          end

          if @role_admin then
            is_pass = true
          else
            if @user_category_ids.include?(f_params[:category_id].to_i) then
              is_pass = true
            else
              is_pass = false
            end
          end

          if is_pass then
            apikey.lock!.update_category f_params[:category_id].to_i
          else
            render :layout => false, :json => { :err_message => '変更に失敗しました。' } and return
          end
        else
          logger.error('invalid data')
          render :layout => false, :json => { :err_message => '変更に失敗しました。' } and return
        end
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '変更に失敗しました。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def delete
    f_params = params.permit(:id)
    id = f_params[:id].to_i
    begin
      apikey = SidfmdProbeApikey.find_by_id(id)
      if apikey then
        is_available_user = apikey.is_available_user?(
          :user_id => User.current.id,
          :user_category_ids => @user_category_ids,
          :sidfm_roles => @sidfm_roles
        ) 
        unless is_available_user then
          render :layout => false, :json => { :err_message => '不正な操作です。' } and return
        end
        SidfmdProbeApikey.delete_apikey(id)
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'APIキーの削除に失敗しました。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def bulk_delete
    f_params = params.permit(:ids)
    begin
      ids = JSON.parse(f_params[:ids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    begin
      is_pass = true
      ids.each do |id|
        apikey = SidfmdProbeApikey.find_by_id id
        if apikey then
          is_available_user = apikey.is_available_user?(
            :user_id => User.current.id,
            :user_category_ids => @user_category_ids,
            :sidfm_roles => @sidfm_roles
          )
          unless is_available_user then
            is_pass = false
            break
          end
        end
      end
      if is_pass then
        SidfmdProbeApikey.bulk_delete_apikey ids
      else
        render :layout => false, :json => { :err_message => '不正な操作です。' } and return
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '一括削除に失敗しました。' } and return
    end

    render :layout => false, :json => { :err_message => '' }
  end

  def api_get_display_summary_content
    GC.start
    f_params = params.fetch(:pages, {}).permit(:per_page)
    parse_params
    create_all_content
    @apikeys[:per_page] = f_params[:per_page].to_i
    sort_content
    filter_content
    @apikeys[:content_filtered] = @apikeys[:all_content]
    @apikeys[:content] = @apikeys[:content_filtered].offset(@apikeys[:offset]).limit(@apikeys[:per_page])

    @item_names = get_item_names.map{|k, v| [k, v[:name]]}.to_h
    all_ids = @apikeys[:content_filtered].pluck(:id)
    ret = {
      :rows => @apikeys[:content].map{|apikey|
        create_summary_apikey_html(
          apikey,
          { :checked =>  @apikeys[:checked] }
        )
      },
      :total => all_ids.size,
      :all_ids => all_ids,
      :sidfm_roles => @sidfm_roles,
      :user_cats => @user_category_ids
    }
    render :layout => false, :json => ret
  end

  #apikey名を返す
  def api_get_apikey_names
    f_params = params.permit(:ids)
    ids = JSON.parse(f_params[:ids])
    @apikeys = {}
    create_all_content
    names = @apikeys[:all_content].where(:id => ids).order('LOWER(sidfmd_probe_apikeys.name COLLATE "C")').pluck(:name)
    render :layout => false, :json => names
  end


end
