# -*- coding: utf-8 -*-

require 'date'
require 'active_record'
require 'sidfm_controller_support'

class SidfmVmApikeysController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private

  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles

  before_action :get_vm_version
  before_action -> { get_summary_table_lines('vm_apikey') }
  def check_roles
     unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    @role_admin = @sidfm_roles.include?(:admin)
    @user_category_ids = if @role_admin then
                           nil
                         else
                           User.current.categories.order(:id).pluck(:id)
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host)
                                           .where(:sidfmd_category_id => @user_category_ids)
                                           .where(:sidfmd_hosts => { :flg_delete => false })
                                           .select(:sidfmd_host_id)
                                           .order(:sidfmd_host_id)
                                           .distinct
                                           .pluck(:sidfmd_host_id)
                     end
  end

  def create_all_content
    @apikeys[:all_content] = SidfmdVmApikey.joins(:user).where.not(:users => { :type => 'AnonymousUser' })
    unless @role_admin then
      if @sidfm_roles.include?(:host_admin) then
        @apikeys[:all_content] = @apikeys[:all_content].where(
          :users => {
            :id => User.get_category_users(@user_category_ids).select(:id)
          }
        )
      else
        @apikeys[:all_content] = @apikeys[:all_content].where(
          :users => { :id => User.current.id }
        )
      end
    end
    @apikeys[:all_content] = @apikeys[:all_content].select(
      :id,
      :user_id,
      :name,
      :api_key,
      :begin_date,
      :end_date,
      :allow_update,
      'users.id AS user_id',
      'users.firstname AS user_firstname',
      'users.lastname AS user_lastname',
      'users.login'
    )
  end

  FILTER_KEYS = {}

  def init_filter_keys
    if FILTER_KEYS.size == 0 then
      FILTER_KEYS[:name] = ''
      FILTER_KEYS[:period] = '::'
      FILTER_KEYS[:allow_update] = ''
      FILTER_KEYS[:username] = ''
    end
  end

  def check_filters
    init_filter_keys
    FILTER_KEYS.each do |key, val|
      unless @apikeys[:filters].has_key?(key) then
        @apikeys[:filters][key] = val
      end
    end
    @default_filters = FILTER_KEYS
  end

  def filter_content
    if @apikeys[:filters][:name].size > 0 then
      @apikeys[:all_content] = @apikeys[:all_content].where(
        'name ILIKE ?', "%#{ActiveRecord::Base.send(:sanitize_sql_like, @apikeys[:filters][:name])}%"
      )
    end
    if @apikeys[:filters][:username].size > 0 then
      str = "%#{ActiveRecord::Base.send(:sanitize_sql_like, @apikeys[:filters][:username])}%"
      @apikeys[:all_content] = @apikeys[:all_content].where("users.login ILIKE ? OR (users.lastname || ' ' || users.firstname) ILIKE ?", str, str)
    end
    if @apikeys[:filters][:allow_update].size > 0 then
       @apikeys[:all_content] = @apikeys[:all_content].where(:allow_update => @apikeys[:filters][:allow_update])
    end
    if @apikeys[:filters][:period] != '::' then
      if /\A(|notset):(|\d{4}\/\d{2}\/\d{2}):(|\d{4}\/\d{2}\/\d{2})\z/ =~ @apikeys[:filters][:period] then
        notset = ('notset' == Regexp.last_match(1))
        begindate = Regexp.last_match(2)
        enddate = Regexp.last_match(3)
        if notset then
          #無期限
          @apikeys[:all_content] = @apikeys[:all_content].where(:begin_date => nil).where(:end_date => nil)
        else
          begin
            begindate = Date.parse(begindate)
          rescue Exception => e
            begindate = nil
          end
          begin
            enddate = Date.parse(enddate)
          rescue Exception => e
            enddate = nil
          end
          if begindate and enddate then
            @apikeys[:all_content] = @apikeys[:all_content].where(%| begin_date >= ? AND end_date <= ?|, begindate,enddate)
          elsif begindate then
            @apikeys[:all_content] = @apikeys[:all_content].where(%| begin_date >= ?|, begindate)
          elsif enddate then
            @apikeys[:all_content] = @apikeys[:all_content].where(%| end_date <= ?|, enddate)
          else
            #invalid
            @apikeys[:filters][:period] = '::'
          end
        end
      end
    end
  end

  def sort_content
    order_param = @apikeys[:order].split('_')
    col = order_param[0]
    order = SidfmControllerSupport.check_order(order_param[1])

    case col
    when 'name' then
      @apikeys[:all_content] = @apikeys[:all_content].order(%|
        LOWER(sidfmd_vm_apikeys.name COLLATE "C") #{order.upcase}
      |)
    when 'user' then
      @apikeys[:all_content] = @apikeys[:all_content].order(%|
        LOWER(users.login COLLATE "C") #{order.upcase}
      |)
      .order(%|
        LOWER(sidfmd_vm_apikeys.name COLLATE "C") #{order.upcase}
      |)
    when 'allow-update'
      @apikeys[:all_content] = @apikeys[:all_content].order(:allow_update => (order == 'asc' ? :desc : :asc))
        .order(%|
          LOWER(sidfmd_vm_apikeys.name COLLATE "C") #{order.upcase}
        |)
    when 'period' then
      @apikeys[:all_content] = @apikeys[:all_content].order(:begin_date => :"#{order}")
        .order(:end_date => :"#{order}")
        .order(%|
          LOWER(sidfmd_vm_apikeys.name COLLATE "C") #{order.upcase}
        |)
    end
  end

  def parse_params
    f_params = params.fetch(:pages, {}).permit(:order, :offset, :per_page, :filters, :checked, :all_checked)
    @apikeys = {}
    @apikeys[:per_page] = (@summary_table_lines || f_params[:per_page] || 10).to_i
    @apikeys[:offset] = (f_params[:offset] || 0).to_i
    @apikeys[:order] = f_params[:order] || 'name_asc'
    @apikeys[:filters] = JSON.parse((f_params[:filters] || '{}'), symbolize_names: true)
    @apikeys[:checked] = JSON.parse(f_params[:checked] || '[]')
    @apikeys[:all_checked] = f_params[:all_checked] || false
  end

  def create_summary_apikey_html(apikey, **opts)
    data = {
      :chkbox => view_context.create_summary_vm_apikey_chkbox(apikey.id, opts[:checked]),
      :content => view_context.create_summary_vm_apikey_content(
        apikey,
        @item_names,
        @sidfm_roles,
      ),
      :operation => view_context.create_summary_vm_apikey_operation(apikey)
    }
  end

  def get_item_names
    {
      :apikey => { :name => SidfmdVmApikey::APIKEY_LABEL },
      :name => { :name => SidfmdVmApikey::NAME_LABEL },
      :user => { :name => SidfmdVmApikey::USER_LABEL },
      :allow_update => { :name => SidfmdVmApikey::ALLOW_UPDATE_LABEL },
      :period => { :name => SidfmdVmApikey::PERIOD_LABEL },
    }
  end

  public

  def index
    parse_params
    check_filters
    @page_name = 'VM APIキー設定'
    @item_names = get_item_names
  end

  def create
    f_params = params.permit(:name, :user_id)
    if !f_params[:name] || !f_params[:user_id] || f_params[:name].length == 0 || f_params[:user_id].length == 0 then 
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    name = f_params[:name]
    user_id = f_params[:user_id].to_i
    apikey = SidfmdVmApikey.new(:user_id => user_id)
    is_available_user = apikey.is_available_user?(
      :user_id => User.current.id,
      :user_category_ids => @user_category_ids,
      :sidfm_roles => @sidfm_roles
    )
    unless is_available_user then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    get_sidfm_project
    #権限がないユーザは設定不可
    unless Member.where(:project_id => @sidfm_project.id).where(:user_id => user_id).exists? then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    begin
      SidfmdVmApikey.create_apikey(name, user_id)
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'APIキーの作成に失敗しました。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def modify
    f_params = params.permit(
      :id,
      :target,
      :name,
      :update_api_key,
      :remove_api_key,
      :allow_update,
      :user_id,
      :begin_date,
      :end_date,
      :is_unlimit_period
    )
    if !f_params[:id] || !f_params[:target] then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    begin
      apikey = SidfmdVmApikey.find(f_params[:id].to_i)
    rescue ActiveRecord::RecordNotFound => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'APIキーが削除されています。' } and return
    end

    is_available_user = apikey.is_available_user?(
      :user_id => User.current.id,
      :user_category_ids => @user_category_ids,
      :sidfm_roles => @sidfm_roles
    )
    unless is_available_user then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    begin
      SidfmdVmApikey.transaction do
        case f_params[:target]
        when 'api_key' then
          if !f_params[:update_api_key] and !f_params[:remove_api_key] then
            render :layout => false, :json => { :err_message => '不正な操作です。' } and return
          end
          if f_params[:update_api_key] then
            apikey.lock!.generate_apikey
          else
            apikey.lock!.remove_apikey
          end
        when 'name' then
          if !f_params[:name] || f_params[:name].length == 0 then
            render :layout => false, :json => { :err_message => '不正な操作です。' } and return
          end
          apikey.lock!.update_name f_params[:name] 
        when 'allow_update'
          if !f_params[:allow_update] then
            render :layout => false, :json => { :err_message => '不正な操作です。' } and return
          end
          value = f_params[:allow_update].to_s == 'true'
          if value && !apikey.is_allowed_allow_update_true? then
            render :layout => false, :json => { :err_message => '変更に失敗しました。' } and return
          else
            apikey.lock!.update_allow_update value
          end
        when 'period' then
          if f_params[:is_unlimit_period].to_s == 'true' then
            apikey.lock!.update_period nil, nil
          else
            if !f_params[:begin_date] or !f_params[:end_date] then
              render :layout => false, :json => { :err_message => '不正な操作です。' } and return
            end
            begin_date = f_params[:begin_date].size > 0 ? Date.parse(f_params[:begin_date]) : nil
            end_date = f_params[:end_date].size > 0 ? Date.parse(f_params[:end_date]) : nil
            if begin_date.is_a?(Date) and end_date.is_a?(Date) and begin_date > end_date then
              render :layout => false, :json => { :err_message => '不正な操作です。' } and return
            end
            apikey.lock!.update_period begin_date, end_date
          end
        when 'user' then
          if !f_params[:user_id] then
            render :layout => false, :json => { :err_message => '不正な操作です。' } and return
          end
          is_pass = true
          if apikey.allow_update then
            is_pass = apikey.is_allowed_allow_update_true?(f_params[:user_id].to_i)
          end
          if is_pass then
            get_sidfm_project
            #権限がないユーザは設定不可
            is_pass = Member.where(:project_id => @sidfm_project.id).where(:user_id => f_params[:user_id].to_i).exists?
          end
          if is_pass then
            apikey.lock!.update_user f_params[:user_id].to_i
          else
            render :layout => false, :json => { :err_message => '変更に失敗しました。' } and return
          end
        else
          logger.error('invalid data')
          render :layout => false, :json => { :err_message => '変更に失敗しました。' } and return
        end
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '変更に失敗しました。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def delete
    f_params = params.permit(:id)
    id = f_params[:id].to_i
    begin
      apikey = SidfmdVmApikey.find_by_id(id)
      if apikey then
        is_available_user = apikey.is_available_user?(
          :user_id => User.current.id,
          :user_category_ids => @user_category_ids,
          :sidfm_roles => @sidfm_roles
        ) 
        unless is_available_user then
          render :layout => false, :json => { :err_message => '不正な操作です。' } and return
        end
        SidfmdVmApikey.delete_apikey(id)
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'APIキーの削除に失敗しました。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  def bulk_delete
    f_params = params.permit(:ids)
    begin
      ids = JSON.parse(f_params[:ids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end
    begin
      is_pass = true
      ids.each do |id|
        apikey = SidfmdVmApikey.find_by_id id
        if apikey then
          is_available_user = apikey.is_available_user?(
            :user_id => User.current.id,
            :user_category_ids => @user_category_ids,
            :sidfm_roles => @sidfm_roles
          )
          unless is_available_user then
            is_pass = false
            break
          end
        end
      end
      if is_pass then
        SidfmdVmApikey.bulk_delete_apikey ids
      else
        render :layout => false, :json => { :err_message => '不正な操作です。' } and return
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '一括削除に失敗しました。' } and return
    end

    render :layout => false, :json => { :err_message => '' }
  end

  def api_get_display_summary_content
    GC.start
    f_params = params.fetch(:pages, {}).permit(:per_page)
    parse_params
    create_all_content
    @apikeys[:per_page] = f_params[:per_page].to_i
    sort_content
    filter_content
    @apikeys[:content_filtered] = @apikeys[:all_content]
    @apikeys[:content] = @apikeys[:content_filtered].offset(@apikeys[:offset]).limit(@apikeys[:per_page])

    @item_names = get_item_names.map{|k, v| [k, v[:name]]}.to_h
    all_ids = @apikeys[:content_filtered].pluck(:id)
    ret = {
      :rows => @apikeys[:content].map{|apikey|
        create_summary_apikey_html(
          apikey,
          { :checked =>  @apikeys[:checked] }
        )
      },
      :total => all_ids.size,
      :all_ids => all_ids,
      :sidfm_roles => @sidfm_roles,
      :user_cats => @user_category_ids
    }
    render :layout => false, :json => ret
  end

  #apikey名を返す
  def api_get_apikey_names
    f_params = params.permit(:ids)
    ids = JSON.parse(f_params[:ids])
    @apikeys = {}
    create_all_content
    names = @apikeys[:all_content].where(:id => ids).order('LOWER(sidfmd_vm_apikeys.name COLLATE "C")').pluck(:name)
    render :layout => false, :json => names
  end

  #更新許可を「許可」にできるかどうかを返す
  def api_is_apikey_allowed_allow_update_true
    f_params = params.permit(:id, :after_change_user_id)
    ret = false
    apikey = SidfmdVmApikey.find_by(:id => f_params[:id])
    if apikey then
      if f_params[:after_change_user_id] then
        if apikey.allow_update then
          ret = apikey.is_allowed_allow_update_true? f_params[:after_change_user_id]
        else
          ret = true
        end
      else
        ret = apikey.is_allowed_allow_update_true?
      end
    end
    render :layout => false, :json => ret
  end

end
