# -*- coding: utf-8 -*-

require 'softek_utils'

class SidfmdRegkey < ActiveRecord::Base
  has_many :sidfmd_regkey_cats, dependent: :destroy

  NAME_LABEL = '登録キー名'
  APIKEY_LABEL = 'APIキー'
  HOSTNUM_LIMIT_LABEL = '制限ホスト数'
  HOSTNUM_PREFIX_LABEL = 'ホスト名接頭辞'
  HOSTNUM_SUFFIX_LABEL = 'ホスト名接尾辞'
  HOSTNUM_DUP_ACTION_LABEL = '同ホスト名が登録済の場合'
  IS_DN_ORIG_LABEL = '表示名を設定'
  CATEGORIES_LABEL = 'カテゴリ'
  PERIOD_LABEL = '有効期間'

  DUP_ACTION_REJECT_LABEL = '無条件で登録拒否'
  DUP_ACTION_USE_LABEL = '既に登録済と判定し、登録済ホストの情報を返す'
  DUP_ACTION_NUM_SUFFIX_LABEL = '数値を付与し、重複しないホスト名を登録'

  # api_keyを生成する。
  # 生成したapi_keyを返す。
  def generate_apikey
    # 重複回避
    salt = Sidfm.salt
    flg = true
    api_key = nil
    while flg do
      api_key = SofTekUtils.hexdigest(post: salt)
      unless self.class.find_by_api_key(api_key) then
        self.update_attribute :api_key, api_key
        flg = false
      end
    end
    api_key
  end

  # 登録キーを無効にする
  def remove_apikey
    self.update_attribute :api_key, nil
  end

  # システムカテゴリを返す。
  def system_category
    cat = self.
      sidfmd_regkey_cats.
      joins(:sidfmd_category).
      where(:sidfmd_categories => { :is_system => true }).
      order(:id).first
    cat ? cat.sidfmd_category : nil
  end

  # カスタムカテゴリを返す。
  def custom_categories
    self.
      sidfmd_regkey_cats.
      joins(:sidfmd_category).
      where(:sidfmd_categories => { :is_system => false }).
      order('sidfmd_categories.name').
      map(&:sidfmd_category)
  end

  # 有効期限チェック
  def expired?
    ret = false
    today = Date.today
    if self.begin_date.nil? and self.end_date.nil? then
      ret = true
    elsif self.begin_date.nil? then
      ret = (today <= self.end_date)
    elsif self.end_date.nil? then
      ret = (today >= self.begin_date)
    else
      ret = (today >= self.begin_date and today <= self.end_date)
    end
    ret
  end

  # APIキーの登録
  def self.register_host_apikey(name)
    apikey = self.new(:name => name)
    #apikey.api_key = apikey.generate_apikey
    apikey.save
  end

  # APIキーの削除
  def self.delete_host_apikey(id)
    self.transaction do
      apikey = self.lock.find_by_id(id.to_i)
      apikey.destroy if apikey 
    end
  end

  # APIキーの一括削除
  def self.bulk_delete_host_apikey(ids)
    self.transaction do
      self.where(:id => ids).destroy_all
    end
  end

  # キー名の更新
  def update_name(name)
    self.update_attribute :name, name
  end

  # ホスト数制限の更新
  def update_hostnum_limit(num)
    self.update_attribute :hostnum_limit, num
  end

  # ホスト名prefixの更新
  def update_hostname_prefix(prefix)
    self.update_attribute :hostname_prefix, prefix
  end

  # ホスト名suffixの更新
  def update_hostname_suffix(suffix)
    self.update_attribute :hostname_suffix, suffix
  end

  # dup_actionの更新
  def update_dup_action(action)
    action = action.to_s
    if ![ 'reject', 'use', 'num_suffix' ].include?(action) then
      logger.error('invalid action')
      raise Exception
    end
    self.update_attribute :dup_action, action
  end

  # is_dn_orig更新
  def update_is_dn_orig(is_dn_orig)
    self.update_attribute :is_dn_orig, is_dn_orig
  end

  # 有効期間の更新
  def update_period(begin_date, end_date)
    self.update_attributes :begin_date =>  begin_date, :end_date => end_date
  end

  # カテゴリの更新
  def update_categories(category_ids)
    self.sidfmd_regkey_cats = []
    self.sidfmd_regkey_cats = category_ids.map{|cat_id| 
        SidfmdRegkeyCat.new(:sidfmd_category_id => cat_id)
    }
    self.save
  end

  # 日付を文字列に変換
  # UI等で、日付をここの表示方法で統一する
  def self.date_to_string(date)
    date.strftime('%Y/%m/%d')
  end

end
