# -*- coding: utf-8 -*-

# Copyright (C) 2020 SofTek Systems, Inc. All rights reserved.

require 'pp'
require 'fileutils'

require 'vm_rule_trigger'
require 'vm_rule_cond'
require 'vm_rule_action'

module SIDfm
  # ルールに関する処理
  module VMRule

    DEBUG = FileTest.exist?('/flag.debug')

    LOGDIR = '/var/lib/redmine/log'
    LOG_INFO = 'vm_rule-info.log'
    LOG_ERROR = 'vm_rule-error.log'
    LOG_DEBUG = 'vm_rule-debug.log'
    LOG_SIZE = 10 * 1024 * 1024

    def self.logging(file, *s)
      if FileTest.exist?(file) and File.size(file) > LOG_SIZE then
        8.times do |i|
          file_n = file + ".#{8 - i}"
          file_old = file + ".#{8 - i + 1}"
          if FileTest.exist?(file_n) then
            FileUtils.mv(file_n, file_old, :force => true)
          end
        end
        if FileTest.exist?(file) then
          FileUtils.mv(file, file + ".0", :force => true)
        end
      end
      File.open(file, 'a') do |f|
        s.each do |i|
          case
          when i.kind_of?(Exception) then
            f.puts(i.message)
            i.backtrace.each do |j|
              f.puts(j)
            end
          when i.kind_of?(String) then
            f.puts(i)
          else
            f.puts(i.inspect)
          end
        end
      end
    end

    def self.log_info(*s)
      logging("#{LOGDIR}/#{LOG_INFO}", *s)
    end

    def self.log_error(*s)
      logging("#{LOGDIR}/#{LOG_ERROR}", *s)
    end

    def self.log_debug(*s)
      logging("#{LOGDIR}/#{LOG_DEBUG}", *s) if DEBUG
    end

    # 最初のアクセスの際に、チェック範囲とする期間
    # 実行時の1日(24時間)前から実行時までのデータを対象とする。
    # FIXME: この設定については利用しない。
    FIRST_LIMIT_DAYS = 1

    ### args について
    # argsは、ルールの処理中にtrigger, cond, actionで共有されるデータを保持する。
    # VMシステムから与えられるデータはtriggerの種類（periodicとvulsync）で大きく値が異なる。

    # :vulsync(脆弱性同期時)
    # 脆弱性の同期タイミングで実行される。
    # :content_add, :content_modify, :content_remove にそれぞれ追加、更新、削除さ
    # れたコンテンツの情報が保持される。
    # cond処理後は、:filtered_* (filtered_content_add等) が作成され、condの条件に
    # 従ったデータのみをフィルタした結果が保存される。

    # :periodic(定期実行)
    # 指定したタイミングで定期的に実行される。
    # :hosts_content, :vuls_content にホスト一覧、脆弱性一覧に相当する情報が入り、
    # cond処理後は、:filtered_* (filtered_hosts_content等)が作成され、condの条件に
    # 従ったデータのみをフィルタした結果が保存される。

    # :trigger(トリガーの種類)
    # triggerの処理後、トリガーの種類(:periodic, :vulsync)が保存される。
    # :invalidなら不正なトリガー種類。nilはトリガー未処理。

    # 実行すべきルールIDの一覧を返す。
    def self.invoke_rule_ids(trigger_type, args  = {})
      log_info("invoke_rule_ids: ", trigger_type, args.keys)
      trigger = Trigger.new(args)
      trigger.run(trigger_type)
    end

    # 指定したルールIDの処理を実行する。
    def self.invoke_rule_run(rule_ids = [], args = {})
      log_info("invoke_rule_run: ", rule_ids, args.keys)
      # Triggerに適合するルールが存在
      rule_ids.each do |rule_id|
        SidfmdRule.transaction do
          rule = SidfmdRule.lock.find(rule_id)
          log_info("rule:", rule)
          trigger = rule.sidfmd_rule_trigger
          unless trigger then
            log_error("no trigger: skip")
            next
          end
          trigger.log_info("ルール処理開始")
          rule_args = args.deep_dup
          rule_args[:category_id] = rule.sidfmd_category_id
          if rule_args[:category_id] then
            begin
              cat = SidfmdCategory.find(rule_args[:category_id])
              rule_args[:host_ids] = cat.sidfmd_hosts.where(:flg_delete => false).select(:id).distinct.collect {|x| x[:id] }
            rescue Exception => e
              log_error(e)
              rule_args[:host_ids] = []
            end
          else
            rule_args[:host_ids] = nil
          end
          if rule_args[:trigger] == :vulsync then
            log_debug("vulsync limit begin:", rule_args[:host_ids])
            detail = rule.sidfmd_rule_trigger.detail
            log_debug(detail)
            t_category_id = detail.sidfmd_category_id
            t_host_id = detail.sidfmd_host_id
            log_debug("trigger.category_id:", t_category_id)
            log_debug("trigger.host_id:", t_host_id)
            if t_host_id then
              if rule_args[:host_ids].nil? or rule_args[:host_ids].include?(t_host_id) then
                rule_args[:host_ids] = [ t_host_id ]
              else
                rule_args[:host_ids] = []
              end
            elsif t_category_id and t_category_id > 0 then
              t_host_ids = []
              begin
                cat = SidfmdCategory.find(t_category_id)
                t_host_ids = cat.sidfmd_hosts.where(:flg_delete => false).select(:id).distinct.collect {|x| x[:id] }
              rescue Exception => e
                log_error(e)
                t_host_ids = []
              end
              if rule_args[:host_ids] then
                rule_args[:host_ids] &= t_host_ids
              else
                rule_args[:host_ids] = t_host_ids
              end
            end
            log_debug("vulsync limit end:", rule_args[:host_ids])
            log_debug("sync_hostids(before):", rule_args[:sync_hostids])
            rule_args[:sync_hostids] &= rule_args[:host_ids] if rule_args[:host_ids]
            log_debug("sync_hostids(after):", rule_args[:sync_hostids])
          end
          cond = Cond.new(rule, rule_args)
          # condチェックはここでは行わず、actionでproc_emptyにて制御する形に変更
          cond.check?
          action = Action.new(rule, rule_args)
          action.run
          trigger.update_attribute :last_checked_at, Time.current
          trigger.log_info("ルール処理終了")
        end
      end
    end

    # ルールの呼び出し
    # ルールに関する処理はこのメソッド（及びinvoke_rule_*）のみから呼び出される。
    # 呼び出し元の情報はargs経由で渡されるのみで、他からダイレクトに取得すること
    # は推奨されない。
    def self.invoke_rule(trigger_type, args = {})
      rule_ids = invoke_rule_ids(trigger_type, args)
      if rule_ids.size > 0 then
        invoke_rule_run(rule_ids, args)
      end
    end

    PREV_CONTENT_KEYS = [ :prev_vuls_content ].freeze
    HOSTS_COTENT_KEYS = [ :hosts_content ].freeze
    VULS_CONTENT_KEYS = [ :vuls_content ].freeze
    VULSYNC_CONTENT_KEYS = [ :content_add, :content_modify, :content_remove ].freeze

    P_FIL = 'filtered_'.freeze

  end
end

if $0 == __FILE__ then

  def catch_exception(&block)
    begin
      block.call
    rescue Exception => e
      STDERR.puts e.message
      e.backtrace.each do |message|
        STDERR.puts message
      end
    end
  end

  corresp_required = Sidfm.corresp_required

  hosts_content = nil
  vuls_content = nil

  catch_exception do
    STDERR.puts("periodic ***")
    STDERR.puts(" getting hosts_content")
    hosts_content = SidfmdHost.create_all_content(no_cache: true, category_id: 0, role_admin: true, corresp_required: corresp_required)
    STDERR.puts(" done: #{hosts_content.size}")
    STDERR.puts(" getting vuls_content")
    vuls_content = SidfmdIssue.create_all_content(no_cache: true, category_id: 0, role_admin: true, corresp_required: corresp_required)
    STDERR.puts(" done: #{vuls_content.size}")
    SIDfm::VMRule.invoke_rule(:periodic, { :hosts_content => hosts_content, :vuls_content => vuls_content, :prev_vuls_content => vuls_content, })
  end

  catch_exception do
    STDERR.puts('vulsync ***')
    STDERR.puts(" getting contents")
    content_add = SidfmdIssue.create_all_content(no_cache: true, category_id: 0, role_admin: true, corresp_required: corresp_required, html_ids: [ 11287, 15917, 19271 ])
    STDERR.puts(" add: #{content_add.size}")
    content_modify = SidfmdIssue.create_all_content(no_cache: true, category_id: 0, role_admin: true, corresp_required: corresp_required, html_ids: [ 15766, 15602, 15918, 26981 ])
    STDERR.puts(" modify: #{content_modify.size}")
    content_remove = SidfmdIssue.create_all_content(no_cache: true, category_id: 0, role_admin: true, corresp_required: corresp_required, html_ids: [ 15602, 16029, 19285 ])
    STDERR.puts(" remove: #{content_remove.size}")
    SIDfm::VMRule.invoke_rule(:vulsync, { :content_add => content_add, :content_modify => content_modify, :content_remove => content_remove, :hosts_content => hosts_content, :vuls_content => vuls_content, :prev_vuls_content => vuls_content, })
  end

end
