# -*- coding: utf-8 -*-

require 'date'
require 'csv'
require 'tempfile'
require 'zip'
require 'active_record'
require 'uri'

require 'sidfmdlm_api'
require 'sidfm_controller_support'
require 'sidfm_licenses_controller_support'

class SidfmLicensesController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  # before_action :api_init, :except => [ :license_setting, :license_update ]

  private

  include SidfmControllerSupport, SidfmLicensesControllerSupport

  before_action :update_session_updated_at
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license, :except => [ :license, :license_setting, :license_update ]
  before_action :check_roles

  before_action :get_vm_version
  before_action :clear_import_session, :except => [ :index, :index_r, :import, :new_import, :progress_import_opts, :progress_import_hosts ]  
  before_action :clear_export_session, :except => [ :new_export ]
  before_action -> { get_summary_table_lines('hostadmin') } 

  # 権限チェック
  def check_roles
    unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    @role_admin = @sidfm_roles.include?(:admin)
    unless @sidfm_roles.include?(:view) then
      redirect_to user_path(User.current), :notice => af_log_user('アクセス権限がありません。') and return
    end
    # SIDfmVM管理者、SIDfmVM報告者、SIDfmVM利用者権限でアクセス可能な範囲
    if [ 'set_display_name', 'update_memo', 'update_memo_bulk', 'update_applications', 'update_applications_bulk', 'set_last_checked_at', 'modify_pkgs' ].include?(action_name) and (!@sidfm_roles.include?(:host_admin) and !@sidfm_roles.include?(:host_limited_admin)) then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    # SIDfmVM管理者、SIDfmVM報告者でアクセス可能な範囲
    if [ 'register_host', 'toggle_host_delete', 'rename_host',
         'update_refdate', 'update_refdate_bulk', 
         'update_categories', 'update_categories_bulk',
         'load_offline_data', 'import', 'new_import',
         'new_export', 'progress_import_opts', 'progress_import_hosts',
         'new_import_apply' 
       ].include?(action_name) and !@sidfm_roles.include?(:host_admin)
    then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    if [ 'license', 'license_setting', 'license_update', 'license_apikey_update', 'host_apikey_update', 'register_apikey_update', 'register_apikey_remove', 'import_apply', 'export' ].include?(action_name) and !@role_admin then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    @user_category_ids = User.current.categories.order(:id).pluck(:id)
    @user_host_ids = SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => @user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
    @cache_prefix = @role_admin ? '/license/admin/0' : "/license/user/#{@user_category_ids.join(",")}"
  end
  
  #ホスト名チェック
  def check_hostname(hostname)
    if /\A[0-9a-zA-Z._-]+\z/ !~ hostname then
      return 'ホスト名に使用できない文字が含まれています。'
    end
    if hostname.bytesize > SidfmdHost::MAX_HOSTNAME_BYTE_LENGTH then
      return SidfmdHost::MAX_HOSTNAME_BYTE_LENGTH.to_s + 'バイトを超えるホスト名は登録できません。'
    end
    return true
  end

  #ホスト付属情報の、SidfmdQueueに紐づくデータの登録
  def regist_host_queue_opts(host_id, package_names, app_names, dist_name, win_app_names)
    begin
      SidfmdQueue.transaction do
        last_queue = SidfmdQueue.where(:sidfmd_host_id => host_id).order(:created_at => :desc).first
        queue = SidfmdQueue.create(sidfmd_host_id: host_id)
        if package_names then
          regist_pkgs = package_names.map{|pkg|
            SidfmdQueuePkg.new(sidfmd_queue_id: queue.id, pkg: pkg)
          }
          SidfmdQueuePkg.import(regist_pkgs) if regist_pkgs.size > 0
        end
        if app_names then
          regist_apps = []
          app_names.each do |app_name|
            app = app_name.split(':')
            if app.size == 3 then #"#{SidfmdQueueApp#name}:#{SidfmdQueueApp#version}:#{SidfmdQueueApp#path}"
              regist_apps << SidfmdQueueApp.new(
                sidfmd_queue_id: queue.id, 
                name: app[0],
                version: app[1],
                path: app[2]
              )
            end  
          end
          SidfmdQueueApp.import(regist_apps) if regist_apps.size > 0
        end
        if win_app_names && win_app_names.size > 0 then
          if last_queue then
            queue_win = last_queue.sidfmd_queue_win
            if queue_win then
              SidfmdQueueWin.lock
              new_queue_win = SidfmdQueueWin.create(
                :id => (SidfmdQueueWin.maximum(:id) || 0) + 1,
                sidfmd_queue_id: queue.id,
                last_date: queue_win.last_date,
                os: queue_win.os,
                osname: queue_win.osname
              )
              SidfmdQueueWinApp.lock
              is_win_app_created = false
              win_app_names.each do |win_app_name|
                win_app = win_app_name.split(':', -1)
                if win_app.size == 4 then
                  win_app_publisher = decode_win_app(win_app[0])
                  win_app_name = decode_win_app(win_app[1])
                  win_app_version = decode_win_app(win_app[2])
                  win_app_uninstall = decode_win_app(win_app[3])
                  SidfmdQueueWinApp.create(
                    :id => (SidfmdQueueWinApp.maximum(:id) || 0) + 1,
                    sidfmd_queue_win_id: new_queue_win.id,
                    publisher: (win_app_publisher.size > 0 ? win_app_publisher : nil),
                    name: (win_app_name.size > 0 ? win_app_name : nil),
                    version: (win_app_version.size > 0 ? win_app_version : nil),
                    uninstall: (win_app_uninstall.size > 0 ? win_app_uninstall : nil)
                  )
                  is_win_app_created = true
                end
              end
              unless is_win_app_created then
                raise Exception
              end
              win_details = queue_win.sidfmd_queue_win_details
              if win_details.exists? then
                SidfmdQueueWinDetail.lock
                win_details.each do |win_d|
                  SidfmdQueueWinDetail.create(
                    :id => (SidfmdQueueWinDetail.maximum(:id) || 0) + 1,
                    :sidfmd_queue_win_id => new_queue_win.id,
                    :title => win_d.title,
                    :kb => win_d.kb,
                    :severity => win_d.severity
                  )
                end
              end
              SidfmdHostApp.where(:sidfmd_host_id => host_id).each do |h_app|
                SidfmdQueueLapp.create(
                  :sidfmd_queue_id => queue.id,
                  :product_id => h_app.product_id,
                  :version_id => h_app.version_id
                )
              end
            end
          end
        end
        if dist_name then
          dist = dist_name.split(':')
          if dist.size == 3 then #"#{SidfmdQueueDist#ident}:#{SidfmdQueueDist#version}:#{SidfmdQueueDist:name}"
            SidfmdQueueDist.create(
              sidfmd_queue_id: queue.id,
              ident: dist[0],
              version: dist[1],
              name: dist[2]
            )
          end
        end
      end
    rescue Exception => e
      log_exception(e)
      return false
    end
    return true
  end
 
  #プロダクトを登録する。インポートで使用する
  def regist_products(manual_product_names, local_product_names, host_id)
    host_products = []
    if manual_product_names.any? then
      unregistrable_product_ids = SidfmdProduct.unregistrable_product_ids
      SidfmdProduct.where(:name => manual_product_names).each{|p| 
        next if unregistrable_product_ids.include?(p.pid)
        host_products << SidfmdHostApp.new(sidfmd_host_id: host_id, product_id: p.pid, version_id: 0)
      }
    end
    host_versions = SidfmdVersion.where(:fullname => manual_product_names).map{|v|
      r = nil
      prd = SidfmdProduct.find_by(:id => v.sidfmd_product_id)
      if prd then
        r = SidfmdHostApp.new(sidfmd_host_id: host_id, product_id: prd.pid, version_id: v.vid)
      end
      r
    }.compact
    regist_products = host_products + host_versions
    SidfmdHostApp.import(regist_products) if regist_products.size > 0
    
    local_products = SidfmdLproduct.where(:name => local_product_names).map{|lp|
      SidfmdHostLapp.new(sidfmd_host_id: host_id, sidfmd_lproduct_id: lp.id, sidfmd_lversion_id: 0)
    }
    local_versions = SidfmdLversion.where(:fullname => local_product_names).map{|lv|
      SidfmdHostLapp.new(sidfmd_host_id: host_id, sidfmd_lproduct_id: lv.sidfmd_lproduct_id, sidfmd_lversion_id: lv.id)
    }
    regist_lproducts = local_products + local_versions
    SidfmdHostLapp.import(regist_lproducts) if regist_lproducts.size > 0
  end

  # get_hosts_by_lmのitemをCSVの1行分のitemに変換する。
  def item2csv(host)
    item = []
    item << (host[:sidfmd_host] ? host[:sidfmd_host].id : '').to_s.encode('UTF-16LE')
    item << host[:apikey].encode('UTF-16LE')
    item << host[:hostname].encode('UTF-16LE')
    item << (host[:sidfmd_host] ? host[:sidfmd_host].sidfmd_categories.distinct : []).collect {|x| x.name.encode('UTF-16LE') }.join(','.encode('UTF-16LE')).encode('UTF-16LE')
    item << (host[:sidfmd_host] ? (host[:sidfmd_host].refdate ? host[:sidfmd_host].refdate.strftime('%Y/%m/%d') : '') : '').encode('UTF-16LE')
    item << host[:flg_delete].to_s.encode('UTF-16LE')
    item << (host[:sidfmd_host] ? host[:sidfmd_host].is_agent.to_s : '').encode('UTF-16LE')
    item << (host[:sidfmd_host] ? (host[:sidfmd_host].memo ? host[:sidfmd_host].memo : '') : '').encode('UTF-16LE')
    if host[:sidfmd_host] then
      host[:sidfmd_host].sidfmd_host_apps.each do |app|
        if app.version_id then
          version = SidfmdVersion.where(:vid => app.version_id).first
          if version then
            item << version.fullname.encode('UTF-16LE')
          elsif app.product_id then
            product = SidfmdProduct.where(:pid => app.product_id).first
            if product then
              item << product.name.encode('UTF-16LE')
            end
          end
        elsif app.product_id then
          product = SidfmdProduct.where(:pid => app.product_id).first
          if product then
            item << product.name.encode('UTF-16LE')
          end
        end
      end
      host[:sidfmd_host].sidfmd_host_lapps.each do |lapp|
        if lapp.sidfmd_lversion_id == 0 then
          lproduct = SidfmdLproduct.find_by(:id => lapp.sidfmd_lproduct_id)
          item << lproduct.name.encode('UTF-16LE') if lproduct
        else 
          lversion = SidfmdLversion.find_by(:id => lapp.sidfmd_lversion_id)
          item << lversion.fullname.encode('UTF-16LE') if lversion
        end
      end
    end
    item
  end

  #get_hosts_by_lmのitemをnew_export用のCSVの1行分のitemに変換する。
  def create_new_export_hosts_info(host, sidfmd_host)
    #列(項目)は下記の通り
    #ホスト削除、ホストID、トークン、ホスト名、表示名、カテゴリ、基準日、削除、Agentフラグ、メモ
    item = []
    if !@role_admin then
      if !sidfmd_host then
         return item
      end
      host_cats = sidfmd_host.sidfmd_categories.distinct.select{|cat| (cat.is_system || @user_category_ids.include?(cat.id))}
    else
      if sidfmd_host then
        host_cats = sidfmd_host.sidfmd_categories.distinct
      end
    end
    get_summary_prior_category unless @summary_prior_category
    item << ''
    item << ''
    item << (sidfmd_host ? sidfmd_host.id : '').to_s
    item << host['apikey']
    item << host['hostname']
    item << (sidfmd_host ? (sidfmd_host.ext_name ? sidfmd_host.ext_name : '') : '')
    item << (sidfmd_host ? host_cats : []).sort{|a, b|
              r = (
                @summary_prior_category == 'custom' ?
                (a.is_system ? 10 : 1) <=> (b.is_system ? 10 : 1) :
                (a.is_system ? 1 : 10) <=> (b.is_system ? 1 : 10)
              )
              if r == 0 then
                r = a.name.downcase <=> b.name.downcase
              end
              r
            }
            .map{|x| escape_category_tsv(x.name) }
            .join(',')
    item << (sidfmd_host ? (sidfmd_host.refdate ? sidfmd_host.refdate.strftime('%Y/%m/%d') : '') : '')
    item << host['flg_delete'].to_s
    item << (sidfmd_host ? sidfmd_host.is_agent.to_s : '')
    item << (sidfmd_host ? (sidfmd_host.memo ? sidfmd_host.memo : '') : '')

    item
  end

  #new_exportのホストパッケージ情報
  def create_host_pkgs_info_for_new_export(sidfmd_host)
    items = []
    return items if !sidfmd_host
    queue = sidfmd_host.get_last_queue
    if queue then
      queue.sidfmd_queue_pkgs.sort_by{|pkg| pkg.pkg}.each do |pkg|
        item = []
        item << sidfmd_host.id.to_s
        item << 'packages'
        item << pkg.pkg
        items << item
      end
    end
    items
  end

  #new_exportのホスト手動登録プロダクト情報
  def create_host_manual_products_info_for_new_export(sidfmd_host)
    items = []
    return items if !sidfmd_host
    host_id = sidfmd_host.id
    product_id_with_version_ids = {}
    prods_data = {}
    prods_for_sort = {}
    sidfmd_host.sidfmd_host_apps.each do |app|
      prod = SidfmdProduct.find_by(:pid => app.product_id)
      if prod then
        prods_for_sort[prod.pid] = prod if !prods_for_sort[prod.pid]
        if app.version_id == nil || app.version_id == 0 then
          item = []
          item << host_id.to_s
          item << 'manual_products'
          item << prod.name
          prods_data[app.product_id] = item
        else
          product_id_with_version_ids[app.product_id] = [] unless product_id_with_version_ids[app.product_id]
          product_id_with_version_ids[app.product_id] << app.version_id
        end
      end
    end
    prods_for_sort.values.sort_by{|prod| prod.name}.each do |prod|
      items << prods_data[prod.pid] if prods_data[prod.pid]
      version_ids = product_id_with_version_ids[prod.pid]
      SidfmdVersion.where(:vid => version_ids).order(:version).each do |ver|
        item = []
        item << host_id.to_s
        item << 'manual_products'
        item << ver.fullname
        items << item
      end
    end
    items
  end

  #new_exportのホストローカルプロダクト情報
  def create_host_local_products_info_for_new_export(sidfmd_host)
    items = []
    return items if !sidfmd_host
    host_id = sidfmd_host.id
    lproduct_id_with_lversion_ids = {}
    lprods_data = {}
    lprods_for_sort = {}
    sidfmd_host.sidfmd_host_lapps.each do |lapp|
      lprod = SidfmdLproduct.find_by(:id => lapp.sidfmd_lproduct_id)
      if lprod then
        lprods_for_sort[lprod.id] = lprod if !lprods_for_sort[lprod.id]
        if lapp.sidfmd_lversion_id == 0 then
          item = []
          item << host_id.to_s
          item << 'local_products'
          item << lprod.name
          lprods_data[lapp.sidfmd_lproduct_id] = item
        else
          lproduct_id_with_lversion_ids[lapp.sidfmd_lproduct_id] = [] unless lproduct_id_with_lversion_ids[lapp.sidfmd_lproduct_id]
          lproduct_id_with_lversion_ids[lapp.sidfmd_lproduct_id] << lapp.sidfmd_lversion_id
        end
      end
    end
    lprods_for_sort.values.sort_by{|lprod| lprod.name}.each do |lprod|
      items << lprods_data[lprod.id] if lprods_data[lprod.id]
      lversion_ids = lproduct_id_with_lversion_ids[lprod.id]
      SidfmdLversion.where(:id => lversion_ids).order(:priority).each do |lver|
        item = []
        item << host_id.to_s
        item << 'local_products'
        item << lver.fullname
        items << item
      end
    end
    items
  end

  #sidfmd_queue_win_appsの値をエンコード
  def encode_win_app(app)
    "#{URI.encode_www_form_component(app.publisher)}:#{URI.encode_www_form_component(app.name)}:#{URI.encode_www_form_component(app.version)}:#{URI.encode_www_form_component(app.uninstall)}"
  end

  #sidfmd_queue_win_appsの値をエンコードしたものをデコード
  def decode_win_app(app)
    "#{URI.decode_www_form_component(app)}"
  end

  #new_exportの最新同期アプリケーション情報
  def create_host_apps_info_for_new_export(sidfmd_host)
    items = []
    return items if !sidfmd_host
    host_id = sidfmd_host.id
    queue = sidfmd_host.get_last_queue
    if queue then
      win_apps = nil
      queue_win = queue.sidfmd_queue_win
      if queue_win then
        win_apps = queue_win.sidfmd_queue_win_apps
      end
      if win_apps && win_apps.exists? then
        win_apps.each do |app|
          item = []
          item << host_id.to_s
          item << 'queue_win_apps'
          item << encode_win_app(app)
          items << item
        end
      else
        queue.sidfmd_queue_apps.sort{|a, b| 
          r = a.name <=> b.name 
          if r == 0 then
            r = a.version <=> b.version
          end
          r
        }.each do |app|
          item = []
          item << host_id.to_s
          item << 'queue_apps'
          item << "#{app.name}:#{app.version}:#{app.path}"
          items << item
        end
      end
    end
    items
  end
 
  #new_exportの最新同期OS情報
  def create_host_os_info_for_new_export(sidfmd_host)
    item = []
    return item if !sidfmd_host
    host_id = sidfmd_host.id
    queue = sidfmd_host.get_last_queue
    if queue then
      dist = queue.sidfmd_queue_dist
      if dist then
        item << host_id.to_s
        item << 'queue_dist'
        item << "#{dist.ident}:#{dist.version}:#{dist.name}"
      end
    end
    item
  end

  def get_import_hostlim_over_categories(action_array)
    host_categoryids = {}
    hostname_categoryids = {}
    cnt = 0
    action_array.each do |ac|
      cnt += 1 
      ac = ac.deep_symbolize_keys
      case ac[:action]
      when 'create_new_host' then
        hostid_for_hostlim_validation = "new_host#{cnt}"
        category_ids = SidfmdCategory.where(:name => ac[:new_data][:categories]).pluck(:id)
        host_categoryids[hostid_for_hostlim_validation] = category_ids
        hostname_categoryids[ac[:new_data][:hostname]] = category_ids
      when 'undelete_host' then
        category_ids = SidfmdCategory.where(:name => ac[:new_data][:categories]).pluck(:id)
        host_categoryids[ac[:old_data][:sidfmd_host][:id]] = category_ids
        hostname_categoryids[ac[:new_data][:hostname]] = category_ids
      when 'modify_host' then
        category_change_op = ac[:options].find{|op| op[:action] == 'category_change' }
        if category_change_op then
          category_ids = SidfmdCategory.where(:name => category_change_op[:new]).pluck(:id)
          host_categoryids[ac[:old_data][:sidfmd_host][:id]] = category_ids
          hostname_categoryids[ac[:new_data][:hostname]] = category_ids
        end
      when 'remove_host' then
        del_sidfmd_host_id = ac[:old_data].first[:host][:sidfmd_host][:id]
        category_ids = [] #削除するので空
        host_categoryids[del_sidfmd_host_id] = category_ids
      end
    end
    hostlim_over_categories = SidfmdCategoriesHost.hostlim_over_categories(host_categoryids)
    ret = {
      :host_categoryids => host_categoryids,
      :hostname_categoryids => hostname_categoryids,
      :hostlim_over_categories => hostlim_over_categories
    }
  end

  FILTER_KEYS = {}

  def init_filter_keys
    if FILTER_KEYS.size == 0 then
      FILTER_KEYS[:category_id] = '0'
      FILTER_KEYS[:hostname] = ''
    end
  end
 
  def check_filters
    init_filter_keys
    FILTER_KEYS.each do |key, val|
      unless @hosts[:filters].has_key?(key) then
        @hosts[:filters][key] = val
      end
    end
    @default_filters = FILTER_KEYS
  end

  REFDATE_INVALID = Date.new(4000,12,31).freeze
  REFDATE_NONE = Date.new(3000,12,31).freeze

  def apply_filters
    get_summary_prior_category unless @summary_prior_category
    filter_hostname = @hosts[:filters][:hostname].downcase
    ret = @hosts[:hosts].find_all {|item| 
      # カテゴリ検索
      if @hosts[:filters][:category_id] != '0' then
        category_id = @hosts[:filters][:category_id].to_i
        if item[:sidfmd_host] then
          next false if category_id == -2
          categories = item[:sidfmd_host].sidfmd_categories
          categories = categories.where(:id => category_id) if category_id > 0
          categories_count = categories.count
          next false if (category_id == -1 and categories_count != 0)
          next false if (category_id > 0 and categories_count == 0)
        else
          next false if category_id != -2
        end
      end
      # ホスト名照合
      if @hosts[:filters][:hostname] != '' then
        next false unless item[:hostname].downcase[filter_hostname] || (item[:sidfmd_host] && item[:sidfmd_host][:ext_name] && item[:sidfmd_host][:ext_name].downcase[filter_hostname])
        #next false unless item[:hostname].downcase[@hosts[:filters][:hostname].downcase]
      end
      # ホスト詳細画面からリンクした時に、表示するホストを１つに絞る
      if @hosts[:filters][:filter_sidfmd_host_id] && @hosts[:filters][:filter_sidfmd_host_id] > 0 then 
        if !item[:sidfmd_host] || item[:sidfmd_host].id != @hosts[:filters][:filter_sidfmd_host_id] then
          next false
        end
      end
      #if @hostid_for_filter_one_host != '' then
      #  @hostid_for_filter_one_host = @hostid_for_filter_one_host.to_i
      #  if item[:sidfmd_host] && item[:sidfmd_host].id == @hostid_for_filter_one_host then
      #    target_host = SidfmdHost.find_by(:id => @hostid_for_filter_one_host)
      #    if target_host then
      #      if !@sidfm_roles.include?(:admin)then
      #        user_host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => @user_category_ids)
      #                          .map{|h| h.sidfmd_host_id}
      #        if !user_host_ids.include?(@hostid_for_filter_one_host) then
      #          next false
      #        else
      #          @hosts[:filters][:hostname] = target_host.hostname
      #        end
      #      else
      #        @hosts[:filters][:hostname] = target_host.hostname
      #      end
      #    else
      #      next false
      #    end
      #  else
      #    next false
      #  end
      #end                    
      true
    }
    case @hosts[:order]
    when 'hostname_asc', 'hostname_desc' then
      temp_ev = proc {|x|
        display_name = x[:sidfmd_host] ? (x[:sidfmd_host][:ext_name] ? x[:sidfmd_host][:ext_name] : x[:hostname]) : x[:hostname]
        [ x[:flg_delete] ? 1000 : 100, display_name.downcase ]
      }

      case @hosts[:order]
      when 'hostname_asc'
        ret.sort! {|x, y|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
          end
          r
        }
      when 'hostname_desc' then
        ret.sort! {|y, x|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
          end
          r
        }
      end
    when 'categories_asc', 'categories_desc' then
      category_hosts_order = {}
      cat_order_num = 1
      if @summary_prior_category == 'custom' then
        #カスタムカテゴリ優先の場合
        order_category_hosts = (
          @role_admin ?
          SidfmdCategory.joins(:sidfmd_categories_hosts) :
          SidfmdCategory.joins(:sidfmd_categories_hosts).where(
            :sidfmd_categories_hosts => { :sidfmd_category_id => @user_category_ids, :sidfmd_host_id => @user_host_ids }
          )
        )
        .order('CASE WHEN sidfmd_categories.is_system IS TRUE THEN sidfmd_categories.id ELSE 0 END')
        .order('LOWER(sidfmd_categories.name COLLATE "C")')
      else
        # システムカテゴリ優先の場合
        order_category_hosts = ( 
          @role_admin ?
          SidfmdCategory.joins(:sidfmd_categories_hosts) :
          SidfmdCategory.joins(:sidfmd_categories_hosts).where(
            :sidfmd_categories_hosts => { :sidfmd_host_id => @user_host_ids }
          )
        )
        .where(:sidfmd_categories => { :is_system => true })
        .order('sidfmd_categories.id') 
      end

      order_category_hosts.select('sidfmd_categories_hosts.sidfmd_host_id AS host_id').each do |data|
        next if category_hosts_order[data.host_id]
        category_hosts_order[data.host_id] = cat_order_num
        cat_order_num += 1
      end

      temp_ev = proc {|x|
        if x and x[:sidfmd_host] then
          order_num = category_hosts_order[x[:sidfmd_host].id]
          if order_num then
            [ x[:flg_delete] ? 1000 : 100, 100, order_num ]
          else
            [ x[:flg_delete] ? 1000 : 100, 200, '' ]
          end
        else
          [ x[:flg_delete] ? 1000 : 100, 300, '' ]
        end
      }

      if @hosts[:order] == 'categories_asc' then
        ret.sort! {|x, y|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
            if r == 0 then
              r = xx[2] <=> yy[2]
            end
          end
          r
        }
      else
        ret.sort! {|y, x|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
            if r == 0 then
              r = xx[2] <=> yy[2]
            end
          end
          r
        }
      end
    when 'os_asc', 'os_desc' then
      host_osnames = {}
      create_display_content_items(
        ret.map{|item| item[:sidfmd_host] ? item[:sidfmd_host].id : nil}.compact,
        [
          'last_queue_info',
          'last_dist_queue',
          'last_win_queue',
          'manual_app_osname'
        ]
      )
      .each do |host_info|
        host_osnames[host_info.id] = get_osname(
          host_info.last_queue_info,
          host_info.last_win_queue,
          host_info.last_dist_queue,
          host_info.manual_app_osname
        )
      end

      temp_ev = proc {|x|
        if x and x[:sidfmd_host] then
          if host_osnames[x[:sidfmd_host].id] then
            [ x[:flg_delete] ? 1000 : 100, 100, host_osnames[x[:sidfmd_host].id] ]
          else
            [ x[:flg_delete] ? 1000 : 100, 200, '' ]
          end
        else
          [ x[:flg_delete] ? 1000 : 100, 300, '' ]
        end
      }

      case @hosts[:order]
      when 'os_asc' then
        ret.sort! {|x, y|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
            if r == 0 then
              r = xx[2] <=> yy[2]
            end
          end
          r
        }
      when 'os_desc' then
        ret.sort! {|y, x|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
            if r == 0 then
              r = xx[2] <=> yy[2]
            end
          end
          r
        }
      end
    when 'agentprds_asc', 'agentprds_desc' then
      last_queue_ids = []
      host_last_queue_existent = {}
      create_display_content_items(
        ret.map{|item| item[:sidfmd_host] ? item[:sidfmd_host].id : nil}.compact,
        [ 'last_queue_info' ]
      )
      .each do |host_info|
        if host_info.last_queue_info then
          last_queue_ids << host_info.last_queue_info[0].to_i
          host_last_queue_existent[host_info.id] = true
        else
          host_last_queue_existent[host_info.id] = false
        end
      end

      app_counts = get_app_counts(last_queue_ids)
      pkg_counts = get_pkg_counts(last_queue_ids)

      temp_ev = proc {|x|
        if x and x[:sidfmd_host] then
          if host_last_queue_existent[x[:sidfmd_host].id] then
            [
              x[:flg_delete] ? 1000 : 100,
              (
                (app_counts[x[:sidfmd_host].id] ? app_counts[x[:sidfmd_host].id] : 0) +
                (pkg_counts[x[:sidfmd_host].id] ? pkg_counts[x[:sidfmd_host].id] : 0)
              )
            ]
          else
            [ x[:flg_delete] ? 1000 : 100, -100 ]
          end
        else
          [ x[:flg_delete] ? 1000 : 100, -200 ]
        end
      }

      case @hosts[:order] 
      when 'agentprds_asc' then
        ret.sort! {|x, y|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = yy[1] <=> xx[1]
          end
          r
        }
      when 'agentprds_desc' then
        ret.sort! {|y, x|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = yy[1] <=> xx[1]
          end
          r
        }
      end
    when 'manualapps_asc', 'manualapps_desc' then
      manual_product_counts = get_manual_product_counts(ret.map{|item| item[:sidfmd_host] ? item[:sidfmd_host].id : nil}.compact)

      temp_ev = proc {|x|
        if x and x[:sidfmd_host] then
          if manual_product_counts[x[:sidfmd_host].id] then
            [ x[:flg_delete] ? 1000 : 100, manual_product_counts[x[:sidfmd_host].id] ]
          else
            [ x[:flg_delete] ? 1000 : 100, -100 ]
          end
        else
          [ x[:flg_delete] ? 1000 : 100, -200 ]
        end
      }

      case @hosts[:order]
      when 'manualapps_asc'
        ret.sort! {|x, y|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = yy[1] <=> xx[1]
          end
          r
        }
      when 'manualapps_desc' then
        ret.sort! {|y, x|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = yy[1] <=> xx[1]
          end
          r
        }
      end
    when 'refdate_asc', 'refdate_desc' then 
      temp_ev = proc {|x|
        if x and x[:sidfmd_host] then
          if x[:sidfmd_host].refdate then
            [ x[:flg_delete] ? 1000 : 100, x[:sidfmd_host].refdate ]
          else
            [ x[:flg_delete] ? 1000 : 100, REFDATE_NONE ]
          end
        else
          [ x[:flg_delete] ? 1000 : 100, REFDATE_INVALID ]
        end
      }

      case @hosts[:order]
      when 'refdate_asc' then
        ret.sort! {|x, y|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
          end
          r
        }
      when 'refdate_desc' then
        ret.sort! {|y, x|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
          end
          r
        }
      end
    when 'memo_asc', 'memo_desc' then
      temp_ev = proc {|x|
        if x and x[:sidfmd_host] then
          if x[:sidfmd_host].memo then
            [ x[:flg_delete] ? 1000 : 100, 100 ]
          else
            [ x[:flg_delete] ? 1000 : 100, 200 ]
          end
        else
          [ x[:flg_delete] ? 1000 : 100, 300 ]
        end
      }

      case @hosts[:order]
      when 'memo_asc'
        ret.sort! {|x, y|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
          end
          r
        }
      when 'memo_desc' then
        ret.sort! {|y, x|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
          end
          r
        }
      end
    when 'manual_asc', 'manual_desc' then
      temp_ev = proc {|x|
        display_name = x[:sidfmd_host] ? (x[:sidfmd_host][:ext_name] ? x[:sidfmd_host][:ext_name] : x[:hostname]) : x[:hostname]
        [ x[:flg_delete] ? 1000 : 100, (x[:sidfmd_host] and !x[:sidfmd_host].is_agent ? 100 : 1000), display_name ]
      }

      case @hosts[:order]
      when 'manual_asc'
        ret.sort! {|x, y|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
            if r == 0 then
              r = xx[2] <=> yy[2]
            end
          end
          r
        }
      when 'manual_desc' then
        ret.sort! {|y, x|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
            if r == 0 then
              r = xx[2] <=> yy[2]
            end
          end
          r
        }
      end
    when 'agent_asc', 'agent_desc' then
      temp_ev = proc {|x| 
        display_name = x[:sidfmd_host] ? (x[:sidfmd_host][:ext_name] ? x[:sidfmd_host][:ext_name] : x[:hostname]) : x[:hostname]
        [ x[:flg_delete] ? 1000 : 100, (x[:sidfmd_host] and x[:sidfmd_host].is_agent ? 100 : 1000), display_name ]
      }

      case @hosts[:order]
      when 'agent_asc' then
        ret.sort! {|x, y|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
            if r == 0 then
              r = xx[2] <=> yy[2]
            end
          end
          r
        }
      when 'agent_desc' then
        ret.sort! {|y, x|
          xx = temp_ev.call(x)
          yy = temp_ev.call(y)
          r = xx[0] <=> yy[0]
          if r == 0 then
            r = xx[1] <=> yy[1]
            if r == 0 then
              r = xx[2] <=> yy[2]
            end
          end
          r
        }
      end
    else
      # どれにも該当しない場合はホスト名検索とする。
      ret.sort! {|x, y| (x[:sidfmd_host] ? (x[:sidfmd_host][:ext_name] ? x[:sidfmd_host][:ext_name] : x[:hostname]) : x[:hostname]).downcase <=> (y[:sidfmd_host] ? (y[:sidfmd_host][:ext_name] ? y[:sidfmd_host][:ext_name] : y[:hostname]) : y[:hostname]).downcase }
      #ret.sort! {|x, y| x[:hostname].downcase <=> y[:hostname].downcase }
    end
    ret
  end

  def create_content
    ret = SidfmdHost.get_hosts_by_lm(:'sort[hostname]' => :asc)
    if ret then
      if @role_admin then
        ret[:hosts].delete_if {|x| x[:sidfmd_host].nil? }
      else
        ret[:hosts].delete_if {|x| x[:sidfmd_host].nil? or !@user_host_ids.include?(x[:sidfmd_host].id) }
      end
      if Sidfm.deleted_host_display then
        ret[:hosts].delete_if{|x| x[:flg_delete] == true }
      end
    end
    ret
  end

  def get_content
    err_message = nil
    content = nil
    content = Rails.cache.fetch(@cache_prefix + '/hosts/0', SidfmControllerSupport::CACHE_OPTIONS) do
      begin
        create_content
      rescue Exception => e
        log_exception(e)
        err_message = 'ライセンスサーバと通信ができないため、ライセンス設定画面とホスト設定画面にアクセスできません。' 
      end
    end
    if content.nil? then
      Rails.cache.clear
      err_message = 'ライセンスサーバにアクセスできません。' 
    end
    ret = { :content => content, :err_message  => err_message }
  end

  def create_summary_host_html(host, data, **opts)
    ret = {
      :chkbox => view_context.create_summary_license_chkbox(host, opts[:checked]),
      :del_flag => view_context.create_summary_license_del_flag(host[:flg_delete]),
      :hostname => view_context.create_summary_license_hostname(host, data[:last_received_at], @sidfm_roles),
      :agent => view_context.create_summary_license_agent_flag(host[:sidfmd_host] && host[:sidfmd_host].is_agent),
      :manual => view_context.create_summary_license_manual_flag(host[:sidfmd_host] && !host[:sidfmd_host].is_agent),
      :categories => view_context.create_summary_license_categories(host, data[:system_category], data[:custom_categories], @sidfm_roles, opts[:summary_prior_category]),
      :os => view_context.create_summary_license_osname(data[:osname]),
      :agentprds => view_context.create_summary_license_agent_prds(data[:pkg_count], data[:app_count], host, data[:osname], @sidfm_roles),
      :manualapps => view_context.create_summary_license_manual_apps(host, data[:lapp_count], @sidfm_roles),
      :refdate => view_context.create_summary_license_basedate(host, @sidfm_roles),
      :memo => view_context.create_summary_license_memo(host, @sidfm_roles),
      :operation => view_context.create_summary_license_operation(host, opts[:license_number], opts[:used_license_number] )
    }
  end

  # ホストのOS名
  # def create_display_content_itemsで取得できる値を使用することを想定
  def get_osname(last_queue_info, last_win_queue, last_dist_queue, manual_app_osname)
    last_queue_id = last_queue_info ? last_queue_info[0].to_i : nil
    win_os_info =
      if last_queue_id && last_win_queue && last_queue_id == last_win_queue[0].to_i then
        SidfmdQueueWin.new(
          os: last_win_queue[1],
          osname: last_win_queue[2]
        ).os_info
      else
        nil
      end
    osname =
      if win_os_info && win_os_info[:version_id] > 0 then
        win_os_info[:name]
      else
        if last_queue_id && last_dist_queue && last_queue_id == last_dist_queue[0].to_i then
          last_dist_queue[1]
        else
          manual_app_osname
        end
      end
  end

  #ホストID毎の検出アプリ数  
  def get_app_counts(queue_ids)
    SidfmdHost.get_queue_app_counts(queue_ids)
  end

  #ホストID毎のパッケージ数  
  def get_pkg_counts(queue_ids)
    SidfmdHost.get_queue_pkg_counts(queue_ids)
  end

  #ホストID毎の手動登録プロダクト数
  def get_manual_product_counts(host_ids)
    SidfmdHost.get_manual_product_counts(host_ids)
  end

  # 個別の項目のデータが必要な場合は、itemsに項目名を格納する
  # itemsが空の場合は、全項目を取得する
  def create_display_content_items(sidfmd_host_ids, items = [])
    hosts = SidfmdHost.where(:id => sidfmd_host_ids)
    all_items_necessary = items.size == 0

    if all_items_necessary or items.include?('last_queue_info') then
      hosts = hosts.select(%|
        (
          SELECT ARRAY[ CAST(id AS text), CAST(created_at AS text) ]
          FROM sidfmd_queues
          WHERE sidfmd_queues.sidfmd_host_id = sidfmd_hosts.id
          ORDER BY created_at DESC
          LIMIT 1
        ) AS last_queue_info
      |)
    end

    if all_items_necessary or items.include?('system_category') then
      hosts = hosts.select(%|
        (
          SELECT ARRAY[ CAST(sidfmd_categories.id AS text), sidfmd_categories.name, sidfmd_categories.desc ]
          FROM sidfmd_categories_hosts
            INNER JOIN sidfmd_categories ON sidfmd_categories_hosts.sidfmd_category_id = sidfmd_categories.id
          WHERE sidfmd_hosts.id = sidfmd_categories_hosts.sidfmd_host_id AND sidfmd_categories.is_system IS TRUE
          LIMIT 1
        ) AS system_category_data
      |)
    end

    if all_items_necessary or items.include?('custom_categories') then
      hosts = hosts.select(%|
        (
          ARRAY(
            SELECT ARRAY_TO_STRING(ARRAY[ CAST(sidfmd_categories.id AS text), sidfmd_categories.name ], ':&&&&&:')
            FROM sidfmd_categories_hosts
              INNER JOIN sidfmd_categories ON sidfmd_categories_hosts.sidfmd_category_id = sidfmd_categories.id
            WHERE sidfmd_hosts.id = sidfmd_categories_hosts.sidfmd_host_id AND sidfmd_categories.is_system IS FALSE#{@role_admin ? "" : " AND sidfmd_categories.id IN (SELECT unnest(ARRAY[#{(@user_category_ids.any? ? @user_category_ids : [ -1 ]).join(', ')}]))"}
            ORDER BY LOWER(sidfmd_categories.name COLLATE "C")
          )
        ) AS custom_categories_data
      |)
    end

    if all_items_necessary or items.include?('last_dist_queue') then
      hosts = hosts.select(%|
        (
          SELECT ARRAY[ CAST(sidfmd_queue_id AS text), concat(sidfmd_queue_dists.name, ' ', sidfmd_queue_dists.version, (CASE WHEN sidfmd_queue_dists.name = 'Ubuntu' THEN ' LTS' ELSE '' END)) ]
          FROM sidfmd_queues
            INNER JOIN sidfmd_queue_dists ON sidfmd_queue_dists.sidfmd_queue_id = sidfmd_queues.id
          WHERE sidfmd_queues.sidfmd_host_id = sidfmd_hosts.id
          ORDER BY sidfmd_queues.created_at DESC
          LIMIT 1
        ) AS last_dist_queue
      |)
    end

    if all_items_necessary or items.include?('last_win_queue') then
      hosts = hosts.select(%|
        (
          SELECT ARRAY[ CAST(sidfmd_queue_id AS text), sidfmd_queue_wins.os, sidfmd_queue_wins.osname ]
          FROM sidfmd_queues
            INNER JOIN sidfmd_queue_wins ON sidfmd_queue_wins.sidfmd_queue_id = sidfmd_queues.id
          WHERE sidfmd_queues.sidfmd_host_id = sidfmd_hosts.id
          ORDER BY sidfmd_queues.created_at DESC
          LIMIT 1
        ) AS last_win_queue
      |)
    end

    if all_items_necessary or items.include?('manual_app_osname') then
      hosts = hosts.select(%|
        (
          SELECT CASE WHEN sidfmd_host_apps.version_id IS NOT NULL AND sidfmd_host_apps.version_id > 0 AND sidfmd_versions.fullname IS NOT NULL THEN sidfmd_versions.fullname ELSE sidfmd_products.name END
          FROM sidfmd_products
            INNER JOIN sidfmd_host_apps ON sidfmd_host_apps.product_id = sidfmd_products.pid
            INNER JOIN sidfmd_versions ON sidfmd_versions.sidfmd_product_id = sidfmd_products.id AND sidfmd_versions.vid = sidfmd_host_apps.version_id
          WHERE sidfmd_products.category = 'OS' AND sidfmd_host_apps.sidfmd_host_id = sidfmd_hosts.id
          ORDER BY sidfmd_host_apps.id ASC /* SidfmdHost#get_osで順指定なしで最初のデータを取得していたため、このようにする */
          LIMIT 1
        ) AS manual_app_osname
      |)
    end

    if all_items_necessary or items.include?('is_agent') then
      hosts = hosts.select(:is_agent)
    end

    hosts = hosts.select(:id) #idは必須
  end

  # 表示する元データの生成。処理速度改善のために使用
  def create_display_content(sidfmd_host_ids)
    sidfmd_host_ids = sidfmd_host_ids && @user_host_ids unless @role_admin
    ret = {}
    last_queue_ids = []
    host_ids = []

    create_display_content_items(sidfmd_host_ids).each do |data|
      system_category = data.system_category_data
      custom_categories = []
      data.custom_categories_data.each do |cat|
        if cat then
          cat = cat.split(':&&&&&:', 2)
          custom_categories << { :id => cat[0].to_i, :name => cat[1] }
        end
      end
      last_queue_id = data.last_queue_info ? data.last_queue_info[0].to_i : nil
      last_queue_created_at = data.is_agent ? (data.last_queue_info ? data.last_queue_info[1].to_time : nil) : nil

      ret[data.id] = {
        :last_received_at => last_queue_created_at,
        :system_category => (
          system_category ?
          { :id => system_category[0].to_i, :name => system_category[1], :desc => system_category[2] } :
          nil
        ),
        :custom_categories => custom_categories,
        :osname => get_osname(data.last_queue_info, data.last_win_queue, data.last_dist_queue, data.manual_app_osname),
      }

      last_queue_ids << last_queue_id if last_queue_id
      host_ids << data.id
    end
   
    app_counts = get_app_counts(last_queue_ids)

    pkg_counts = get_pkg_counts(last_queue_ids)

    lapp_counts = get_manual_product_counts(host_ids)

    sidfmd_host_ids.each do |host_id|
      host_id = host_id.to_i
      if ret[host_id] then
        ret[host_id][:app_count] = app_counts[host_id] ? app_counts[host_id] : 0
        ret[host_id][:pkg_count] = pkg_counts[host_id] ? pkg_counts[host_id] : 0
        ret[host_id][:lapp_count] = lapp_counts[host_id] ? lapp_counts[host_id] : 0
      end
    end

    ret
  end

  public

  def index
    pages_params = params.fetch(:pages, {}).permit(:offset, :per_page, :filters, :order, :checked, :all_checked)
    @hosts = {}
    @hosts[:offset] = (pages_params[:offset] || 0).to_i
    @hosts[:per_page] = (pages_params[:per_page] || 100).to_i
    @hosts[:order] = (pages_params[:order] || 'hostname_asc')
    @hosts[:checked] = JSON.parse(pages_params[:checked] || '[]')
    @hosts[:all_checked] = ((pages_params[:all_checked] || '') == 'true')
    @hosts[:filters] = JSON.parse(pages_params[:filters] || '{}').symbolize_keys
  end

  def index_r
    pages_params = params.fetch(:pages, {}).permit(:offset, :per_page, :filters, :order, :checked, :all_checked, :host_id, :clear_session)
    if pages_params[:clear_session] then
      clear_import_session
      redirect_to :action => :index_r
    end
    #get_summary_prior_category
    #@license_key = Sidfm.sidfmdlm_license_key
    #@license_apikey = Sidfm.sidfmdlm_license_apikey
    #@register_apikey = Sidfm.sidfmdlm_register_apikey
    #@hosts = Rails.cache.fetch(@cache_prefix + '/hosts/0', SidfmControllerSupport::CACHE_OPTIONS) do
    #  begin
    #    create_content
    #  rescue Exception => e
    #    log_exception(e)
    #    redirect_to :controller => :sidfm, :action => :index_r, :notice => 'ライセンスサーバと通信ができないため、ライセンス設定画面とホスト設定画面にアクセスできません。' and return
    #  end
    #end
    #if @hosts.nil? then
    #  Rails.cache.clear
    #  redirect_to :action => :license, :notice => 'ライセンスサーバにアクセスできません。' and return
    #end
    #if !@sidfm_roles.include?(:admin) then
    #  @hosts[:hosts].delete_if {|x| x[:flg_delete] }
    #end
    @hosts = {}
    @hosts[:offset] = (pages_params[:offset] || 0).to_i
    @hosts[:per_page] = (@summary_table_lines || pages_params[:per_page] || 100).to_i
    @hosts[:order] = (pages_params[:order] || 'hostname_asc')
    @hosts[:checked] = JSON.parse(pages_params[:checked] || '[]')
    @hosts[:all_checked] = ((pages_params[:all_checked] || '') == 'true')
    @hosts[:filters] = JSON.parse(pages_params[:filters] || '{}').symbolize_keys
    if pages_params[:host_id] && pages_params[:host_id] != '' then
      #ホスト詳細画面からリンクした時、１つのホストに絞る
      target_host = SidfmdHost.find_by(:id => pages_params[:host_id].to_i)
      if target_host then
        @hosts[:filters][:filter_sidfmd_host_id] = target_host.id
        @hosts[:filters][:hostname] = target_host.hostname
      end
    end
    #@hostid_for_filter_one_host = (pages_params[:host_id] || '')#ホスト詳細画面からリンクした時、１つのホストに絞るための変数
    check_filters
    #@hosts[:hosts_filtered] = apply_filters
    #@all_hostnames =  @hosts[:hosts_filtered].map{|host| if host[:flg_delete] then nil else host[:hostname] end}.compact
    #@hosts[:content] = @hosts[:hosts_filtered].slice(@hosts[:offset], @hosts[:per_page])
    #@hosts[:content] = [] unless @hosts[:content]
    #@hosts[:total_size] = @hosts[:hosts_filtered].size
    @system_categories = SidfmdCategory.system
    @not_system_categories = @role_admin ? SidfmdCategory.not_system : SidfmdCategory.where(:id => @user_category_ids)
    @new_import_notices = session[:import_notices] ? session[:import_notices] : nil
    clear_import_session
    #ブラウザバックかどうか判断するための処理
    #if !session['chkbok_ids'] then
    #  session['chkbok_ids'] = {}
    #end
    #if !session['chkbok_ids']['hostadmin'] then
    #  session['chkbok_ids']['hostadmin'] = {}
    #end
    #session['chkbok_ids']['hostadmin']['is_browser_back'] = false
  end

  def license
    begin
      @license_key = Sidfm.sidfmdlm_license_key
      @license_apikey = Sidfm.sidfmdlm_license_apikey
      @register_apikey = Sidfm.sidfmdlm_register_apikey
      #filtered_params = params.fetch(:params, {}).permit(:offset, :limit, :hostname, :'sort[hostname]', :'sort[flg_delete]', :'sort[created_at]')
      @hosts = SidfmdHost.get_hosts_by_lm(:'sort[hostname]' => :asc)
    rescue Exception => e
      log_exception(e)
      redirect_to :controller => :sidfm, :action => :index_r, :notice => 'ライセンスサーバと通信ができないため、ライセンス設定画面とホスト設定画面にアクセスできません。' and return
    end
  end

  def license_setting
    @license_key = Sidfm.sidfmdlm_license_key
    @license_apikey = Sidfm.sidfmdlm_license_apikey
  end

  def license_update
    filtered_params = params.fetch(:params, {}).permit(:license_key, :license_password)
    @license_key = filtered_params[:license_key]
    license_password = filtered_params[:license_password]
    begin
      if !SidfmdHost.auth_api(@license_key, license_password) then
        redirect_to :action => :license_setting, :notice => 'ライセンス認証に失敗しました。' and return
      end
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :license_setting, :notice => 'ライセンスサーバへのアクセスに失敗しました。' and return
    end
    Rails.cache.clear

    redirect_to :action => :license
  end

  def license_apikey_update
    res = SidfmdHost.license_apikey_update
    if res.respond_to?(:[]) and res[:success] then
      redirect_to :action => :license and return
    end
    redirect_to :action => :license, :notice => 'APIキーの更新に失敗しました。'
  end

  # ホストAPIキーの更新
  # deprecated:使用せず。
  def host_apikey_update
    filtered_params = params.fetch(:params, {}).permit(:id, :hostname)
    host_id = filtered_params[:id]
    hostname = filtered_params[:hostname]
    if host_id and hostname then
      SidfmdHost.transaction do
        res = SidfmdHost.update_host_apikey(host_id.to_i)
        if res.respond_to?(:[]) and res[:success] then
          host = SidfmdHost.where(:hostname => hostname).first
          if host then
            host.update_attribute :host_apikey, res[:apikey]
          else
            res = SidfmdHost.establish_consistency
          end
          redirect_to :action => :index and return
        end
      end
    end
    redirect_to :action => :index_r, :notice => 'APIキーの更新に失敗しました。'
  end

  # 削除状態の変更
  def toggle_host_delete
    filtered_params = params.permit(:host_id, :sidfmd_host_id)
    err_message = ''
    r = {}
    host_id = filtered_params[:host_id]
    sidfmd_host_id = filtered_params[:sidfmd_host_id]

    if host_id && sidfmd_host_id then
      host = SidfmdHost.find_by(:id => sidfmd_host_id)
      if host then
        if !@role_admin and !@user_host_ids.include?(host.id) then
          err_message = '自分の属するカスタムカテゴリのホスト以外は操作できません。'
          render :layout => false, :json => { :err_message => err_message } and return
        end
      end
    end
    res = toggle_delete_host(
      host_id: host_id,
      sidfmd_host_id: sidfmd_host_id,
      role_admin: @role_admin,
      sidfm_roles: @sidfm_roles,
      user_host_ids: @user_host_ids,
      user_category_ids: @user_category_ids,
      update_view: false
    )
    render :layout => false, :json => { :err_message => res[:success] ? '' : res[:message] }
  end

  # ホスト名変更
  def rename_host
    filtered_params = params.permit(:hostid, :hostname, :old_hostname)

    hostid = filtered_params[:hostid]
    hostname = filtered_params[:hostname]
    old_hostname = filtered_params[:old_hostname]
    err_message = ''
    unless hostid or hostname or old_hostname then
      err_message = 'パラメータが不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    if hostid.size == 0 or hostname.size == 0 or old_hostname.size == 0 then
      err_message = 'パラメータが不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
   
    if hostid and hostname then
      if /\A[0-9a-zA-Z._-]+\z/ !~ hostname then
        err_message = 'ホスト名が不正です。'
        render :layout => false, :json => { :err_message => err_message } and return
      end
      if hostname.bytesize > SidfmdHost::MAX_HOSTNAME_BYTE_LENGTH then
        err_message = SidfmdHost::MAX_HOSTNAME_BYTE_LENGTH.to_s + 'バイトを超えるホスト名は登録できません。'
        render :layout => false, :json => { :err_message => err_message } and return
      end
      unless @role_admin then
        host = SidfmdHost.where(:hostname => old_hostname).first
        if host then
          if !@user_host_ids.include?(host.id) then
            err_message = '自分の属するカスタムカテゴリのホスト以外は操作できません。'
            render :layout => false, :json => { :err_message => err_message } and return
          end
        else
          err_message = 'ホストが存在しません。'
          render :layout => false, :json => { :err_message => err_message } and return
        end
      end
      SidfmdHost.transaction do
        res = SidfmdHost.rename_host(hostid, hostname)
        if res.respond_to?(:[]) then
          if res[:success] then
            host = SidfmdHost.where(:hostname => old_hostname).first
            if host then
              host.update_attribute :hostname, hostname
            else
              res = SidfmdHost.establish_consistency
            end
            Rails.cache.clear
          else
            err_message = '登録済のホスト名、もしくは不正な文字を指定したため、操作に失敗しました。'
            render :layout => false, :json => { :err_message => err_message } and return
          end
        else
          err_message = 'ライセンスサーバとの通信に失敗しました。'
          render :layout => false, :json => { :err_message => err_message } and return
        end
      end
    else
      err_message = 'ホスト名の変更に失敗しました。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    render :layout => false, :json => { :err_message => err_message }
  end

  def set_display_name
    filtered_params = params.permit(:hostid, :display_name)
    hostid = filtered_params[:hostid]
    display_name = filtered_params[:display_name]
    err_message = ''
    unless hostid || display_name then
      err_message = 'パラメータが不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    if hostid.size == 0 then
      err_message = 'パラメータが不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    if display_name.size > SidfmdHost::MAX_DISPLAY_NAME_LENGTH then
      err_message = '100文字を超える表示名は登録できません。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    if display_name =~ /<|>|"|'/ then
      err_message = '表示名に使用できない文字が含まれています。'
      render :layout => false, :json => { :err_message => err_message } and return
    elsif display_name.size > 0 && display_name.strip.size === 0 then
      err_message = '空白文字のみの表示名は登録できません。'
      render :layout => false, :json => { :err_message => err_message } and return 
    elsif display_name =~ /&&&&&/ then
      err_message = '「&&&&&」の文字列(5つ連なるアンド)は使用できません。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    if hostid && display_name then
      host = SidfmdHost.find_by(:id => hostid)
      unless @role_admin then
        if host && !@user_host_ids.include?(host.id) then
          err_message = '自分の属するカスタムカテゴリのホスト以外は操作できません。'
          render :layout => false, :json => { :err_message => err_message } and return
        end
      end
      SidfmdHost.transaction do
        if host then
          display_name = nil if display_name.size == 0
          host.update_attribute :ext_name, display_name
          Rails.cache.clear
        else
          err_message = 'ホストが存在しません。'
          render :layout => false, :json => { :err_message => err_message } and return
        end
      end
    else
      err_message = '表示名の変更に失敗しました。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    render :layout => false, :json => { :err_message => err_message } and return
  end

  # メモ更新
  def update_memo
    filtered_params = params.permit(:hostname, :memo)
    hostname = filtered_params[:hostname]
    memo = filtered_params[:memo]
    if !hostname or !memo then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    if hostname.size == 0 then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    if memo.length > SidfmdHost::MEMO_MAX_LEN then
      render :layout => false, :json => { :err_message => 'メモに入力可能な最大文字数を超えています' } and return
    end
    SidfmdHost.transaction do
      host = SidfmdHost.where(:hostname => hostname).first
      unless host then
        render :layout => false, :json => { :err_message => 'ホストが存在しません。' } and return
      end
      host.update_attribute :memo, memo.size > 0 ? memo : nil
      Rails.cache.clear
    end
    render :layout => false, :json => { :err_message => '' }
  end

  # メモ一括更新
  def update_memo_bulk
    filtered_params = params.permit(:hostnames, :memo)
    hostnames = filtered_params[:hostnames]
    memo = filtered_params[:memo]
    unless hostnames or memo then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    if memo.size > SidfmdHost::MEMO_MAX_LEN then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    begin
      hostnames = JSON.parse(hostnames)
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    if hostnames.size == 0 then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    memo = nil if memo.size == 0
    SidfmdHost.transaction do
      hostnames.each do |hostname|
        host = SidfmdHost.where(:hostname => hostname).first
        if host then
          host.update_attribute :memo, memo
        end
      end
      Rails.cache.clear
    end
    render :layout => false, :json => { :err_message => '' }
  end

  # 基準日更新
  def update_refdate
    filtered_params = params.permit(:hostname, :refdate)
    hostname = filtered_params[:hostname]
    refdate = filtered_params[:refdate]
    unless hostname or refdate then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    if hostname.size == 0 then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    SidfmdHost.transaction do
      host = SidfmdHost.where(:hostname => hostname).first
      unless host then 
        render :layout => false, :json => { :err_message => 'ホストが存在しません。' } and return
      end
      if refdate.size > 0 then
        begin
          refdate = Date.parse(refdate) if refdate.size > 0
        rescue ArgumentError => e
          log_exception(e)
          render :layout => false, :json => { :err_message => '不正な日付です。' } and return
        end
      else
        refdate = nil
      end
      begin
        host.update_attribute :refdate, refdate
        Rails.cache.clear
      rescue ActiveRecord::StatementInvalid => e
        log_exception(e)
        render :layout => false, :json => { :err_message => '不正な日付です。' } and return
      end
    end
    render :layout => false, :json => { :err_message => '' }
  end

  # 基準日一括更新
  def update_refdate_bulk
    filtered_params = params.permit(:hostnames, :refdate)
    hostnames = filtered_params[:hostnames]
    refdate = filtered_params[:refdate]
    unless hostnames or refdate then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    begin
      hostnames = JSON.parse(hostnames)
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    if hostnames.size == 0 then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    if refdate.size > 0 then
      begin
        refdate = Date.parse(refdate) if refdate.size > 0
      rescue ArgumentError => e
        log_exception(e)
        render :layout => false, :json => { :err_message => '不正な日付です。' } and return
      end
    else
      refdate = nil
    end
    begin
      SidfmdHost.transaction do
        hostnames.each do |hostname|
          host = SidfmdHost.where(:hostname => hostname).first
          if host then
            host.update_attribute :refdate, refdate
          end
        end
      end
      Rails.cache.clear
    rescue ActiveRecord::StatementInvalid => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '不正な日付です。' } and return
    end
    render :layout => false, :json => { :err_message => '' }
  end

  # カテゴリ更新
  def update_categories
    
    filtered_params = params.permit(:sidfmdhostid, :hostname, :selected_category_ids, :system_category_id)
    err_message = ''
    #begin
    #  dump = JSON.parse(filtered_params[:dump])
    #rescue JSON::ParserError => e
    #  log_exception(e)
    #  err_message = '入力が不正です。'
    #  render :layout => false, :json => { :err_message => err_message } and return
    #end
    #if dump["pages[filters]"] then
    #  dump["pages[filters]"] = JSON.generate(dump["pages[filters]"])
    #end
    #dump[:action] = :index_r
    sidfmdhostid = filtered_params[:sidfmdhostid].to_i
    hostname = filtered_params[:hostname]
    selected_category_ids = filtered_params[:selected_category_ids]
    system_category_id = filtered_params[:system_category_id].to_i
    if sidfmdhostid == 0 or system_category_id == 0 then
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    if !hostname or !selected_category_ids then
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    begin
      selected_category_ids = JSON.parse(selected_category_ids)
    rescue JSON::ParserError => e
      log_exception(e)
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    if !@role_admin then
      if !@user_host_ids.include?(sidfmdhostid) then
        err_message = '入力が不正です。'
        render :layout => false, :json => { :err_message => err_message } and return
      end
      diff = selected_category_ids & @user_category_ids
      if diff.size == 0 then
        err_message = '少なくとも一つはカスタムカテゴリを指定する必要があります。'
        render :layout => false, :json => { :err_message => err_message } and return
      #elsif SidfmdCategory.where.not(:id => @user_category_ids).where(:id => diff).where(:is_system => false).exists? then
      #  err_message = '自分の属するカスタムカテゴリ以外を指定することはできません。'
      #  render :layout => false, :json => { :err_message => err_message } and return
      #else
      #  diff = selected_category_ids & @user_category_ids
      #  if diff.size == 0 then
      #    err_message = '自分の属するカスタムカテゴリ以外を指定することはできません。'
      #    render :layout => false, :json => { :err_message => err_message } and return
      #  end
      #  selected_category_ids = diff
      end
    end

    host = nil
    host = SidfmdHost.where(:id => sidfmdhostid).first 
    hostlim_over_categories = SidfmdCategoriesHost.hostlim_over_categories( { host.id => selected_category_ids } )
    if hostlim_over_categories.any? then
      err_message = "以下のカテゴリは、ホスト登録数が上限を超えるため、登録できません。\n#{hostlim_over_categories.map{|cat| cat.name}.join("\n")}"
      render :layout => false, :json => { :err_message => err_message } and return
    end

    begin
      SidfmdHost.transaction do
        unless host then
          host = SidfmdHost.where(:hostname => hostname).first
          unless host then
            err_message = '対象ホストが見つかりません。'
            render :layout => false, :json => { :err_message => err_message } and return
          end
        end
        change_category_ids_for_rules = SidfmdHost.get_change_category_ids(
          @role_admin,
          @user_category_ids,
          { host.hostname => (@role_admin ? selected_category_ids : selected_category_ids & @user_category_ids) }
        )
        raise Exception if change_category_ids_for_rules[:err]
        process_rules = SidfmdHost.process_related_rules(
          :role_admin => @role_admin,
          :user_category_ids => @user_category_ids,
          :action => 'category_change',
          :change_category_host_ids => change_category_ids_for_rules[:change_category_ids],
        )
        raise Exception unless process_rules

        SidfmdIssueCtag.where(:sidfmd_issue_id =>
          host.sidfmd_issues.select(:id)
        ).destroy_all

        res = host.modify_categories2(system_category_id, selected_category_ids,
                                     @role_admin ? nil : @user_category_ids)
        unless res then
          raise StandardError, 'modify_categories failed'
        end
      end

    rescue Exception => e
      log_exception(e)
      err_message = 'カテゴリ更新に失敗しました。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    Rails.cache.clear
    render :layout => false, :json => { :err_message => err_message }
  end

  # カテゴリ一括変更
  def update_categories_bulk
    #filtered_params = params.fetch(:categories, {}).permit(:hostnames, :selected_category_ids, :system_category_id, :dump)
    #dump = {}
    filtered_params = params.permit(:hostnames, :selected_category_ids, :system_category_id)
    err_message = ''
    #begin
      #dump = JSON.parse(filtered_params[:dump])
    #rescue JSON::ParserError => e
    #  log_exception(e)
    #  err_message = '入力が不正です。'
    #  render :layout => false, :json => { :err_message => err_message } and return
    #end
    #dump[:action] = :index_r
    hostnames = filtered_params[:hostnames]
    unless hostnames then
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    begin
      hostnames = JSON.parse(hostnames)
    rescue JSON::ParserError => e
      log_exception(e)
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    system_category_id = filtered_params[:system_category_id]
    unless system_category_id then
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    selected_category_ids = filtered_params[:selected_category_ids]
    unless selected_category_ids then
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    begin
      selected_category_ids = JSON.parse(selected_category_ids)
    rescue JSON::ParserError => e
      log_exception(e)
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    if !@role_admin then
      target_host_ids = SidfmdHost.where(:hostname => hostnames).pluck(:id)
      if (target_host_ids - @user_host_ids).size > 0 then
        err_message = '入力が不正です。'
        render :layout => false, :json => { :err_message => err_message } and return
      end
      if selected_category_ids.size == 0 then
        err_message = '少なくとも一つはカスタムカテゴリを指定する必要があります。'
        render :layout => false, :json => { :err_message => err_message } and return
      elsif SidfmdCategory.where.not(:id => @user_category_ids).where(:id => selected_category_ids).where(:is_system => false).exists? then
        err_message = '自分の属するカスタムカテゴリ以外を指定することはできません。'
        render :layout => false, :json => { :err_message => err_message } and return
      #diff = selected_category_ids & @user_category_ids
      #if !SidfmdCategory.where(:id => diff).where(:is_system => false).exists? then
      #   err_message = '少なくとも一つはカスタムカテゴリを指定する必要があります。'
      #   render :layout => false, :json => { :err_message => err_message } and return
      #else
      #  diff = selected_category_ids & @user_category_ids
      #  if diff.size == 0 then
      #    err_message = '自分の属するカスタムカテゴリ以外を指定することはできません。'
      #    render :layout => false, :json => { :err_message => err_message } and return
      #  end
      #  selected_category_ids = diff
      end
    end

    host_categoryids = {}
    SidfmdHost.where(:hostname => hostnames).each do |host|
      host_categoryids[host.id] = selected_category_ids
    end
    hostlim_over_categories = SidfmdCategoriesHost.hostlim_over_categories(host_categoryids)
    if hostlim_over_categories.any? then
      err_message = "以下のカテゴリは、ホスト登録数が上限を超えるため、登録できません。\n#{hostlim_over_categories.map{|cat| cat.name}.join("\n")}"
      render :layout => false, :json => { :err_message => err_message } and return
    end

    begin
      host_ids = []
      SidfmdHost.transaction do
        #関連するルールの削除・変更
        change_category_ids = {}
        hostnames.each do |hostname|
          change_category_ids[hostname] = selected_category_ids
        end
        change_category_ids_for_rules = SidfmdHost.get_change_category_ids(@role_admin, @user_category_ids, change_category_ids)
        raise Exception if change_category_ids_for_rules[:err]
        process_rules = SidfmdHost.process_related_rules(
          :role_admin => @role_admin,
          :user_category_ids => @user_category_ids,
          :action => 'bulk_category_change',
          :change_category_host_ids => change_category_ids_for_rules[:change_category_ids],
        )
        raise Exception unless process_rules

        SidfmdIssueCtag.where(:sidfmd_issue_id =>
          SidfmdIssue.joins(:sidfmd_host).where(:sidfmd_hosts => { :hostname => hostnames }).select(:id)
        ).destroy_all

        SidfmdHost.where(:hostname => hostnames).each do |host|
          res = host.modify_categories2(system_category_id, selected_category_ids, @role_admin ? nil : @user_category_ids)
          unless res then
            logger.info('modify_categories failed')
            raise StandardError
          else
            host_ids << host.id if host.last_checked_at
          end
        end
      end
      Rails.cache.clear
    rescue Exception => e
      log_exception(e)
      err_message = 'カテゴリ更新に失敗しました。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    render :layout => false, :json => { :err_message => err_message } and return
  end

  def register_apikey_update
    res = Sidfm.sidfmdlm_register_apikey_update
    if res then
      redirect_to :action => :license
      return
    end
    redirect_to :action => :license, :notice => 'APIキーの登録・更新に失敗しました。'
  end

  def register_apikey_remove
    Sidfm.sidfmdlm_register_apikey = nil
    unless Sidfm.sidfmdlm_register_apikey then
      redirect_to :action => :license
      return
    end
    redirect_to :action => :license, :notice => 'APIキーの削除に失敗しました。'
  end

  # ホストを登録する。
  def register_host
    f_params = params.permit(
      :hostname,
      :display_name,
      :category_id,
      :selected,
      :selected_category_ids,
      :refdate,
      :memo,
      :selected_lproducts
    )

    if !f_params[:selected_category_ids] then
      render :layout => false, :json => { :err_message => 'カスタムカテゴリが不正です。' } and return
    end
    begin
      selected_category_ids = JSON.parse(f_params[:selected_category_ids])
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => 'カスタムカテゴリが不正です。' } and return
    end

    selected = f_params[:selected]
    selected_lproducts = f_params[:selected_lproducts]
    if !selected || !selected_lproducts then
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end
    begin
      selected = JSON.parse(selected)
      selected_lproducts = JSON.parse(selected_lproducts)
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '入力が不正です。' } and return
    end

    host_apps = []
    host_lapps = []
    os_count=0
    selected.each_pair do |k, v|
      v.each do |vv|
        pid = vv['pid'].to_i
        vid = vv['vid'].to_i

        if SidfmdProduct.where(pid: pid).pluck(:category)[0] == 'OS' then
          os_count+=1
          if os_count > 1 then
            err_message = 'プロダクトのOSは一つまでしか登録できません'
            render :layout => false, :json => { :err_message => err_message } and return
          end
        end
        host_apps << SidfmdHostApp.new(
          :product_id => pid,
          :version_id => vid
        )
      end
    end

    selected_lproducts.each do |k, v|
      v.each do |vv|
        pid = vv['pid'].to_i
        vid = vv['vid'].to_i
        host_lapps << SidfmdHostLapp.new(
          :sidfmd_lproduct_id => pid,
          :sidfmd_lversion_id => vid
        )
      end
    end

    host_apps.uniq!
    host_lapps.uniq!

    res = create_host(
      hostname: f_params[:hostname],
      display_name: f_params[:display_name],
      system_category_id: f_params[:category_id],
      custom_category_ids: selected_category_ids,
      refdate: f_params[:refdate],
      memo: f_params[:memo],
      host_apps: host_apps,
      host_lapps: host_lapps,
      role_admin: @role_admin,
      user_category_ids: @user_category_ids,
      sidfm_roles: @sidfm_roles
    )
    render :layout => false, :json => { :err_message => res[:success] ? '' : res[:message] }
  end

  # 設定されているアプリケーションを更新する。
  def update_applications
    f_params = params.permit(:hostname, :selected, :selected_lproducts)
    err_message = ''
    begin
      selected = JSON.parse(f_params[:selected])
      selected_lproducts = JSON.parse(f_params[:selected_lproducts])
    rescue JSON::ParserError => e
      log_exception(e)
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return 
    end
  
    unregistrable_product_ids = SidfmdProduct.unregistrable_product_ids 
    all_selected_products_size = 0
    selected.each_pair do |k, v|
      v.each do |vv|
        pid = vv['pid'].to_i
        vid = vv['vid'].to_i
        if vid == 0 and unregistrable_product_ids.include?(pid) then
          err_message = 'プロダクトに、「全て」を登録することができないプロダクトが含まれています。'
          render :layout => false, :json => { :err_message => err_message } and return
        end
      end
      all_selected_products_size += v.size
    end
    selected_lproducts.each_pair do |k, v|
      all_selected_products_size += v.size
    end

    if all_selected_products_size > SidfmdHost::MAX_REGISTER_PRODUCTS then
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end

    hostname = f_params[:hostname]


    host = nil
    SidfmdHost.transaction do
      host = SidfmdHost.where(:hostname => hostname).first
      unless host then
        err_message = 'ホストが存在しません'
        render :layout => false, :json => { :err_message => err_message } and return
      end
      host.sidfmd_host_apps = []
      host.sidfmd_host_lapps = []
      host.save

      apps = []
      lapps = []
      selected.each_pair do |k, v|
        v.each do |vv|

          pid = vv['pid'].to_i
          vid = vv['vid'].to_i
          apps << SidfmdHostApp.new(:sidfmd_host_id => host.id,
                                    :product_id => pid,
                                    :version_id => vid)
        end
      end
      selected_lproducts.each do |k, v|
        v.each do |vv|
          pid = vv['pid'].to_i
          vid = vv['vid'].to_i
          lapps << SidfmdHostLapp.new(:sidfmd_host_id => host.id,
                                      :sidfmd_lproduct_id => pid,
                                      :sidfmd_lversion_id => vid)
        end
      end
    
      SidfmdHostApp.import(apps) if apps.any?
      SidfmdHostLapp.import(lapps) if lapps.any?
    end
    Rails.cache.clear
 
    render :layout => false, :json => { :err_message => err_message } and return
  end

  # プロダクト一括更新
  def update_applications_bulk
    filtered_params = params.permit(:hostnames, :selected, :selected_lproducts)
    err_message = ''
    hostnames = filtered_params[:hostnames]
    selected = filtered_params[:selected]
    selected_lproducts = filtered_params[:selected_lproducts]
    unless hostnames or selected or selected_lproducts then
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    begin
      hostnames = JSON.parse(hostnames)
    rescue JSON::ParserError => e 
      log_exception(e)
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end
    begin
      selected = JSON.parse(selected)
      selected_lproducts = JSON.parse(selected_lproducts)
    rescue JSON::ParserError => e
      log_exception(e)
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end

    unregistrable_product_ids = SidfmdProduct.unregistrable_product_ids
    all_selected_products_size = 0
    selected.each_pair do |k, v|
      v.each do |vv|
        pid = vv['pid'].to_i
        vid = vv['vid'].to_i
        if vid == 0 and unregistrable_product_ids.include?(pid) then
          err_message = 'プロダクトに、「全て」を登録することができないプロダクトが含まれています。'
          render :layout => false, :json => { :err_message => err_message } and return
        end
      end
      all_selected_products_size += v.size
    end
    selected_lproducts.each_pair do |k, v|
      all_selected_products_size += v.size
    end

    if all_selected_products_size > SidfmdHost::MAX_REGISTER_PRODUCTS then
      err_message = '入力が不正です。'
      render :layout => false, :json => { :err_message => err_message } and return
    end

    host_ids = []
    apps = []
    lapps = []

    SidfmdHost.transaction do
      hostnames.each do |hostname|
        host = SidfmdHost.where(:hostname => hostname).first
        if host then
          host.sidfmd_host_apps = []
          host.sidfmd_host_lapps = []
          host.save

          selected.each_pair do |k, v|
            v.each do |vv|
              pid = vv['pid'].to_i
              vid = vv['vid'].to_i
              apps << SidfmdHostApp.new(:sidfmd_host_id => host.id,
                                        :product_id => pid,
                                        :version_id => vid)
            end
          end
          
          selected_lproducts.each do |k, v|
            v.each do |vv|
              pid = vv['pid'].to_i
              vid = vv['vid'].to_i
              lapps << SidfmdHostLapp.new(:sidfmd_host_id => host.id,
                                          :sidfmd_lproduct_id => pid,
                                          :sidfmd_lversion_id => vid)
            end
          end
        end
      end
    end

    SidfmdHostApp.import(apps) if apps.any?
    SidfmdHostLapp.import(lapps) if lapps.any?
   
    Rails.cache.clear
    
    render :layout => false, :json => { :err_message => err_message }
  end

  #パッケージ更新
  def modify_pkgs
    f_params = params.permit(:pkgs, :host_id, :osname)
    pkgs = f_params[:pkgs]
    osname = f_params[:osname]
    host_id = f_params[:host_id]
    if !pkgs or !host_id or !osname then
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    begin
      pkgs = JSON.parse(pkgs)
    rescue JSON::ParserError => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '不正な操作です。' } and return
    end

    res = {}
    begin
      SidfmdHost.transaction do
        host = SidfmdHost.lock.find_by(:id => host_id.to_i)
        if !host or host.flg_delete then
          render :layout => false, :json => { :err_message => 'ホストが削除されています。' } and return
        end
        res = update_host_packages(pkgs: pkgs, host: host, osname: osname)
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message =>  'パッケージに更新に失敗しました。' } and return
    end

    Rails.cache.clear
    render :layout => false, :json => { :err_message => res[:success] ? '' : res[:message] }
  end

  # ホスト一覧のインポートを行い、操作予定の内容を出力する
  def import
    @actions = session[:import_actions]
    @errs = session[:import_errs]
    @notices = session[:import_notices]
    @is_new_import = session[:is_new_import]
    if @actions then
      #ルールに関係するデータ
      @rule_del_host_ids = []
      @rule_cat_change_hostnames = {}
      @actions.each do |action|
        action_name = action['action']
        if action_name == 'modify_host' then
          a_options = action['options']
          a_options.each do |op|
            if op['action'] == 'category_change' then
              hostname = action['old_data']['hostname']
              @rule_cat_change_hostnames[hostname] = op['new']
            end
          end
        elsif action_name == 'remove_host' then
          old_data = action['old_data']
          old_data.each do |od|
            host_info = od['host']['sidfmd_host']
            if host_info then
              host_id = host_info['id']
              @rule_del_host_ids << host_id
            end
          end
        end
      end
    else
      redirect_to :action => :index_r, :notice => 'インポートするデータがありません。'
    end
  end

  def new_import
    f_params = params.permit(:import_file)
    zip_file = f_params[:import_file]
    err = false
    unless zip_file then
      logger.info('invalid zip file')
      err = true
    end
    hosts_info_file = nil
    hosts_info_file_lines = []
    host_opts_info_files = []
    new_host_opts_info_files = []
    begin
      Zip::File.open(zip_file) do |zip|
        zip.each do |entry|
          next if entry.name && (entry.name.index('_MACOSX') || entry.name.index('.DS_Store'))
          if entry.file? then
            if entry.name && entry.name.index('hostinfo.csv') then
              hosts_info_file = entry.get_input_stream.read.gsub(/^$\n/, '')
            elsif entry.name && entry.name.index('regist') then
              file = entry.get_input_stream.read
              if file.getbyte(0) == 255 && file.getbyte(1) == 254 then
                file = file[2, file.length - 2]
              end
              file = file.force_encoding('UTF-16LE').encode('UTF-8')
              new_host_opts_info_files << {:name => entry.name, :file => file}
            elsif entry.name && entry.name.index('opt') then
              file = entry.get_input_stream.read
              if file.getbyte(0) == 255 && file.getbyte(1) == 254 then
                file = file[2, file.length - 2]
              end
              file = file.force_encoding('UTF-16LE').encode('UTF-8')
              host_opts_info_files << {:name => entry.name, :file => file}
            end
          end
        end
      end
    rescue Exception => e
      log_exception(e)
      err = true
    end

    clear_import_session
    session[:import_actions] = []
    session[:import_errs] = []
    session[:import_notices] = []
    session[:is_new_import] = true
    session[:hosts_info_file_lines] = hosts_info_file_lines
    session[:host_opts_info_files] = host_opts_info_files
    session[:new_host_opts_info_files] = new_host_opts_info_files
    session[:host_opts_info] = {}
    session[:new_host_opts_info] = {}
    unless hosts_info_file then
      logger.info('hostinfo.csv is not found')
      err = true
    else
      if hosts_info_file.getbyte(0) == 255 && hosts_info_file.getbyte(1) == 254 then
        hosts_info_file = hosts_info_file[2, hosts_info_file.length - 2]
      end
      hosts_info_file = hosts_info_file.force_encoding('UTF-16LE').encode('UTF-8')
      hosts_info_file = CSV.parse(hosts_info_file, :col_sep => "\t", :headers => :first_row)
      cnt = 1
      hosts_info_file.each do |line|
        cnt += 1
        begin 
          cats = (line['カテゴリ'] && line['カテゴリ'].size > 0) ? CSV.parse_line(line['カテゴリ']) : nil
        rescue Exception => e
          session[:import_notices] << "hostinfo.csvの#{cnt}行目: カテゴリ列のフォーマットに誤りがあります。"
          err = true
        end
        hosts_info_file_lines << { 
          'ホスト登録ID' => (line['ホスト登録ID'] && line['ホスト登録ID'].size > 0) ? line['ホスト登録ID'] : nil,
          'ホスト削除' => (line['ホスト削除'] && line['ホスト削除'].size > 0) ? line['ホスト削除'] : nil,
          'ホストID' => (line['ホストID'] && line['ホストID'].size > 0) ? line['ホストID'] : nil,
          'トークン' => (line['トークン'] && line['トークン'].size > 0) ? line['トークン'] : nil,
          'ホスト名' => (line['ホスト名'] && line['ホスト名'].size > 0) ? line['ホスト名'] : nil,
          '表示名' => (line['表示名'] && line['表示名'].size > 0) ? line['表示名'] : nil, 
          'カテゴリ' => cats,
          '基準日' => (line['基準日'] && line['基準日'].size > 0) ? line['基準日'] : nil,
          '削除' => (line['削除'] && line['削除'].size > 0) ? line['削除'] : nil,
          'Agentフラグ' => (line['Agentフラグ'] && line['Agentフラグ'].size > 0) ? line['Agentフラグ'] : nil,
          'メモ' => (line['メモ'] && line['メモ'].size > 0) ? line['メモ'] : nil
        }
      end
    end
    begin
      lmhosts = SidfmdHost.get_hosts_by_lm(:'sort[hostname]' => :asc)
    rescue Exception => e
      log_exception(e)
      err = true
    end
    unless lmhosts and lmhosts[:success] then
      logger.info('failed to get hosts')
      err = true
    end
    session[:lmhosts] = lmhosts
    total_size = hosts_info_file_lines.size + host_opts_info_files.size + new_host_opts_info_files.size
   
    render :layout => false, :json => {:err => err, :total_size => total_size}
  end

  def progress_import_opts
    errs = []
    is_exception = false
    product_names = SidfmdProduct.all.map{|p| [ p.name.strip.tr('０-９ａ-ｚＡ-Ｚ','0-9a-zA-Z').gsub('：', ':'), p.name ] }.to_h
    version_names = SidfmdVersion.all.map{|v| [ v.fullname.tr('０-９ａ-ｚＡ-Ｚ','0-9a-zA-Z').gsub('：', ':'), v.fullname ] }.to_h
    lproduct_names = SidfmdLproduct.all.map{|lp| [ lp.name.strip.tr('０-９ａ-ｚＡ-Ｚ','0-9a-zA-Z').gsub('：', ':'), lp.name ] }.to_h
    lversion_names = SidfmdLversion.all.map{|lv| [ lv.fullname.tr('０-９ａ-ｚＡ-Ｚ','0-9a-zA-Z').gsub('：', ':'), lv.fullname ] }.to_h
    unregistrable_product_ids = SidfmdProduct.unregistrable_product_ids

    create_opts_info = ->(opts_file, file_name){
      opts_csv = CSV.parse(opts_file, :col_sep => "\t", :headers => :first_row)
      line_num = 1

      if Sidfm.product_name_inconsistency_support then
        set4manualproducts = Set.new()
        set4localproducts = Set.new()
      end

      opts_csv.each do |line|
        line_num += 1
        new_host_id = line['ホスト登録ID']
        host_id = line['ホストID']
        if new_host_id && host_id then
          errs << "#{file_name}の#{line_num}行目:「ホスト登録ID」列と「ホストID」の両方が入力されています。"
          next
        end
        item_name = line['種類']
        item_data = line['データ']
        next if !item_data || item_data.strip.size == 0 || !item_name || item_name.strip.size == 0
        item_data = item_data.gsub("\n", '').strip.tr('０-９ａ-ｚＡ-Ｚ','0-9a-zA-Z').gsub('：', ':')
        item_name = item_name.gsub("\n", '').strip.tr('ａ-ｚＡ-Ｚ','a-zA-Z')
        if item_name != 'packages' && item_name != 'manual_products' && item_name != 'local_products' && item_name != 'queue_apps' && item_name != 'queue_dist' && item_name != 'queue_win_apps' then
          errs << "#{file_name}の#{line_num}行目: 「種類」列の値が正しくありません。"
          next
        end
        info_hash = {
          :packages => [],
          :manual_products => [],
          :local_products => [],
          :queue_apps => [],
          :queue_win_apps => []
        }
        if new_host_id && new_host_id.strip.size > 0 then
          new_host_id = new_host_id.strip
          unless session[:new_host_opts_info][new_host_id] then
            session[:new_host_opts_info][new_host_id] = info_hash
          end
          if item_name == 'queue_dist' then
            if item_data.count(':') != 2 then
              errs << "#{file_name}の#{line_num}行目: queue_distの「データ」列の値が正しくありません。"
              next
            end
            if session[:new_host_opts_info][new_host_id][item_name.strip.to_sym] then
              session[:import_notices] << "#{file_name}の#{line_num}行目: ホスト登録ID「#{new_host_id}」のqueue_distのデータが複数あります。"
            end
            session[:new_host_opts_info][new_host_id][item_name.strip.to_sym] = item_data
          else
            if item_name == 'queue_apps' && item_data.count(':') != 2 then
              errs << "#{file_name}の#{line_num}行目: queue_appsの「データ」列の値が正しくありません。"
              next

            elsif item_name == 'queue_win_apps' then
              if item_data.count(':') != 3 then
                errs << "#{file_name}の#{line_num}行目: queue_win_appsの「データ」列の値が正しくありません。"
                next
              end
              errs << "#{file_name}の#{line_num}行目: 新規ホストに、queue_win_appsのデータは登録できません。"
            else
              if Sidfm.product_name_inconsistency_support then
                if item_name == 'manual_products' || item_name == 'local_products' then
                  case item_name
                  when 'manual_products' then
                    result = searchProduct(item_data, product_names, version_names, file_name, line_num, item_name)
                  when 'local_products' then
                    result = searchProduct(item_data, lproduct_names, lversion_names, file_name, line_num, item_name)
                  end
  
                  if !result[:success] then
                    errs << result[:err]
                    next
                  else

                    # remove duplication
                    if item_name == 'manual_products' then
                      removeDuplication(set4manualproducts, result[:value])
                    else
                      removeDuplication(set4localproducts, result[:value])
                    end
                  end
                end
              else
                if item_name == 'manual_products' then
                  if !product_names[item_data] && !version_names[item_data] then
                    errs << "#{file_name}の#{line_num}行目: manual_productsの「データ」列の値に、登録されていないプロダクト名が入力されています。" 
                    next
                  elsif !version_names[item_data] then
                    pid = SidfmdProduct.find_by(:name => item_data).pid
                    if unregistrable_product_ids.include?(pid) then
                      errs << "#{file_name}の#{line_num}行目: manual_productsの「データ」列の値に、 仕様によりバージョンを指定する場合にのみしか登録することができないプロダクト名が入力されています。"
                      next
                    end
                  end
                elsif item_name == 'local_products' && !lproduct_names[item_data] && !lversion_names[item_data] then
                  errs << "#{file_name}の#{line_num}行目: local_productsの「データ」列の値に、登録されていないプロダクト名が入力されています。"
                  next
                end
              end
            end
            if (item_name == 'queue_apps' && session[:new_host_opts_info][new_host_id][:queue_win_apps].size > 0) || 
               (item_name == 'queue_win_apps' && session[:new_host_opts_info][new_host_id][:queue_apps].size > 0)
            then
              errs << "#{file_name}の#{line_num}行目: 「queue_apps」と「queue_win_apps」のデータは、どちらか一方のみ登録可能です。"
              next
            end
            session[:new_host_opts_info][new_host_id][item_name.to_sym] << case item_name 
                                                                           when 'manual_products' then 
                                                                            if Sidfm.product_name_inconsistency_support then
                                                                              result[:value]
                                                                            else
                                                                              (product_names[item_data] || version_names[item_data])
                                                                            end
                                                                           when 'local_products'
                                                                            if Sidfm.product_name_inconsistency_support then
                                                                              result[:value]
                                                                            else
                                                                              (lproduct_names[item_data] || lversion_names[item_data])
                                                                            end
                                                                           else
                                                                             item_data
                                                                           end
          end
        elsif host_id && host_id.strip.size > 0 then
          host_id = host_id.strip
          queue = SidfmdQueue.where(:sidfmd_host_id => host_id).order(:created_at => :desc).first
          unless session[:host_opts_info][host_id] then
            session[:host_opts_info][host_id] = info_hash
          end
          if item_name == 'queue_dist' then
            if item_data.count(':') != 2 then
              errs << "#{file_name}の#{line_num}行目: queue_distの「データ」列の値が正しくありません。"
              next
            end
            if session[:host_opts_info][host_id][item_name.strip.to_sym] then
              session[:import_notices] << "#{file_name}の#{line_num}行目: ホストID「#{host_id}」のqueue_distのデータが複数あります。"
            end
            session[:host_opts_info][host_id][item_name.strip.to_sym] = item_data
          else
            if item_name == 'queue_apps' && item_data.count(':') != 2 then
              errs << "#{file_name}の#{line_num}行目: queue_appsの「データ」列の値が正しくありません。"
              next
            elsif item_name == 'manual_products' then
              if !product_names[item_data] && !version_names[item_data] then
                errs << "#{file_name}の#{line_num}行目: manual_productsの「データ」列の値に、登録されていないプロダクト名が入力されています。"
                next
              elsif !version_names[item_data] then
                pid = SidfmdProduct.find_by(:name => item_data).pid
                if unregistrable_product_ids.include?(pid) then
                  errs << "#{file_name}の#{line_num}行目: manual_productsの「データ」列の値に、 仕様によりバージョンを指定する場合にのみしか登録することができないプロダクト名が入力されています。"
                  next
                end
              end
            elsif item_name == 'local_products' && !lproduct_names[item_data] && !lversion_names[item_data] then
              errs << "#{file_name}の#{line_num}行目: local_productsの「データ」列の値に、登録されていないプロダクト名が入力されています。"
              next
            elsif item_name == 'queue_win_apps' then
              if item_data.count(':') != 3 then
                errs << "#{file_name}の#{line_num}行目: queue_win_appsの「データ」列の値が正しくありません。"
                next
              end
              if !queue || !queue.sidfmd_queue_win then
                errs << "#{file_name}の#{line_num}行目: このホストに、queue_win_appsのデータは登録できません。"
                next
              end
            end
            if (item_name == 'queue_apps' && session[:host_opts_info][host_id][:queue_win_apps].size > 0) || 
               (item_name == 'queue_win_apps' && session[:host_opts_info][host_id][:queue_apps].size > 0)
            then
              errs << "#{file_name}の#{line_num}行目: 「queue_apps」と「queue_win_apps」のデータは、どちらか一方のみ登録可能です。"
              next
            end
            session[:host_opts_info][host_id][item_name.to_sym] << case item_name
                                                                   when 'manual_products' then
                                                                     (product_names[item_data] || version_names[item_data])
                                                                   when 'local_products'
                                                                     (lproduct_names[item_data] || lversion_names[item_data])
                                                                   else
                                                                     item_data
                                                                   end
          end
        else
          next
        end
      end
    }

    begin
      is_complete = false
      executed_size = 0
      if session[:host_opts_info_files].size > 0 then
        files = session[:host_opts_info_files].shift(100)
        executed_size = files.size
        files.each{|file| create_opts_info.(file[:file], file[:name])}
      elsif session[:new_host_opts_info_files].size > 0 then
        files = session[:new_host_opts_info_files].shift(100)
        executed_size = files.size
        files.each{|file| create_opts_info.(file[:file], file[:name])}
      else
        is_complete = true
      end 
    rescue Exception => e
      log_exception(e)
      is_exception = true
    end
    session[:import_errs] = session[:import_errs] + errs
    render :layout => false, :json => {:err => is_exception, :is_complete => is_complete, :executed_size => executed_size}
  end

  def progress_import_hosts
    #importの処理を進める
    @hosts = session[:lmhosts].symbolize_keys[:hosts]
    @actions = []
    all_category_names = []
    all_s_category_names =  []
    SidfmdCategory.all.each do |cat|
      all_category_names << cat.name
      all_s_category_names << cat.name if cat.is_system
    end
    old_tokens = []
    @hosts.each do |host|
      if host['sidfmd_host'] then
        apikey = host['sidfmd_host']['host_apikey']
        old_tokens << apikey if apikey
      end
    end
    if !@role_admin then
      user_c_cat_names = User.current.categories.pluck(:name)
    end

    get_filtered_category_names = ->(target_category_names, user_category_names, all_system_category_names){
      cat_names = target_category_names.map{|c| c.gsub("\n", '').strip}.uniq

      filtered_cats = []
      s_cat_names = []
      cat_names.each do |c_name|
        if all_system_category_names.include?(c_name) then
          if s_cat_names.size == 0 then
            filtered_cats << c_name
          end
          s_cat_names << c_name
        elsif user_category_names.include?(c_name) then
          filtered_cats << c_name
        end
      end
      filtered_cats = filtered_cats.sort{|a, b|
        r = (all_system_category_names.include?(a) ? 1: 10) <=> (all_system_category_names.include?(b) ? 1 : 10)
        if r == 0 then
          r = a <=> b
        end
        r
      }
      return { :filtered_category_names => filtered_cats, :system_category_names => s_cat_names }
    }

    is_err = false
    @errs = []
    @notices = []
    host_opts_info = session[:host_opts_info]
    new_host_opts_info = session[:new_host_opts_info]
    session[:new_host_ids] = [] unless session[:new_host_ids]
    session[:import_hostnames] = {} unless session[:import_hostnames]
    hostname_errs = []
    executed_size = 0
    session[:line_num] = 1 unless session[:line_num]
    session[:hosts_info_file_lines].shift(100).each do |line|
      session[:line_num] += 1
      executed_size += 1
      item = {}
      item[:new_host_id] = line['ホスト登録ID']
      item[:flg_host_delete] = line['ホスト削除']
      item[:host_id] = line['ホストID']
      item[:token] = line['トークン']
      item[:hostname] = line['ホスト名']
      item[:ext_name] = line['表示名']
      item[:categories] = line['カテゴリ']
      item[:refdate] = line['基準日']
      item[:flg_delete] = line['削除']
      item[:is_agent] = line['Agentフラグ']
      item[:memo] = line['メモ']
      item[:new_host_id] = item[:new_host_id].strip.gsub("\n", '').tr('０-９','0-9').to_i if item[:new_host_id] && item[:new_host_id].strip.size > 0
      item[:flg_host_delete] = item[:flg_host_delete].strip.gsub("\n", '') if item[:flg_host_delete]
      item[:host_id] = item[:host_id].strip.gsub("\n", '').tr('０-９','0-9').to_i if item[:host_id] && item[:host_id].strip.size > 0
      item[:token] = item[:token].strip.gsub("\n", '').tr('０-９ａ-ｚＡ-Ｚ','0-9a-zA-Z') if item[:token]
      if !item[:hostname] || item[:hostname].strip.size == 0 then
        @errs << "hostinfo.csvの#{session[:line_num]}行目:「ホスト名」列が入力されておりません。"
        next
      end
      item[:hostname] = item[:hostname].gsub("\n", ' ').strip.tr('０-９ａ-ｚＡ-Ｚ','0-9a-zA-Z').gsub('ー', '-')
      check = check_hostname(item[:hostname])
      if check != true then
        msg = check
        @errs << "hostinfo.csvの#{session[:line_num]}行目: #{msg}"
        next
      end
      if session[:import_hostnames][item[:hostname]] then
        hostname_errs << "hostinfo.csvの#{session[:import_hostnames][item[:hostname]]}行目と#{session[:line_num]}行目:「ホスト名」が重複しています。※同じホスト名を複数登録することはできません。"
        is_err = true
        #break
      else
         session[:import_hostnames][item[:hostname]] = session[:line_num]
      end

      item[:ext_name] = item[:ext_name].gsub("\n", ' ').strip if item[:ext_name]
      if item[:ext_name] and item[:ext_name] =~ /&&&&&/ then
        @errs << "hostinfo.csvの#{session[:line_num]}行目: 「&&&&&」の文字列(5つ連なるアンド)は使用できません。"
        next
      end
      item[:flg_delete] = item[:flg_delete].gsub("\n", '').strip if item[:flg_delete]
     
      if item[:flg_host_delete] && item[:flg_host_delete].downcase != 'true' && item[:flg_host_delete].downcase != 'false' then
        @errs << "hostinfo.csvの#{session[:line_num]}行目: 「ホスト削除」列は\"TRUE\"と入力して下さい。"
        next
      end
 
      if item[:flg_delete] && item[:flg_delete].downcase != 'true' && item[:flg_delete].downcase != 'false' then
        @errs << "hostinfo.csvの#{session[:line_num]}行目: 「削除」列は\"TRUE\"もしくは \"FALSE\"と入力して下さい。"
        next
      end

      item[:is_agent] = 'false' if !item[:is_agent] || item[:is_agent].size == 0
      item[:is_agent] = item[:is_agent].strip.gsub("\n", '')
      
      if item[:refdate] && item[:refdate].size > 0 then
        begin
          item[:refdate] = item[:refdate].gsub("\n", '').strip.tr('０-９','0-9').gsub('ー', '/').gsub('-', '/').sub('年', '/').sub('月', '/').sub('日', '').tr('０-９ａ-ｚＡ-Ｚ','0-9a-zA-Z')
          refdate = Date.parse(item[:refdate])
        rescue ArgumentError => e
          log_exception(e)
          msg = "hostinfo.csvの#{session[:line_num]}行目:「基準日」列の値が正しくありません。"
          @errs << msg
        end
      end
      
      if item[:token] && item[:token].size > 0 && !old_tokens.include?(item[:token]) then
        #ライセンスサーバーとDBのホストのapikeyが異なるホストはimportの対象から除外する
        next
      end
    
      if (item[:host_id] && item[:host_id].size > 0) && (!item[:token] || item[:token].size == 0) then
        msg = "hostinfo.csvの#{session[:line_num]}行目:「ホストID」があり「トークン」がありません。更新には両方の値が必要です。"
        @errs << msg
        next
      elsif (!item[:host_id] || item[:host_id].size == 0 ) && (item[:token] && item[:token].size > 0) then
        msg = "hostinfo.csvの#{session[:line_num]}行目:「トークン」があり「ホストID」ありません。更新には両方の値が必要です。"
        @errs << msg
        next
      end

      if !item[:is_agent] && item[:host_id] && item[:token] then
        if item[:flg_host_delete] != 'TRUE' && item[:flg_host_delete] != 'true' then 
          @errs << "hostinfo.csvの#{session[:line_num]}行目:「Agentフラグ」列が入力されておりません。"
          next
        end
      end 

      if item[:host_id] && item[:token] && item[:is_agent] && item[:is_agent].downcase != 'true' && item[:is_agent].downcase != 'false' then
        @errs << "hostinfo.csvの#{session[:line_num]}行目:「Agentフラグ」列は \"TRUE\"もしくは \"FALSE\"と入力して下さい。"
        next
      end
      
      if !@role_admin && item[:host_id] && !@user_host_ids.include?(item[:host_id]) then
        @errs << "hostinfo.csvの#{session[:line_num]}行目: ログインユーザでの操作は許可されておりません。"
        next
      end

      #削除チェック
      if item[:flg_host_delete] && item[:flg_host_delete].downcase == 'true' then
        t_host = @hosts.find {|y| item[:token] == y['apikey'] && !y['flg_delete']}
        if t_host then
          @actions << {
            :action => 'remove_host',
            :action_name => '削除',
            :old_data => [ { :token => item[:token], :host => t_host } ],
            :new_data => nil,
            :options => nil,
          }
        end
        next
      end
      if item[:categories] then
        #カテゴリ名をfilter
        if @role_admin then
          filtered_category_names = get_filtered_category_names.(item[:categories], all_category_names, all_s_category_names)
        else
          filtered_category_names = get_filtered_category_names.(item[:categories], user_c_cat_names, all_s_category_names)
        end
        filtered_cats = filtered_category_names[:filtered_category_names]
        s_cat_names = filtered_category_names[:system_category_names]
        #システムカテゴリが無いホストはエラー
        if s_cat_names.size == 0 && (item[:flg_delete].downcase == 'false' || (!item[:host_id] && !item[:token])) then
          msg = "hostinfo.csvの#{session[:line_num]}行目:「カテゴリ」列に、有効なシステムカテゴリ名が入力されていません。"
          @errs << msg
          next
        end
        if !@role_admin then
          f_cats = filtered_cats.reject{|c_name| all_s_category_names.include?(c_name)}
          if f_cats.size == 0 then
            msg = "hostinfo.csvの#{session[:line_num]}行目: ログインユーザのカテゴリから外れるホストは操作できません。"
            @errs << msg
            next
          end
        end
        item[:categories] = filtered_cats
      else
        if (item[:flg_delete] && item[:flg_delete].downcase == 'false') || (!item[:host_id] && !item[:token])then
          msg = "hostinfo.csvの#{session[:line_num]}行目:「カテゴリ」列に、カテゴリが入力されていません。"  
          @errs << msg
          next
        end
      end
      #新規登録
      if (item[:new_host_id] && item[:new_host_id].size > 0) || (!item[:host_id] && !item[:token]) then
        if session[:new_host_ids].include?(item[:new_host_id]) then
          session[:import_notices] = ['hostinfo.csvの「ホスト登録ID」列に、同じIDが入力されているデータが複数あります。']
          is_err = true
          break
        end
        if SidfmdHost.where(:flg_delete => false).where(:hostname => item[:hostname]).count > 0 then
          msg = "hostinfo.csvの#{session[:line_num]}行目: 既存の削除されていないホストと同じホスト名は使用できません。"
          @errs << msg
          next
        end
        session[:new_host_ids] << item[:new_host_id] if item[:new_host_id]
        #報告者権限でログインユーザのカスタムカテゴリに属さないものはエラー
        if !@role_admin && item[:categories].size == 1 then
          msg = "hostinfo.csvの#{session[:line_num]}行目: ログインユーザのカスタムカテゴリの範囲外となるため、登録の対象となりません。"
          @errs << msg
          next
        end
        if !item[:refdate] then
          #新規登録で基準日がないデータは登録当日の値にする
          @notices << "hostinfo.csvの#{session[:line_num]}行目: 基準日が入力されていない新規ホストデータには、本日の日付が登録されます"
          item[:refdate] = Date.today.strftime('%Y/%m/%d')
        end
        if item[:categories] then
          if s_cat_names.size > 1 then
            @notices << "hostinfo.csvの#{session[:line_num]}行目: システムカテゴリの登録は1つのみ可能なため、システムカテゴリは「#{s_cat_names[0]}」が登録されます。"
          end
        end
        opts_info = new_host_opts_info[item[:new_host_id].to_s]
        if opts_info then
          opts_info = opts_info.symbolize_keys
          item[:packages] = opts_info[:packages].uniq
          item[:apps] = opts_info[:queue_apps].uniq
          item[:manual_products] = opts_info[:manual_products].uniq
          item[:local_products] = opts_info[:local_products].uniq
          item[:dist] = opts_info[:queue_dist]
          item[:win_apps] = opts_info[:queue_win_apps].uniq
          if item[:is_agent].downcase == 'false' then
            if (item[:packages] && item[:packages].size > 0) || (item[:apps] && item[:apps].size > 0) || (item[:dist] && item[:dist].size > 0) || (item[:win_apps] && item[:win_apps].size > 0)then
              @notices << "hostinfo.csvの#{session[:line_num]}行目:「Agentフラグ」列が \"FALSE \"のホストには、registフォルダ下の「packages」「queue_apps」「queue_dist」「queue_win_apps」のデータは登録できません。※脆弱性同期時に自動生成されます。"
              item[:packages] = []
              item[:apps] = []
              item[:win_apps] = []
              item[:dist] = nil
            end
          end
          if (item[:manual_products].size + item[:local_products].size) > SidfmdHost::MAX_REGISTER_PRODUCTS then
            @notices << "hostinfo.csvの#{session[:line_num]}行目: プロダクトの最大登録数は#{SidfmdHost::MAX_REGISTER_PRODUCTS}個のため、このホストのプロダクトの登録は行われません。optsフォルダ以下のファイルをご確認下さい。"
            item[:manual_products] = []
            item[:local_products] = []
          end
        end
        @actions << {
          :action => 'create_new_host',
          :action_name => '新規作成',
          :new_data => item,
          :old_data => nil,
          :options => nil,
          #:new_host_id => item[:new_host_id]
        }
        next
      end

      same_hostname = @hosts.find {|x| x['hostname'] == item[:hostname] }
  
      if item[:flg_delete] and item[:flg_delete].downcase == 'false' and same_hostname and same_hostname['sidfmd_host'] and same_hostname['sidfmd_host']['host_apikey'] != item[:token] and !same_hostname['flg_delete'] then
        msg = "hostinfo.csvの#{session[:line_num]}行目: 既存の削除されていないホストと同じホスト名は使用できません。"
        @errs << msg
        next
      end
      if same_hostname and same_hostname['flg_delete'] and same_hostname['sidfmd_host'] and
         item[:flg_delete] and item[:flg_delete].downcase == 'false'
      then
      # 復元
        if item[:categories] then
          if s_cat_names.size > 1 then
            @notices << "hostinfo.csvの#{session[:line_num]}行目: システムカテゴリの登録は1つのみ可能なため、システムカテゴリは「#{s_cat_names[0]}」が登録されます。"
          end
        end
        host_opts = host_opts_info[item[:host_id].to_s.strip]
        if host_opts then
          host_opts = host_opts.symbolize_keys
          if item[:is_agent].downcase == 'false' && ((host_opts[:packages] && host_opts[:packages].size > 0) || (host_opts[:apps] && host_opts[:apps].size > 0) || (host_opts[:dist] && host_opts[:dist].size > 0) || (host_opts[:queue_win_apps] && host_opts[:queue_win_apps].size > 0)) then
            @notices << "hostinfo.csvの#{session[:line_num]}行目:「Agentフラグ」列が \"FALSE \"のホストには、optsフォルダ下の「packages」「queue_apps」「queue_dist」「queue_win_apps」のデータは登録できません。※脆弱性同期時に自動生成されます。"
          else
            item[:packages] = host_opts[:packages].uniq
            item[:apps] = host_opts[:queue_apps].uniq
            item[:win_apps] = host_opts[:queue_win_apps].uniq
            item[:dist] = host_opts[:queue_dist]
          end
          item[:manual_products] = host_opts[:manual_products].uniq
          item[:local_products] = host_opts[:local_products].uniq
          if (item[:manual_products].size + item[:local_products].size) > SidfmdHost::MAX_REGISTER_PRODUCTS then
            @notices << "hostinfo.csvの#{session[:line_num]}行目: プロダクトの最大登録数は#{SidfmdHost::MAX_REGISTER_PRODUCTS}個のため、このホストのプロダクトの登録は行われません。optsフォルダ以下のファイルをご確認下さい。"
            item[:manual_products] = []
            item[:local_products] = []
          end
        end
        @actions << {
          :action => 'undelete_host',
          :action_name => '復元',
          :new_data => item,
          :old_data => same_hostname,
          :options => nil,
        }
        next
      end
      same_token = @hosts.find {|x| x['apikey'] == item[:token]}
      # 更新のチェック
      if same_token and same_token['sidfmd_host'] then
        target = []
        # カテゴリ比較
        i_categories = item[:categories] ? item[:categories] : []
        t_host = SidfmdHost.find_by(:id => same_token['sidfmd_host']['id'])
        if !t_host then
          @err << [ "hostinfo.csvの#{session[:line_num]}行目: 変更対象のホストが削除されています。" ]
          next
        end
        same_token['sidfmd_host'] = t_host
        if @role_admin then
          s_categories = t_host.sidfmd_categories.map{|x| x.name.strip}
        else
          s_categories = t_host.sidfmd_categories.map{|x|
            (user_c_cat_names.include?(x.name.strip) || x.is_system) ? x.name.strip : nil
          }.compact
        end
        s_categories = s_categories.uniq.sort{|a, b|
          r = (all_s_category_names.include?(a) ? 1: 10) <=> (all_s_category_names.include?(b) ? 1 : 10)
          if r == 0 then
            r = a <=> b
          end
          r
        }
        #報告者権限で、ログインユーザのカスタムカテゴリが対象ホストのカスタムカテゴリから外れる場合はエラーとする
        if !@role_admin && s_categories.size > 1 && filtered_cats.size == 1 then
          #元データにシステムカテゴリが1つとカスタムカテゴリが1つ以上あり、
          #カスタムカテゴリが全てログインユーザのカスタムカテゴリから外れている場合
          @notices << "hostinfo.csvの#{session[:line_num]}行目: ログインユーザのカスタムカテゴリの範囲外になるため、登録されません。"
          next
        end
        is_multiple_system_categories = false
        notice = ""
        if i_categories.size > 1 && s_cat_names.size > 1 then
            is_multiple_system_categories = true
            notice = "hostinfo.csvの#{session[:line_num]}行目: システムカテゴリの登録は1つのみ可能なため、ホストのシステムカテゴリは変更されません。"
        end
        if (i_categories - s_categories).size != 0 or
            (s_categories - i_categories).size != 0 then
          if is_multiple_system_categories then
            notice =  "hostinfo.csvの#{session[:line_num]}行目: システムカテゴリの登録は1つのみ可能なため、システムカテゴリは「#{s_cat_names[0]}」が登録されます。"
          end
          target << {
            :action => 'category_change',
            :action_name => 'カテゴリ変更',
            :old => s_categories,
            :new => i_categories,
          }
        end
        if is_multiple_system_categories then
          @notices << notice
        end
        # ホスト名比較
        if item[:hostname] != t_host.hostname then
          if same_hostname and same_hostname['sidfmd_host'] and same_hostname['sidfmd_host']['host_apikey'] != item[:token] and !same_hostname['flg_delete'] then
            msg = "hostinfo.csvの#{session[:line_num]}行目: 既存の削除されていないホストと同じホスト名は使用できません。"
            @errs << msg
            next 
          else  
            target << {
              :action => 'rename_host',
              :action_name => 'ホスト名変更',
              :old => t_host.hostname,
              :new => item[:hostname],
            }
          end
        end
        # 表示名比較
        old_extname = t_host.ext_name ? t_host.ext_name : ''
        new_extname = item[:ext_name] ? item[:ext_name]  : ''
        if old_extname != new_extname then
          target << {
            :action => 'rename_extname',
            :action_name => '表示名変更',
            :old => old_extname,
            :new => new_extname,
          }
        end
        # 基準日比較
        i_refdate = item[:refdate].nil? ? '' : item[:refdate]
        s_refdate = t_host.refdate
        if s_refdate then
          s_refdate = s_refdate.strftime('%Y/%m/%d')
        else
          s_refdate = ''
        end
        ev_i_refdate = ''
        if i_refdate.size > 0 then
          begin
            ev_i_refdate = Date.parse(i_refdate)
            if ev_i_refdate then
              ev_i_refdate = ev_i_refdate.strftime('%Y/%m/%d')
            end
          rescue Exception => e
            log_exception(e)
          end
        end
        if ev_i_refdate != s_refdate then
          target << {
            :action =>'refdate_change',
            :action_name => '基準日変更',
            :old => s_refdate,
            :new => i_refdate,
          }
        end
        # メモ比較
        i_memo = item[:memo]
        s_memo = t_host.memo
        if (item[:memo] ? item[:memo] : '') != (t_host.memo ? t_host.memo : '') then
          if i_memo and i_memo.size > SidfmdHost::MEMO_MAX_LEN then
            @errs << "hostinfo.csvの#{session[:line_num]}行目:「メモ」の長さが最大値の#{SidfmdHost::MEMO_MAX_LEN}文字を超えています。"
            next
          end
          target << {
            :action => 'memo_change',
            :action_name => 'メモ変更',
            :old => s_memo,
            :new => i_memo,
          }
        end

        host_opts = host_opts_info[t_host.id.to_s]
        if item[:is_agent].downcase == 'false' && t_host.is_agent then
           @notices << "hostinfo.csvの#{session[:line_num]}行目: 「Agentフラグ」列が \"FALSE \"のため、現在登録されている「queue_packages」「queue_apps」「queue_dist」の情報は、次の脆弱性同期のタイミングでホストの情報として反映されなくなります。"
        end
        if host_opts then
          host_opts = host_opts.symbolize_keys
          if item[:is_agent].downcase == 'false' && ((host_opts[:packages] && host_opts[:packages].size > 0) || (host_opts[:apps] && host_opts[:apps].size > 0) || (host_opts[:dist] && host_opts[:dist].size > 0) || (host_opts[:queue_win_apps] && host_opts[:queue_win_apps].size > 0)) then
            @notices << "hostinfo.csvの#{session[:line_num]}行目:「Agentフラグ」列が \"FALSE \"のホストには、optsフォルダ下の「packages」「queue_apps」「queue_dist」「queue_win_apps」のデータは登録できません。※脆弱性同期時に自動生成されます。"
            new_pkgs = []
            new_apps = []
            new_win_apps = []
            new_dist = ''
          else
            new_pkgs = host_opts[:packages].uniq
            new_apps = host_opts[:queue_apps].uniq
            new_win_apps = host_opts[:queue_win_apps].uniq
            new_dist = host_opts[:queue_dist] ? host_opts[:queue_dist] : ''
          end
          new_products = host_opts[:manual_products].uniq
          new_lproducts = host_opts[:local_products].uniq
        else
          new_pkgs = []
          new_apps = []
          new_win_apps = []
          new_products = []
          new_lproducts = []
          new_dist = ''
        end
        if item[:is_agent].downcase == 'false' && t_host.is_agent then
          is_os_prod = false
          os_pids = []
          SidfmdProduct.where(:category => 'OS').each do |p|
            if new_products.include?(p.name) then
              is_os_prod = true
              break
            end
            os_pids << p.id
          end
          SidfmdVersion.where(:sidfmd_product_id => os_pids).each do |v|
            if new_products.include?(v.fullname) then
              is_os_prod = true
              break
            end
          end
          unless is_os_prod then
            @notices << "hostinfo.csvの#{session[:line_num]}行目: OSのプロダクトが登録されないため、ホストのOSは出力されません。"
          end
        end
        if t_host.is_agent == false && item[:is_agent].downcase == 'true' && new_pkgs.size == 0 && new_apps.size == 0 && new_dist.size == 0 then
          @notices << "hostinfo.csvの#{session[:line_num]}行目: 「queue_packages」「queue_apps」「queue_dist」の情報が入力されておりません。"
        end
        queue = t_host.get_last_queue
        if queue && item[:is_agent].downcase == 'true'then
          # パッケージ比較
          s_pkgs = queue.sidfmd_queue_pkgs.map{|pkg| pkg.pkg}
          if (new_pkgs - s_pkgs).size != 0 ||
             (s_pkgs - new_pkgs).size != 0
          then
            target << {
              :action => 'packages_change',
              :action_name => 'パッケージ変更',
              :old => s_pkgs,
              :new => new_pkgs,
            }
          end
          # 最新同期アプリケーション比較
          s_apps = queue.sidfmd_queue_apps.map{|app| "#{app.name}:#{app.version}:#{app.path}"}
          queue_win = queue.sidfmd_queue_win
          if queue_win then
            win_apps = queue_win.sidfmd_queue_win_apps
            if win_apps && win_apps.exists? then
              s_apps += win_apps.map{|w_app| encode_win_app(w_app)}
            end
          end
          if new_apps.size == 0 || new_win_apps.size == 0 then # 事前にsidfmd_queue_appsか、sidfmd_queue_win_appsしか登録できないように処理済み
            new_apps += new_win_apps
            if (new_apps - s_apps).size != 0 ||
               (s_apps - new_apps).size != 0
            then
              apps_change_data = {
                :action => 'apps_change',
                :action_name => '最新同期アプリケーション変更',
                :old => s_apps,
                :new => new_apps,
              }
              if new_win_apps.size > 0 then
                apps_change_data[:is_win_app] = true
              end
              target << apps_change_data
            end
          end
          # 最新同期OS情報比較
          dist = queue.sidfmd_queue_dist
          if dist then
            dist = "#{dist.ident}:#{dist.version}:#{dist.name}"
          else
             dist = ''
          end
          if dist != new_dist then
            target << {
              :action => 'dist_change',
              :action_name => '最新同期OS変更',
              :old => dist,
              :new => new_dist,
            }
          end
        else
          #queueが無い場合
          if new_pkgs.size > 0 then
            target << {
              :action => 'packages_change',
              :action_name => 'パッケージ変更',
              :old => [],
              :new => new_pkgs,
            }
          end
          if new_apps.size > 0 then
            target << {
              :action => 'apps_change',
              :action_name => '最新同期アプリケーション変更',
              :old => [],
              :new => new_apps,
            }
          end
          if new_dist.size > 0 then
            target << {
              :action => 'dist_change',
              :action_name => '最新同期OS変更',
              :old => '',
              :new => new_dist,
            }
          end
        end
        # 製品比較
        if (new_products.size + new_lproducts.size) > SidfmdHost::MAX_REGISTER_PRODUCTS then
          @notices << "hostinfo.csvの#{session[:line_num]}行目: プロダクトの最大登録数は#{SidfmdHost::MAX_REGISTER_PRODUCTS}個のため、このホストのプロダクトの変更は行われません。optsフォルダ以下のファイルをご確認下さい。"
        else
          same_token_products = []
          same_token_lproducts = []
          prod_ids = []
          ver_ids = []
          lprod_ids = []
          lver_ids = []
          is_sidfmd_products = false
          t_host.sidfmd_host_apps.each do |app|
            if app.version_id && app.version_id != 0 then
              ver_ids << app.version_id
            else
              prod_ids << app.product_id
            end
          end
          SidfmdVersion.where(:vid => ver_ids).each do |ver|
            same_token_products << ver.fullname
          end
          SidfmdProduct.where(:pid => prod_ids).each do |prd|
            same_token_products << prd.name
          end
          t_host.sidfmd_host_lapps.each do |lapp|
            if lapp.sidfmd_lversion_id == 0 then
              lprod_ids << lapp.sidfmd_lproduct_id
            else
              lver_ids << lapp.sidfmd_lversion_id
            end
          end
          SidfmdLproduct.where(:id => lprod_ids).each do |lprd|
            same_token_lproducts << lprd.name
          end
          SidfmdLversion.where(:id => lver_ids).each do |lver|
            same_token_lproducts << lver.fullname
          end
          same_token_products.compact!
          same_token_products.uniq!
          same_token_lproducts.compact!
          same_token_lproducts.uniq!
          if (new_products - same_token_products).size != 0 or
             (same_token_products - new_products).size != 0 or
             (new_lproducts - same_token_lproducts).size != 0 or
             (same_token_lproducts - new_lproducts).size != 0
          then
            is_sidfmd_products = true if new_products.size > 0 
            target << {
              :action => 'product_change',
              :action_name => '製品変更',
              :old => same_token_products + same_token_lproducts,
              :new => new_products + new_lproducts,
              :new_manual_products => new_products,
              :new_local_products => new_lproducts
            }
          end
        end

        #is_agent比較
        i_is_agent = item[:is_agent].downcase
        s_is_agent = t_host.is_agent.to_s.downcase
        if i_is_agent != s_is_agent then
          if i_is_agent == 'false' && s_is_agent == 'true' && 
            !is_sidfmd_products && t_host.sidfmd_host_apps.size == 0 
          then
            if target.size > 0 then
              @notices << "hostinfo.csvの#{session[:line_num]}行目: Agentフラグを \"TRUE \"から \"FALSE \"にする場合は、ローカルプロダクトではない製品の登録が必要なため、Agentフラグは変更されません。"
            else
              @notices = @notices.reject{|n| n.index("hostinfo.csvの#{session[:line_num]}")}
              @errs << "hostinfo.csvの#{session[:line_num]}行目: Agentフラグを \"TRUE \"から \"FALSE \"にする場合は、ローカルプロダクトではない製品の登録が必要です。"
            end
          else
            target << {
              :action => 'agent_change',
              :action_name => 'Agentフラグ変更',
              :old => s_is_agent,
              :new => i_is_agent
            }
          end
        end

        if target.size > 0 then
          if (item[:flg_delete] && item[:flg_delete].downcase == 'true') then
            #@errs << "hostinfo.csvの#{session[:line_num]}行目: 削除されているホストの設定を変更することはできません。"
            next
          end
          @actions << {
            :action => 'modify_host',
            :action_name => '設定変更',
            :old_data => same_token,
            :new_data => item,
            :options => target,
          }
        end
      end
    end

    session[:import_actions] = session[:import_actions] + @actions
    session[:import_errs] = session[:import_errs] + @errs
    session[:import_notices] = hostname_errs.size > 0 ? hostname_errs : session[:import_notices] + @notices
    session[:is_new_import] = true

    is_change = true
    #変更対象のデータが全てエラーになる場合は、index_rでエラーメッセージ表示
    if session[:import_actions].size == 0 && (session[:import_notices].size > 0 || session[:import_errs].size > 0) && executed_size == 0 then
      #:noticeに入れると、パラメーター制限値を超えてしまう可能性があるため、sessionに保存。
      if !is_err then
        session[:import_notices] = ["変更が無いため、インポートを終了しました。", "※インポートしたファイルにエラーがあります。"] + session[:import_errs] + session[:import_notices]
      end
      is_change = false
    elsif session[:import_actions].size == 0 && executed_size == 0 then
      #変更もエラーもない場合
      if !is_err then
        session[:import_notices] = ['変更が無いため、インポートを終了しました。']
      end
      is_change = false
    elsif executed_size == 0 then
      # hostlimを超過するカテゴリがあるか否かをチェック
      host_lines = session[:import_hostnames]
      hostlim_over_categories_data = get_import_hostlim_over_categories(session[:import_actions])
      hostname_categoryids = hostlim_over_categories_data[:hostname_categoryids]
      hostlim_over_categories = hostlim_over_categories_data[:hostlim_over_categories]

      if hostlim_over_categories.any? then
        hostlim_over_category_ids = []
        hostlim_over_category_names = {}
        hostlim_over_categories.each do |cat|
          hostlim_over_category_ids << cat.id
          hostlim_over_category_names[cat.id] = cat.name
        end

        hostname_categoryids.each do |hostname, category_ids|
          hostlim_over_cat_ids = category_ids & hostlim_over_category_ids
          if hostlim_over_cat_ids.any? then
            session[:import_notices] << "hostinfo.csvの#{host_lines[hostname]}行目: 次のカテゴリは、インポートによってホストの登録数が上限を超過します。 #{hostlim_over_cat_ids.map{|cat_id| hostlim_over_category_names[cat_id]}.join(', ')}"
            is_err = true
          end
        end
      end
      # ライセンスチェック
      create_undelete_count = @actions.count {|x| [ 'create_new_host', 'undelete_host' ].include?(x[:action]) }
      delete_count = @actions.count {|x| 'remove_host' == x[:action] }
      create_count = create_undelete_count - delete_count
      if create_count > 0 then
        # ライセンス追加がある場合は、ライセンス数の確認を行う。
        if @hosts and @hosts[:license_number] - @hosts[:used_license_number] < create_count then
          if !is_err then
            session[:import_notices] = ['ライセンス数を超えるため、インポートを実行することができませんでした。']
          end
          is_change = false
        end
      end
    end
    #logger.info(session[:import_actions].inspect)
  
    render :layout => false, :json => {:err => is_err, :executed_size => executed_size, :is_change => is_change}
  end

  def new_import_apply
    f_params = params.permit(:action_data, :is_exec_synchro_hosts, :process_rules, :delete_host_ids, :change_category_hostnames)
    err_message = ''
    if !f_params[:action_data] || !f_params[:is_exec_synchro_hosts] then
      err_message = 'パラメータが不正です。'
    end
    begin
      action = JSON.parse(f_params[:action_data], :symbolize_names => true)
    rescue JSON::ParserError => e
      log_exception(e)
      err_message = 'パラメータが不正です。'
    end

    if f_params[:process_rules] then #ループでの処理の一番最初だけ実行(=最初にルールを処理する)
      begin
        delete_host_ids = JSON.parse(f_params[:delete_host_ids])
        change_category_hostnames = JSON.parse(f_params[:change_category_hostnames]) #これはホスト名(key)とカテゴリ名の配列(value)
        change_category_hostnames.each do |hostname, category_names|
          change_category_hostnames[hostname] = SidfmdCategory.where(:name => category_names).where(:is_system => false).pluck(:id)
        end
        change_category_host_ids_data = SidfmdHost.get_change_category_ids(@role_admin, @user_category_ids, change_category_hostnames)
        if change_category_host_ids_data[:err] then
          render :layout => false, :json => { :rules_error => true } and return
        end
      rescue Exception => e
        log_exception(e)
        render :layout => false, :json => { :rules_error => true } and return
      end
      if err_message.size == 0 then
        begin
          SidfmdRule.transaction do
            process_rules = SidfmdHost.process_related_rules(
              :role_admin => @role_admin,
              :user_category_ids => @user_category_ids,
              :delete_host_ids => delete_host_ids,
              :change_category_host_ids => change_category_host_ids_data[:change_category_ids],
              :action => 'import'
            )
            if !process_rules then
              render :layout => false, :json => { :rules_error => true } and return
            end
          end
        rescue Exception => e
          render :layout => false, :json => { :rules_error => true } and return
        end
      end
    end

    is_exec_synchro_hosts = f_params[:is_exec_synchro_hosts].to_s
    if err_message.size > 0 then
      render :layout => false, :json => { :err_message => err_message  } and return
    end
    host_ids_for_view = []
    target_html_ids = []
    is_remove_host = false #remove_hostかmodify_categoryの時に関連するルールをチェックする
    is_mod_cat_host = false
    case action[:action]
    when 'create_new_host' then # 新規登録
      all_categories = {}
      all_system_categories ={}
      SidfmdCategory.all.each do |c|
        all_categories[c.name] = c.id
        all_system_categories[c.id] = c.name if c.is_system
      end
      user_c_cat_ids = (@role_admin ? all_categories.values : @user_category_ids)
      if !@role_admin then
        user_cat_names = SidfmdCategory.where(:id => user_c_cat_ids).where(:is_system => false).pluck(:name)
        if (user_cat_names - action[:new_data][:categories]).size == user_cat_names.size then
          err_message = "不正な操作のため、「#{action[:new_data][:hostname]}」の登録はされませんでした。"
          render :layout => false, :json => { :err_message => err_message  } and return
        end
      end

      if action[:new_data][:ext_name] and action[:new_data][:ext_name] =~ /&&&&&/ then
        err_message = "不正な操作のため、「#{action[:new_data][:hostname]}」の登録はされませんでした。"
        render :layout => false, :json => { :err_message => err_message  } and return
      end

      #hostlim_over_categories = SidfmdCategoriesHost.hostlim_over_categories(
      #  { action[:new_data][:new_host_id] => SidfmdCategory.where(:name => action[:new_data][:categories]).pluck(:id) }
      #)
      #if hostlim_over_categories.any? then
      #  err_message = "「#{action[:new_data][:hostname]}」は、指定されたカテゴリのうち、次のカテゴリがホスト登録数の上限を超えるため、登録されませんでした。 #{hostlim_over_categories.map{|cat| cat.name}.join(', ')}"
      #  render :layout => false, :json => { :err_message => err_message  } and return
      #end

      # ライセンスチェック
      lmhosts = SidfmdHost.get_hosts_by_lm(:limit => 0)
      if lmhosts and lmhosts[:license_number] <= lmhosts[:used_license_number] then
        logger.info('license number is upper limit')
        err_message = 'ホスト数がライセンスの上限に達しています。'
        render :layout => false, :json => { :err_message => err_message  } and return
      end
      
      is_hostname_correct = check_hostname(action[:new_data][:hostname])
      if is_hostname_correct != true then
        logger.info('hostname is invalid')   
        err_message = "「#{action[:new_data][:hostname]}」のホスト名は使用できないため、「#{action[:new_data][:hostname]}」の登録に失敗しました。"
        render :layout => false, :json => { :err_message => err_message  } and return 
      end

      if !@role_admin && SidfmdHost.find_by(:hostname => action[:new_data][:hostname]) then
        err_message = "「#{action[:new_data][:hostname]}」のホスト名は既に使用されているため、「#{action[:new_data][:hostname]}」の登録に失敗しました。"
        render :layout => false, :json => { :err_message => err_message  } and return
      end

      # ライセンスサーバとVMでホストの登録を行う。
      temp = SidfmdHost.register_host(action[:new_data][:hostname], nil, false)
      if temp.nil? then
        logger.info('failed to register host')
        err_message = "「#{action[:new_data][:hostname]}」の登録に失敗しました。"
        render :layout => false, :json => { :err_message => err_message  } and return
      end
      
    
      # 作成されたホストを取得
      host = SidfmdHost.where(:hostname => action[:new_data][:hostname]).first
      unless host then
        logger.info('failed to register host')
        err_message = "「#{action[:new_data][:hostname]}」の登録に失敗しました。"
        render :layout => false, :json => { :err_message => err_message  } and return
      end
     
      SidfmdHost.transaction do
        begin  
          host.lock!
          #Agentフラグ
          host.update_attribute :is_agent, (action[:new_data][:is_agent].to_s.downcase == 'true' ? true : false)

          #表示名登録
          host.update_attribute :ext_name, action[:new_data][:ext_name]
      
          # カテゴリ登録
          if action[:new_data][:categories] then
            is_s_cat_registered = false
            action[:new_data][:categories].each do |category_name|
              cat_id = all_categories[category_name]
              if cat_id then
                if all_system_categories[cat_id] then
                  if !is_s_cat_registered then
                    #公開サーバがデフォルトで登録されるため、システムカテゴリが２つにならないように、削除してから登録する
                    SidfmdCategoriesHost.where(:sidfmd_host_id => host.id)
                                        .where(:sidfmd_category_id => all_system_categories.keys)
                                        .destroy_all
                    SidfmdCategoriesHost.create(:sidfmd_host_id => host.id,:sidfmd_category_id => cat_id)
                    is_s_cat_registered = true
                  end
                else
                  if user_c_cat_ids.include?(cat_id) then
                    SidfmdCategoriesHost.create(:sidfmd_host_id => host.id,:sidfmd_category_id => cat_id)
                  end
                end
              end
            end
          end
         
          #基準日登録
          refdate = action[:new_data][:refdate]
          if refdate then
            begin
              refdate = Date.parse(refdate)
            rescue ArgumentError => e
              log_exception(e)
              raise Exception
            end
          else
            refdate = Date.today
          end
          host.update_attribute :refdate, refdate
          # メモ登録
          if action[:new_data][:memo] and action[:new_data][:memo].size > SidfmdHost::MEMO_MAX_LEN then
            logger.info('memo is over max length')
            raise Exception
          end
          host.update_attribute :memo, action[:new_data][:memo]

          #削除フラグ
          #host.update_attribute :flg_delete, (action[:new_data][:flg_delete].to_s.downcase == 'true' ? true : false)

          host.save
        rescue Exception => e
          log_exception(e)
          err_message = "「#{host.hostname}」に関連する情報の登録に失敗しました。※ホストの登録は正常に行われました。"  
          Rails.cache.clear
          render :layout => false, :json => { :err_message => err_message  } and return
        end
      end
      #パッケージ・最新同期アプリケーション・最新同期OS情報登録
      regist_queue_info = regist_host_queue_opts(host.id, action[:new_data][:packages], action[:new_data][:apps], action[:new_data][:dist], [])
      if !regist_queue_info then
        err_message = "「#{host.hostname}」の付属情報の登録に失敗しました。※ホストの登録は正常に行われました。"
        Rails.cache.clear
        render :layout => false, :json => { :err_message => err_message  } and return
      end
      #プロダクト登録
      host.sidfmd_host_apps = []
      host.sidfmd_host_lapps = []
      host.save
      manual_prds = action[:new_data][:manual_products] ? action[:new_data][:manual_products] : []
      local_prds = action[:new_data][:local_products] ? action[:new_data][:local_products] : []
      if (manual_prds.size > 0 || local_prds.size > 0) && (manual_prds.size + local_prds.size) <= SidfmdHost::MAX_REGISTER_PRODUCTS then
        begin 
          regist_products(manual_prds, local_prds, host.id)
        rescue Exception => e
          log_exception(e)
          err_message = "「#{host.hostname}」のプロダクトの登録に失敗しました。※ホストの登録は正常に行われました。"
          Rails.cache.clear
          render :layout => false, :json => { :err_message => err_message  } and return
        end
      end
      SidfmdHostView.sync_data([host.id])
    when 'undelete_host' then # 復元（削除状態のものを復元する）
      host_id = action[:old_data][:id].to_i
      hostname = action[:old_data][:hostname]
      sidfmd_hostid = (action[:old_data][:sidfmd_host] ? action[:old_data][:sidfmd_host][:id] : nil)
      host = nil
      if !@role_admin && hostname then
        err_message = "「#{hostname}」の復元は、ログインユーザでは許可されていない操作です。"
        render :layout => false, :json => { :err_message => err_message  } and return
      end
      if action[:new_data][:ext_name] and action[:new_data][:ext_name] =~ /&&&&&/ then
        err_message = "不正な操作のため、「#{action[:new_data][:hostname]}」の登録はされませんでした。"
        render :layout => false, :json => { :err_message => err_message  } and return
      end
      if host_id > 0 and hostname and hostname.size > 0 and sidfmd_hostid then
        SidfmdHost.transaction do
          all_categories = {}
          all_system_categories ={}
          SidfmdCategory.all.each do |c|
            all_categories[c.name] = c.id
            all_system_categories[c.id] = c.name if c.is_system
          end
          res = SidfmdHost.toggle_host_delete(host_id)
          if res then
            if res[:success] then
              host = SidfmdHost.where(:id => sidfmd_hostid ).first
              if host then
                host.lock!
                host.update_attribute :flg_delete, false
              else
                res = SidfmdHost.establish_consistency
              end
            else
              err_message = "ライセンス数を超過するため、「#{hostname}」の復元に失敗しました。"
              render :layout => false, :json => { :err_message => err_message  } and return
            end
          else
            err_message = "ライセンスサーバとの通信に失敗したため、「#{hostname}」の復元に失敗しました。"
            render :layout => false, :json => { :err_message => err_message  } and return
          end
        end
      else
        err_message = "「#{hostname}」の復元に失敗しました。"
        render :layout => false, :json => { :err_message => err_message  } and return
      end
      # 復元されたホストが取得できなかった場合。
      unless host then
        host = SidfmdHost.where(:hostname => hostname).first
        unless host then
          err_message = "「#{hostname}」の復元に失敗しました。"
          render :layout => false, :json => { :err_message => err_message  } and return
        end
      end
      begin
        SidfmdHost.transaction do
          host.lock!
          #表示名更新
          host.update_attribute :ext_name, action[:new_data][:ext_name]
          # カテゴリ更新
          cat_names = action[:new_data][:categories]
          if cat_names and cat_names.size > 0 then
            c_category_ids = []
            system_category_id = nil
            cat_names.map{|c| c.strip}.uniq.each do |c_name|
              cat_id = all_categories[c_name]
              if cat_id then
                if all_system_categories[cat_id] then
                  if !system_category_id then
                    system_category_id = cat_id
                  end
                else
                  c_category_ids << cat_id
                end
              end
            end
            if system_category_id then
              res = host.modify_categories(system_category_id, c_category_ids)
              unless res then
                logger.info('SidfmdHost#modify_categories failure')
                raise Exception
                #raise StandardError, 'SidfmdHost#modify_categories failure'
              end
            end
          else
            SidfmdCategoriesHost.destroy_all(:sidfmd_host_id => host.id)
          end
          #is_agent更新
          host.update_attribute :is_agent, (action[:new_data][:is_agent].to_s.downcase == 'true' ? true : false)

          # 基準日更新
          refdate = action[:new_data][:refdate]
          if refdate then
            begin
              refdate = Date.parse(refdate)
            rescue ArgumentError => e
              log_exception(e)
              logger.info('ignore invalid refdate:' + e.inspect)
               refdate = nil
            end
          end
          host.update_attribute :refdate, refdate
          # メモ更新
          memo = action[:new_data][:memo]
          if memo and memo.size > SidfmdHost::MEMO_MAX_LEN then
            logger.info('memo is over max length')
            raise Exception
          elsif memo then
            host.update_attribute :memo, memo
          end
        end
        SidfmdHostView.sync_data([host.id])
      rescue Exception => e
        log_exception(e)
        Rails.cache.clear
        err_message = "「#{hostname}」に関連する情報の登録に失敗しました。※ホストの復元は正常に行われました。"
        render :layout => false, :json => { :err_message => err_message  } and return
      end
      #パッケージ・最新同期アプリケーション・最新同期OS情報登録
      regist_queue_info = regist_host_queue_opts(host.id, action[:new_data][:packages], action[:new_data][:apps], action[:new_data][:dist], [])
      if !regist_queue_info then
        Rails.cache.clear
        err_message = "「#{host.hostname}」の付属情報の登録に失敗しました。※ホストの登録は正常に行われました。"
        render :layout => false, :json => { :err_message => err_message  } and return
      end
      #プロダクト更新
      manual_prds = action[:new_data][:manual_products] ? action[:new_data][:manual_products] : []
      local_prds = action[:new_data][:local_products] ? action[:new_data][:local_products] : []
      if (manual_prds.size > 0 || local_prds.size > 0) && (manual_prds.size + local_prds.size) <= SidfmdHost::MAX_REGISTER_PRODUCTS then
        host.sidfmd_host_apps = []
        host.sidfmd_host_lapps = []
        host.save
        begin
          regist_products(manual_prds, local_prds, host.id)
        rescue Exception => e
          log_exception(e)
          Rails.cache.clear
          err_message = "「#{host.hostname}」のプロダクトの登録に失敗しました。※ホストの復元は正常に行われました。"
          render :layout => false, :json => { :err_message => err_message  } and return
        end
      else
        SidfmdHostApp.destroy_all(:sidfmd_host_id => host.id)
        SidfmdHostLapp.destroy_all(:sidfmd_host_id => host.id)
      end 
    when 'remove_host' then
      is_remove_host = true
      # ライセンスサーバとVMでホストの削除を行う。
      action[:old_data].each do |i|
        host_id = i[:host][:id]
        hostname = i[:host][:hostname]
        sidfmdhostid = i[:host][:sidfmd_host] ? i[:host][:sidfmd_host][:id] : nil
        if !@role_admin && hostname then
          if sidfmdhostid then
            t_host = SidfmdHost.find_by(:id => sidfmdhostid)
            if t_host then
              host_cat_ids = t_host.sidfmd_categories.pluck(:id)
              if (@user_category_ids - host_cat_ids).size == @user_category_ids.size then
                err_message = "ログインユーザでの操作は許可されていないため、「#{host.hostname}」の削除に失敗しました。"
                render :layout => false, :json => { :err_message => err_message  } and return
              end
            else
              err_message = "「#{host.hostname}」の削除に失敗しました。"
              render :layout => false, :json => { :err_message => err_message  } and return
            end
          else
            err_message = "ログインユーザでの操作は許可されていないため、「#{host.hostname}」の削除に失敗しました。"
            render :layout => false, :json => { :err_message => err_message  } and return
          end
        end

        if host_id and hostname and sidfmdhostid then
          t_html_ids = []
          t_host_id = nil
          begin
            SidfmdHost.transaction do
              host = nil
              if sidfmdhostid > 0 then
                host = SidfmdHost.find_by(:id => sidfmdhostid)
              else
                host = SidfmdHost.where(:hostname => hostname).first
              end
              raise Exception unless host

              res = SidfmdHost.toggle_host_delete(host_id)
              if res then
                if res[:success] then
                  if host then
                    t_host_id = host.id
                    t_html_ids = SidfmdHostView.relate_htmlids([ t_host_id ])
                    host.lock!
                    host.update_attribute :flg_delete, true
                    host.last_checked_at = nil
                    host.memo = nil
                    host.is_agent = false
                    host.save
             
                    SidfmdCategoriesHost.where(:sidfmd_host_id => sidfmdhostid).delete_all
                    SidfmdHostApp.where(:sidfmd_host_id => sidfmdhostid).delete_all
                    SidfmdHostLapp.where(:sidfmd_host_id => sidfmdhostid).delete_all
                    SidfmdHostDisabledApp.where(:sidfmd_host_id => sidfmdhostid).delete_all

                    sidfmd_queue_ids = SidfmdQueue.where(:sidfmd_host_id => sidfmdhostid).collect {|x| x.id }.uniq
                    sidfmd_queue_ids.each do |sidfmd_queue_id|
                      SidfmdQueueDist.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
                      SidfmdQueuePkg.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
                      SidfmdQueueApp.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
                      SidfmdQueueLapp.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
                      SidfmdQueueWinApp.where(:sidfmd_queue_win_id => SidfmdQueueWin.where(:sidfmd_queue_id => sidfmd_queue_id).select(:id)).delete_all
                      SidfmdQueueWin.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
                      sidfmd_queue_vul_ids = SidfmdQueueVul.where(:sidfmd_queue_id => sidfmd_queue_id).collect {|x| x.id }.uniq
                      sidfmd_queue_vul_ids.each do |sidfmd_queue_vul_id|
                        SidfmdQueueVulCve.where(:sidfmd_queue_vul_id => sidfmd_queue_vul_id).delete_all
                        SidfmdQueueVulSri.where(:sidfmd_queue_vul_id => sidfmd_queue_vul_id).delete_all
                        SidfmdQueueVulAffprod.where(:sidfmd_queue_vul_id => sidfmd_queue_vul_id).delete_all
                        SidfmdQueueVulAffver.where(:sidfmd_queue_vul_id => sidfmd_queue_vul_id).delete_all
                        SidfmdQueueVulAffpkg.where(:sidfmd_queue_vul_id => sidfmd_queue_vul_id).delete_all
                      end
                      SidfmdQueueVul.where(:sidfmd_queue_id => sidfmd_queue_id).delete_all
                    end
                    SidfmdQueue.where(:id => sidfmd_queue_ids).delete_all
  
                    sidfmd_issue_ids = SidfmdIssue.where(:sidfmd_host_id => sidfmdhostid).collect {|x| x.id }.uniq
                    sidfmd_issue_ids.each do |sidfmd_issue_id|
                      SidfmdIssueCve.where(:sidfmd_issue_id => sidfmd_issue_id).delete_all
                      SidfmdIssueSri.where(:sidfmd_issue_id => sidfmd_issue_id).delete_all
                      SidfmdIssueAffprod.where(:sidfmd_issue_id => sidfmd_issue_id).delete_all
                      SidfmdIssueAffver.where(:sidfmd_issue_id => sidfmd_issue_id).delete_all
                      SidfmdIssueAffpkg.where(:sidfmd_issue_id => sidfmd_issue_id).delete_all
                      SidfmdIssueCtag.where(:sidfmd_issue_id => sidfmd_issue_id).delete_all
                    end
                   SidfmdIssue.where(:id => sidfmd_issue_ids).delete_all
                  else
                    res = SidfmdHost.establish_consistency
                  end
                else
                  err_message = "「#{host.hostname}」の削除に失敗しました。"
                  render :layout => false, :json => { :err_message => err_message  } and return
                end
              else
                err_message = 'ライセンスサーバとの通信に失敗しました。' 
                render :layout => false, :json => { :err_message => err_message  } and return
              end
            end
          rescue Exception => e
            logger.error(e)
            err_message = "「#{hostname}」の削除に失敗しました。"
            render :layout => false, :json => { :err_message => err_message  } and return
          end

          SidfmdVulView.sync_data(t_html_ids) if t_html_ids.any?
          SidfmdHostView.sync_data([ t_host_id ]) if t_host_id 
        end
      end
    when 'modify_host' then
      host_id = action[:old_data][:id]
      hostname = action[:old_data][:hostname]
      sidfmd_hostid = (action[:old_data][:sidfmd_host] ? action[:old_data][:sidfmd_host][:id] : nil)
      unless sidfmd_hostid then
        err_message = "「#{hostname}」の登録情報の変更に失敗しました。"
      else
        host = SidfmdHost.where(:id => sidfmd_hostid).first
        if host then
          if !@role_admin then
            if (@user_category_ids - host.sidfmd_categories.pluck(:id)).size == @user_category_ids.size then
              err_message = "ログインユーザでの操作は許可されていないホストのため、「#{hostname}」の登録情報の変更に失敗しました。"
              render :layout => false, :json => { :err_message => err_message  } and return
            end
          end
          host_ids_for_view << host.id
          host_queue_data = {}
          option_err_messages = []
          action[:options].each do |option|
            case option[:action]
            when 'agent_change' then
              SidfmdHost.transaction do
                begin 
                  host.lock!
                  host.update_attribute :is_agent, (option[:new].to_s.downcase == 'true' ? true : false)
                rescue Exception => e
                  log_exception(e)
                  err_message = "「#{hostname}」のAgentフラグの変更に失敗しました。"
                  option_err_messages << err_message
                end
              end
            when 'rename_host' then
              new_name = option[:new]
              SidfmdHost.transaction do
                begin 
                  res = SidfmdHost.rename_host(host_id, new_name)
                  if res.respond_to?(:[]) then
                    if res[:success] then
                       host.lock!
                       host.update_attribute :hostname, new_name
                    else
                      err_message = "登録済のホスト名、もしくは不正な文字を指定したため、「#{hostname}」のホスト名変更に失敗しました。"
                      option_err_messages << err_message
                    end
                  else
                    err_message = "ライセンスサーバとの通信に失敗したため、「#{hostname}」のホスト名変更は行われませんでした。"
                    option_err_messages << err_message
                  end
                rescue Exception => e
                  log_exception(e)
                  err_message = "「#{host.hostname}」のホスト名変更に失敗しました。"
                  option_err_messages << err_message
                end
              end
            when 'rename_extname' then
              SidfmdHost.transaction do
                begin
                  host.lock!
                  new_extname = option[:new]
                  if new_extname =~ /&&&&&/ then
                    raise Exception
                  end
                  host.update_attribute :ext_name, (new_extname.size > 0 ? new_extname : nil)
                rescue Exception => e
                  log_exception(e)
                  err_message = "「#{host.hostname}」の表示名変更に失敗しました。"
                  option_err_messages << err_message
                end
              end
            when 'packages_change', 'apps_change', 'dist_change' then
              #後でまとめて処理
              if host then
                if option[:action] == 'apps_change' && option[:is_win_app] then
                  host_queue_data[:win_apps_change] = option[:new]
                else
                  host_queue_data[option[:action].to_sym] = option[:new]
                end
              end  
            when 'product_change' then
              # 製品変更
              SidfmdHost.transaction do
                begin
                  host.lock!
                  host.sidfmd_host_apps = []
                  host.sidfmd_host_lapps = []
                  host.save
                  manual_prds = option[:new_manual_products] 
                  local_prds = option[:new_local_products]
                  if (manual_prds.size > 0 || local_prds.size > 0) && (manual_prds.size + local_prds.size) <= SidfmdHost::MAX_REGISTER_PRODUCTS then
                    regist_products(manual_prds, local_prds, host.id)
                  end
                rescue Exception => e
                  log_exception(e)
                  err_message = "「#{host.hostname}」のプロダクト変更に失敗しました。"
                  option_err_messages << err_message
                end
              end
            when 'category_change' then 
              is_mod_cat_host = true
              # カテゴリ変更
              host = SidfmdHost.where(:hostname => hostname).first
              if host then
                new_categories = option[:new].map{|c_name| c_name.strip}.uniq
                begin
                  SidfmdCategoriesHost.transaction do
                    categories = []
                    unless @role_admin then
                      if @user_category_ids.any? then
                        unchangeable_category_ids = host.sidfmd_categories
                                                        .where(:is_system => false)
                                                        .where.not(:id => @user_category_ids)
                                                        .pluck(:id)
                        categories = SidfmdCategory.where(%|
                          name IN (?) OR id IN (?)
                        |, new_categories, unchangeable_category_ids)
                      end
                    else
                      categories = SidfmdCategory.where(:name => new_categories)
                    end
                    if categories.respond_to?(:where) and categories.where(:is_system => true).count == 1 then
                      #hostlim_over_categories = SidfmdCategoriesHost.hostlim_over_categories( { host.id => categories.pluck(:id)} )
                      #if hostlim_over_categories.any? then
                      #  err_message = "「#{host.hostname}」は、指定されたカテゴリのうち、次のカテゴリがホスト登録数の上限を超えるため、カテゴリの変更に失敗しました。 #{hostlim_over_categories.map{|cat| cat.name}.join(', ')}"
                      #  option_err_messages << err_message
                      #else
                      SidfmdIssueCtag.where(:sidfmd_issue_id =>
                          host.sidfmd_issues.select(:id)
                      ).destroy_all
                      host.sidfmd_categories = []
                      host.sidfmd_categories = categories.uniq
                      host.save
                      #end
                    else
                      raise Exception
                    end
                  end
                rescue Exception => e
                  log_exception(e)
                  err_message = "「#{host.hostname}」のカテゴリ変更に失敗しました。"
                  option_err_messages << err_message
                end
              else
                err_message = "「#{host.hostname}」のカテゴリ変更に失敗しました。"  
                option_err_messages << err_message
              end
            when 'refdate_change' then
              # 基準日変更
              SidfmdHost.transaction do
                begin
                  host.lock!
                  if option[:new] and option[:new].size > 0 then
                    host.update_attribute :refdate, Date.parse(option[:new])
                  else
                    host.update_attribute :refdate, nil
                  end
                rescue ArgumentError => e
                  log_exception(e)
                  logger.info('ignore change refdate: new_refdate=' + option[:new].inspect)
                  err_message = "「#{host.hostname}」の基準日変更に失敗しました。"
                  option_err_messages << err_message
                end
              end
            when 'memo_change' then 
              # メモ変更
              SidfmdHost.transaction do
                begin 
                  host.lock!
                  if option[:new] and option[:new].size > 0 then
                    if option[:new].size > SidfmdHost::MEMO_MAX_LEN then
                      logger.info("ignore change memo(over length): new_memo=#{option[:new].inspect}")
                      err_message = "「#{host.hostname}」のメモ変更に失敗しました。"
                      option_err_messages << err_message
                    else
                      host.update_attribute :memo, option[:new]
                    end
                  else
                    host.update_attribute :memo, nil
                  end
                rescue Exception => e
                  log_exception(e)
                  err_message = "「#{host.hostname}」のメモ変更に失敗しました。"
                  option_err_messages << err_message
                end
              end
            end
          end
          packages_data = host_queue_data[:packages_change]
          apps_data = host_queue_data[:apps_change]
          dist_data = host_queue_data[:dist_change]
          win_apps_data = host_queue_data[:win_apps_change]
          if packages_data || apps_data || dist_data || win_apps_data then
            queue = host.get_last_queue
            if queue then
              if !packages_data then
                packages_data = SidfmdQueuePkg.where(:sidfmd_queue_id => queue.id).pluck(:pkg)
              end
              if !apps_data then
                apps_data = SidfmdQueueApp.where(:sidfmd_queue_id => queue.id).map{|app| "#{app.name}:#{app.version}:#{app.path}"}
              end
              if !win_apps_data then
                win_apps = nil
                queue_win = queue.sidfmd_queue_win
                if queue_win then
                  win_apps = queue_win.sidfmd_queue_win_apps
                end
                if win_apps then
                  win_apps_data = win_apps.map{|w_app| "#{w_app.publisher}:#{w_app.name}:#{w_app.version}:#{w_app.uninstall}"}
                else
                  win_apps_data = []
                end
              end
              if !dist_data then
                dist = SidfmdQueueDist.where(:sidfmd_queue_id => queue.id).first
                if dist then
                  dist_data = "#{dist.ident}:#{dist.version}:#{dist.name}"
                end
              end
              regist_queue_info = regist_host_queue_opts(host.id, packages_data, apps_data, dist_data, win_apps_data)
              if !regist_queue_info then
                err_message = "「#{hostname}」の「packages」「queue_apps」「queue_dist」「queue_win_apps」の変更に失敗しました。"
                option_err_messages << err_message
              end
            else
              regist_queue_info = regist_host_queue_opts(host.id, packages_data, apps_data, dist_data, win_apps_data)
              if !regist_queue_info then
                err_message = "「#{hostname}」の「packages」「queue_apps」「queue_dist」「queue_win_apps」の変更に失敗しました。"
                option_err_messages << err_message
              end
            end
          else
            if action[:old_data][:sidfmd_host] && 
               action[:old_data][:sidfmd_host][:is_agent].to_s.downcase == 'false' && 
               action[:new_data][:is_agent] && action[:new_data][:is_agent].to_s.downcase == 'true'
            then
              SidfmdQueue.create(sidfmd_host_id: action[:old_data][:sidfmd_host][:id].to_i) 
            end
          end
          if is_exec_synchro_hosts == 'true' then
            SidfmdHost.transaction do
              begin
                host.lock!
                host.update_attribute :last_checked_at, nil
              rescue Exception => e
                log_exception(e)
              end
            end
          end
          SidfmdHostView.sync_data([host.id]) if host.sidfmd_categories.where(:is_system => true).any? && host.last_checked_at
          err_message = option_err_messages.join("\n")
        else
          err_message = "「#{hostname}」の登録情報の変更に失敗しました。"
        end
      end
    end
    if host_ids_for_view.any? then
      host_ids_for_view.uniq!
      target_html_ids |= SidfmdHostView.relate_htmlids(host_ids_for_view)
      SidfmdVulView.sync_data(target_html_ids)
      SidfmdHostView.sync_data(host_ids_for_view)
    end
    Rails.cache.clear
    render :layout => false, :json => { :err_message => err_message }
  end

  CSV_BOM = "\xff\xfe".force_encoding('UTF-16LE').freeze

  def new_export
    f_params = params.permit(:checked, :process, :executed_size)
    process = f_params[:process]
    err = false
    message = ''
    r = {}
    if process == 'start_export' then
      begin
        checked = JSON.parse(f_params[:checked])
      rescue JSON::ParserError => e
        log_exception(e)
        err = true
        message = '不正な操作です。'
      end
      hosts = nil
      begin
        hosts = SidfmdHost.get_hosts_by_lm(:'sort[hostname]' => :asc)
      rescue Exception => e
        log_exception(e)
        err = true
        message = 'ライセンスサーバへのアクセスに失敗しました。'
      end
      unless hosts then
        err = true
        message = 'データ取得に失敗しました。'
      end
      if hosts and hosts[:success] then
        hosts = hosts[:hosts]
        if hosts.nil? then
          err = true
          message = 'データ取得に失敗しました。'
        end
      end
      if Sidfm.deleted_host_display then
        hosts.delete_if{|x| x[:flg_delete] == true }
      end
      if checked.size > 0 then
        hosts = hosts.select{|d| checked.include?(d[:hostname])}
      end
      clear_export_session
      session[:export_hosts] = hosts
      r[:total_hosts] = hosts.size
      r[:err] = err
      r[:message] = message
      render :layout => false, :json => r
    elsif process == 'progress_export' then
      hosts = session[:export_hosts] 
      if !hosts then
        err = true
        message = 'データ取得に失敗しました。'
      else
        begin
          executed_size = f_params[:executed_size].to_i
          t_hosts = hosts.slice(executed_size, 50)
          t_hosts = [] unless t_hosts
          executed_hosts = t_hosts.size
          session[:export_hosts_lines] = [] unless session[:export_hosts_lines]
          session[:export_opts_lines] = {} unless session[:export_opts_lines]
          t_hosts.each do |host|
            sidfmd_host_id = (host['sidfmd_host'] ? host['sidfmd_host']['id'] : nil)
            if sidfmd_host_id then
              sidfmd_host = SidfmdHost.find_by(:id => sidfmd_host_id)
              if sidfmd_host then
                if !@role_admin && ((@user_category_ids - sidfmd_host.sidfmd_categories.pluck(:id)).size == @user_category_ids.size || sidfmd_host.flg_delete) then
                  sidfmd_host = nil
                end
              end
            else
              sidfmd_host = nil
            end
            csv_data = create_new_export_hosts_info(host, sidfmd_host)
            session[:export_hosts_lines] << csv_data if csv_data.size > 0
            if sidfmd_host then
              opts = []
              if sidfmd_host.is_agent then
                pkgs_info = create_host_pkgs_info_for_new_export(sidfmd_host)
                pkgs_info.each do |pkg_info|
                  opts << pkg_info
                end
              end
              manual_prods_info = create_host_manual_products_info_for_new_export(sidfmd_host)
              manual_prods_info.each do |prod_info|
                opts << prod_info
              end
              local_prods_info = create_host_local_products_info_for_new_export(sidfmd_host)
              local_prods_info.each do |lprod_info|
                opts << lprod_info
              end
              if sidfmd_host.is_agent then
                apps_info = create_host_apps_info_for_new_export(sidfmd_host)
                apps_info.each do |app_info|
                  opts << app_info
                end
                os_info = create_host_os_info_for_new_export(sidfmd_host)
                opts << os_info if os_info.size > 0
              end
              session[:export_opts_lines][sidfmd_host.id] = opts
            end
          end  
          r[:executed_hosts] = executed_hosts
        rescue Exception => e
          log_exception(e)
          err = true
          message = 'データの処理に失敗しました。'
        end
      end
      r[:err] = err
      r[:message] = message
      render :layout => false, :json => r
    else
      hostinfo_header = [
        'ホスト登録ID'.encode('UTF-16LE'),
        'ホスト削除'.encode('UTF-16LE'),
        'ホストID'.encode('UTF-16LE'),
        'トークン'.encode('UTF-16LE'),
        'ホスト名'.encode('UTF-16LE'),
        '表示名'.encode('UTF-16LE'),
        'カテゴリ'.encode('UTF-16LE'),
        '基準日'.encode('UTF-16LE'),
        '削除'.encode('UTF-16LE'),
        'Agentフラグ'.encode('UTF-16LE'),
        'メモ'.encode('UTF-16LE')
      ]
      opt_header = [
        'ホストID'.encode('UTF-16LE'),
        '種類'.encode('UTF-16LE'),
        'データ'.encode('UTF-16LE')
      ]
      new_host_opt_header = [
        'ホスト登録ID'.encode('UTF-16LE'),
        '種類'.encode('UTF-16LE'),
        'データ'.encode('UTF-16LE')
      ]
      sorted_opts_host_ids = session[:export_opts_lines].keys.sort() 
      Tempfile.create('temp') do |f|
        path = f.path
        Zip::File.open(path, Zip::File::CREATE) do |zipfile|
          zipfile.get_output_stream("hostinfo.csv") do |f|
            f.puts(
              CSV.generate("#{CSV_BOM}", :col_sep => "\t".encode('UTF-16LE')) do |o|
                o << hostinfo_header
                session[:export_hosts_lines].each do |line|
                  o << line.map{|item| item.encode('UTF-16LE')}
                end
              end
            )
          end
          sorted_opts_host_ids.each do |host_id|
            zipfile.get_output_stream("regist/1.csv") do |f|
              f.puts(
                CSV.generate("#{CSV_BOM}", :col_sep => "\t".encode('UTF-16LE')) do |o|
                  o << new_host_opt_header  #新規ホストの付属情報登録用のファイル
                end
              )
            end
            zipfile.get_output_stream("opt/#{host_id}.csv") do |f|
              f.puts(
                CSV.generate("#{CSV_BOM}", :col_sep => "\t".encode('UTF-16LE')) do |o|
                  o << opt_header
                  host_opts_lines = session[:export_opts_lines][host_id]
                  host_opts_lines.each do |line|
                    o << line.map{|item| item.encode('UTF-16LE')}
                  end
                end
              )
            end
          end
        end
        send_data(File.read(path), :filename => "vm-export-#{Time.now.strftime('%Y%m%d%H%M%S')}.zip", :disposition => 'attachment')
        cookies['export_flg'] = true
        clear_export_session
      end
    end
  end

  #プロダクトマスタ情報のダウンロードを行う
  def download_products
    begin
      header = [
        '種類'.encode('UTF-16LE'),
        'サポート終了'.encode('UTF-16LE'),
        'プロダクトID'.encode('UTF-16LE'),
        'バージョンID'.encode('UTF-16LE'),
        'プロダクト名'.encode('UTF-16LE'),
        'バージョン名'.encode('UTF-16LE')
      ]
      csv = CSV.generate("#{CSV_BOM}", :col_sep => "\t".encode('UTF-16LE')) do |o|
        o << header
        SidfmdProduct.order(:name).each do |prd|
          pid = prd.pid
          name = prd.name
          o << [
            'sidfm'.encode('UTF-16LE'),
            prd.is_support_end.to_s.upcase.encode('UTF-16LE'),
            pid.to_s.encode('UTF-16LE'),
            0.to_s.encode('UTF-16LE'),
            name.encode('UTF-16LE'),
            ''.encode('UTF-16LE')
          ]
          SidfmdVersion.where(:sidfmd_product_id => prd.id).order(:version).each do |ver|
            o << [
              'sidfm'.encode('UTF-16LE'),
               ver.is_support_end.to_s.upcase.encode('UTF-16LE'),
               pid.to_s.encode('UTF-16LE'),
               ver.vid.to_s.encode('UTF-16LE'), 
               name.encode('UTF-16LE'),
               ver.fullname.encode('UTF-16LE')
            ]
          end
        end
        SidfmdLproduct.order(:priority).each do |lprd|
          lpid = lprd.id
          name = lprd.name
          o << [
            'local'.encode('UTF-16LE'),
            lprd.is_support_end.to_s.encode('UTF-16LE'),
            lpid.to_s.encode('UTF-16LE'),
            0.to_s.encode('UTF-16LE'),
            name.encode('UTF-16LE'),
            ''.to_s.encode('UTF-16LE')
          ]
          SidfmdLversion.where(:sidfmd_lproduct_id => lpid).order(:priority).each do |lver|
            o << [
              'local'.encode('UTF-16LE'),
              (lprd.is_support_end ? true : lver.is_support_end).to_s.encode('UTF-16LE'),
              lpid.to_s.encode('UTF-16LE'),
              lver.id.to_s.encode('UTF-16LE'),
              name.encode('UTF-16LE'),
              lver.fullname.encode('UTF-16LE')
            ]
          end
        end
      end
    rescue Exception => e
      logger.info(e)
      redirect_to :action => :index_r, :notice => 'データ取得に失敗しました。' and return
    end
    send_data(csv, :filename => 'vm-products-master.csv', :disposition => 'attachment')
  end

  ### 以下、内部API

  # アプリケーション名の一部からアプリケーション情報を返す。
  def api_substr_search
    # params[keyword]
    f_params = params.fetch(:params, {}).permit(:keyword)
    keyword = f_params[:keyword]
    ret = {}
    if keyword and keyword.size > 1 then
      products = SidfmdProduct.name_matched(keyword).order(:category)
      products.each do |product|
        ret[product.pid] = { 'product' => product }
      end
      versions = SidfmdVersion.fullname_matched(keyword).order(:position => :desc)
      versions.each do |version|
        product = version.sidfmd_product
        if !ret.has_key?(product.pid) then
          ret[product.pid] = { 'product' => product }
        end
        ret[product.pid]['versions'] = [] unless ret[product.pid].has_key?('versions')
        ret[product.pid]['versions'] << version
      end
      #ローカルプロダクト
      temp_lproducts_info = {}

      lproducts = SidfmdLproduct.name_matched(keyword).where(:flg_enable => true).order(:priority)
      get_lproduct_info = ->(lprod){
        {
          :id => lprod.id,
          :flg_enable => lprod.flg_enable,
          :name => lprod.name,
          :priority => lprod.priority,
          :is_support_end => lprod.is_support_end
        }
      }
      lproducts.each do |lp|
        temp_lproducts_info[lp.id] = {} unless temp_lproducts_info[lp.id]
        temp_lproducts_info[lp.id][:product] = get_lproduct_info.(lp)
        temp_lproducts_info[lp.id][:versions] = [] unless temp_lproducts_info[lp.id][:versions]
      end

      lversions = SidfmdLversion.fullname_matched(keyword).where(:flg_enable => true).order(:priority)
      lversions.each do |lv|
        temp_lproducts_info[lv.sidfmd_lproduct_id] = {} unless temp_lproducts_info[lv.sidfmd_lproduct_id]
        temp_lproducts_info[lv.sidfmd_lproduct_id][:versions] = [] unless temp_lproducts_info[lv.sidfmd_lproduct_id][:versions]
        temp_lproducts_info[lv.sidfmd_lproduct_id][:versions] << {
          :id => lv.id,
          :sidfmd_lproduct_id => lv.sidfmd_lproduct_id,
          :flg_enable => lv.flg_enable,
          :version => lv.version,
          :fullname => lv.fullname,
          :priority => lv.priority,
          :is_support_end => lv.is_support_end
        }
        temp_lproducts_info[lv.sidfmd_lproduct_id][:product] = get_lproduct_info.(lv.sidfmd_lproduct) unless temp_lproducts_info[lv.sidfmd_lproduct_id][:product]
      end

      hit_lprods = []
      temp_lproducts_info.each do |lprod_id, data|
        hit_lprods << {
          :product => data[:product],
          :versions => data[:versions]
        } 
      end
      ret['lproducts'] = hit_lprods
    end
    render :layout => false, :json => ret.to_json
  end

  # hostnameに関連づけられたアプリケーション情報を返す
  def api_get_apps_by_hostname
    f_params = params.fetch(:params, {}).permit(:hostname)
    hostname = f_params[:hostname]
    ret = {}
    host = SidfmdHost.where(:hostname => hostname).first
    if host then
      apps = host.sidfmd_host_apps
      if apps then
        apps.each do |app|
          ret[app.product_id] = [] unless ret.has_key?(app.product_id)
          pid = app.product_id
          vid = app.version_id
          if !(ret[app.product_id].find {|x| x[:vid] == vid }) then
            name = ''
            fullname = ''
            if pid > 0 then
              product = SidfmdProduct.where(:pid => pid).first
              if product then
                name = product.name
              end
            end
            if vid && vid > 0 then
              version = SidfmdVersion.where(:vid => vid).first
              if version then
                fullname = version.fullname
              end
            end
            ret[app.product_id] << {
              :pid => pid, :vid => vid,
              :name => name, :fullname => fullname, }
          end
        end
      end
      #ローカルプロダクト
      temp_lproducts_info = {}
      lapps = host.sidfmd_host_lapps
      lapps.each do |lapp|
        pid = lapp.sidfmd_lproduct_id
        vid = lapp.sidfmd_lversion_id
        temp_lproducts_info[pid] = [] unless temp_lproducts_info.has_key?(pid)
        name = ''
        fullname = ''
        if pid > 0 then
          product = SidfmdLproduct.find_by_id(pid)
          if product then
            name = product.name
          end
        end
        if vid > 0 then
          version = SidfmdLversion.find_by_id(vid)
          if version then
            fullname = version.fullname
          end
        end 
        temp_lproducts_info[pid] << {
          :pid => pid, :vid => vid, :name => name, :fullname => fullname
        }
      end
      ret['lproducts'] = temp_lproducts_info
    end
    render :layout => false, :json => ret.to_json
  end
 
  #新規ホスト登録時のvalidation
  def api_validate_new_host
    all_hostnames = SidfmdHost.where(:flg_delete => false).map{|host| host.hostname}
    render :layout => false, :json => all_hostnames.to_json
  end

  # ホストに登録する(紐づける)カテゴリのうち、hostlimの値が登録済みホスト数を超えているカテゴリを返す
  def api_hostlim_over_categories
    f_params = params.permit(:host_categoryids, :is_key_hostname)
    host_categoryids = JSON.parse(f_params[:host_categoryids])
    if f_params[:is_key_hostname] then
      temp = {}
      SidfmdHost.where(:hostname => host_categoryids.keys).each do |host|
        temp[host.id] = host_categoryids[host.hostname]
      end
      host_categoryids = temp
    end
    ret = SidfmdCategoriesHost.hostlim_over_categories(host_categoryids)
    render :layout => false, :json => ret
  end

  def api_get_import_hostlim_over_categories
    f_params = params.permit(:actions_data)
    actions_data = JSON.parse(f_params[:actions_data])
    hostlim_over_categories = get_import_hostlim_over_categories(actions_data)[:hostlim_over_categories]
    ret = hostlim_over_categories.map{|cat| cat.name }
    render :layout => false, :json => ret
  end 

  # オフラインデータの取り込み
  def load_offline_data
    f_params = params.permit(:file_data, :fileId, :initialPreview, :initialPreviewConfig, :initialPreviewThumbTags, :system_category_id, :custom_category_id)
    file_data = f_params[:file_data]
    #fileId = f_params[:fileId]
    #initialPreview = f_params[:initialPreview]
    #initialPreviewConfig = f_params[:initialPreviewConfig]
    #initialPreviewThumbTags = f_params[:initialPreviewThumbTags]
    system_category_id = (f_params[:system_category_id] || SidfmdCategory.label2id(:public_server)).to_i
    custom_category_id = (f_params[:custom_category_id] || '0').to_i
    ret = {}
    hostname = nil
    host = nil
    agent_type = :unix
    # 利用するパラメータのチェック
    begin
      if file_data then
        file_data = JSON.parse((file_data.read || '{}'), :symbolize_names => true)
        hostname = file_data[:hostname]
        unless hostname then
          logger.error('LackHostname')
          ret[:error] = 'ファイルが不正です'
          render :layout => false, :json => ret.to_json and return
        end
        host = SidfmdHost.enable_hosts.where(:hostname => hostname).first
        if host and !@role_admin and !@user_host_ids.include?(host.id) then
          logger.error('RejectByRole')
          ret[:error] = 'アクセス権限がありません'
          render :layout => false, :json => ret.to_json and return
        end
        if file_data.has_key?(:kb) then
          agent_type = :windows
        end
      else
        logger.error('InvalidParameter')
        ret[:error] = 'ファイルが不正です'
        render :layout => false, :json => ret.to_json and return
      end
    rescue JSON::ParserError => e
      log_exception(e)
      ret[:error] = 'ファイルが不正です'
      render :lauout => false, :json => ret.to_json and return
    rescue Exception => e
      log_exception(e)
      ret[:error] = 'ファイルが不正です'
      render :layout => false, :json => ret.to_json and return
    end
    unless host then
      if @sidfm_roles.include?(:admin) then
        system_category = SidfmdCategory.find(system_category_id)
        if system_category.nil? or !system_category.is_system then
          logger.error('InvalidSystemCategoryId')
          ret[:error] = 'システムカテゴリが不正です'
          render :layout => false, :json => ret.to_json and return
        end
        if custom_category_id > 0 then
          custom_category = SidfmdCategory.find(custom_category_id)
          if custom_category.nil? or custom_category.is_system then
            logger.error('InvalidCustomCategoryId')
            ret[:error] = 'カスタムカテゴリが不正です'
            render :layout => false, :json => ret.to_json and return
          end
        end
      elsif @sidfm_roles.include?(:use) then
        system_category = SidfmdCategory.find(system_category_id)
        if system_category.nil? or !system_category.is_system then
          logger.error('InvalidSystemCategoryId')
          ret[:error] = 'システムカテゴリが不正です'
          render :layout => false, :json => ret.to_json and return
        end
        if custom_category_id > 0 then
          custom_category = SidfmdCategory.find(custom_category_id)
          if custom_category.nil? or custom_category.is_system then
            logger.error('InvalidCustomCategoryId')
            ret[:error] = 'カスタムカテゴリが不正です'
            render :layout => false, :json => ret.to_json and return
          end
          unless @user_category_ids.include?(custom_category_id) then
            logger.error('RejectByRole')
            ret[:error] = 'アクセス権限がありません'
            render :layout => false, :json => ret.to_json and return
          end
        else
          logger.error('InvalidCustomCategoryId')
          ret[:error] = 'カスタムカテゴリが不正です'
          render :layout => false, :json => ret.to_json and return
        end
      else
        logger.error('RejectByRole')
        ret[:error] = 'アクセス権限がありません'
        render :layout => false, :json => ret.to_json and return
      end
    end
    unless host then
      # ホスト登録処理
      # 登録APIキーのチェックは無視。
      host_apikey = SidfmdHost.register_host(hostname, nil, true, false)
      if host_apikey then
        SidfmdHost.transaction do
          host = SidfmdHost.where(:hostname => hostname).first
          unless host then
            logger.error('FailureRegistHost')
            ret[:error] = 'ホストの登録に失敗しました'
            render :layout => false, :json => ret.to_json and return
          end
          host.lock!
          host.update_attribute :flg_delete, false if host.flg_delete
          SidfmdCategoriesHost.where(:sidfmd_host_id => host.id).delete_all
          SidfmdCategoriesHost.create(:sidfmd_category_id => system_category_id,
                                      :sidfmd_host_id => host.id)
          if custom_category_id > 0 then
            SidfmdCategoriesHost.create(:sidfmd_category_id => custom_category_id,
                                        :sidfmd_host_id => host.id)
          end
        end
        Rails.cache.clear
      else
        logger.error('FailureRegistHost')
        ret[:error] = 'ホストの登録に失敗しました'
        render :layout => false, :json => ret.to_json and return
      end
    end
    # データ登録処理
    if agent_type == :windows then
      begin
        SidfmdHost.transaction do
          host.lock!
          host.update_attribute :is_agent, true unless host.is_agent
          host.create_queue_win(file_data)
        end
        SidfmdHostView.sync_data([host.id])
      rescue Exception => e
        log_exception(e)
        ret[:error] = e.message
        render :layout => false, :json => ret.to_json and return
      end
    elsif agent_type == :unix then
      begin
        SidfmdHost.transaction do
          host.lock!
          host.update_attribute :is_agent, true unless host.is_agent
          host.create_queue(file_data)
        end
        SidfmdHostView.sync_data([host.id])
      rescue Exception => e
        log_exception(e)
        ret[:error] = e.message
        render :layout => false, :json => ret.to_json and return
      end
    else
      logger.error('LastResort')
      ret[:error] = 'システムの不具合'
      render :layout => false, :json => ret.to_json and return
    end
    render :layout => false, :json => ret.to_json
  end

  def set_last_checked_at 
    f_params = params.permit(:time_with_hostid)
    begin 
      SidfmdHost.transaction do 
        data = JSON.parse(f_params[:time_with_hostid])
        hostid = data['hostid'].to_i
        time = data['time'].to_i
        host = SidfmdHost.find(hostid)
        if time == -1 then
          set_time = nil
        else
          if time < 10 then
            time = '0' + time.to_s
          end
          last_time = host.last_checked_at
          if last_time then
            last_time = last_time.to_time
            set_time = (last_time.to_s.slice(0, 11) + time.to_s + ':00:00 +0900').to_time
            if set_time - last_time < 0 then
              set_time = set_time + 24*60*60
            end
            check_time = set_time - Time.current.to_time
            if check_time > 0 && (check_time / 60) / 60 >= 24 then
              set_time = set_time - 24*60*60
            end
          else
            set_time = (Date.today.yesterday.to_time.to_s.slice(0, 11) + time.to_s + ':00:00 +0900').to_time
            if ((Time.current.to_time - set_time) / 60) / 60 >= 24 then
              set_time = set_time + 24*60*60
            end
          end 
        end
        host.update_attribute :last_checked_at, set_time
      end
    rescue Exception => e
      log_exception(e)
      render :layout => false, :json => { :err_message => '同期時間の設定に失敗しました。' } and return
    end
    Rails.cache.clear
    render :layout => false, :json => { :err_message => '' }
  end

  #同期時間の確認
  def api_get_new_last_checked_at
    f_params = params.permit(:time_with_hostid)
      data = JSON.parse(f_params[:time_with_hostid])
      hostid = data['hostid'].to_i
      time = data['time'].to_i
      host = SidfmdHost.find(hostid)
      if time == -1 then
        set_time = 'now'
        synchro_time = 'now'
      else
        if time < 10 then
          time = '0' + time.to_s
        end
        last_time = host.last_checked_at
        if last_time then
          last_time = last_time.to_time
          set_time = (last_time.to_s.slice(0, 11) + time.to_s + ':00:00 +0900').to_time
          if set_time - last_time < 0 then
            set_time = set_time + 24*60*60
          end
          check_time = set_time - Time.current.to_time
          if check_time > 0 && (check_time / 60) / 60 >= 24 then
            set_time = set_time - 24*60*60
          end
        else
          set_time = (Date.today.yesterday.to_time.to_s.slice(0, 11) + time.to_s + ':00:00 +0900').to_time
          if ((Time.current.to_time - set_time) / 60) / 60 >= 24 then
            set_time = set_time + 24*60*60
          end
        end
        hour = set_time.hour.to_s
        synchro_time = (set_time + 24*60*60).strftime("%Y年%m月%d日 " + hour + "時")
        set_time = set_time.strftime("%Y年%m月%d日 " + hour + "時")
      end
    render :layout => false, :json => [set_time, synchro_time].to_json
  end

=begin
  def api_get_display_summary_rows
    #GC.start
    f_params = params.permit(:ids, :checked, :filters)
    hostnames = JSON.parse(f_params[:ids])
    filters = JSON.parse(f_params[:filters], :symbolize_names => true)
    checked = JSON.parse(f_params[:checked]).map(&:to_i)
    ret = {}
    err_message = ''
    content_data = get_content
    if content_data[:err_message] then
      render :layout => false, :json => { :err_message => content_data[:err_message] } and return 
    end
  
    unless content_data[:err_message] then
      content = content_data[:content]
      license_number = content[:license_number]
      used_license_number = content[:used_license_number]
      deleted_license_number = content[:deleted_license_number]

      content[:hosts].delete_if {|x|
        if !checked.include?(x[:hostname]) then
          return true
        end
        if !@role_admin then
          return x[:flg_delete]
        end
        return false
      }
   
      get_summary_prior_category 
      user = User.current
      content[:hosts].each do |host|
        if !checked.include?(x[:hostname]) || (!@role_admin && x[:flg_delete]) then
          next
        end
        ret[host[:hostname]] = create_summary_host_html(
          host,
          {
            :checked => checked,
            :user => user,
            :summary_prior_category => @summary_prior_category,
            :license_number => license_number,
            :used_license_number => used_license_number
          }
        )
      end
      ret[:new_host_registerable] =
        (@sidfm_roles & [ :host_admin ]).size > 0 ?
        license_number > used_license_number :
        false
      if @role_admin then
        ret[:license_number] = license_number
        ret[:used_license_number] = used_license_number
        ret[:deleted_license_number] = deleted_license_number
      end
    end

    render :layout => false, :json => ret
  end
=end

  # サマリー表のデータ
  def api_get_display_summary_content
    #GC.start
    f_params = params.fetch(:pages, {}).permit(:offset, :per_page, :filters, :order, :checked, :all_checked, :host_id)
    #@license_key = Sidfm.sidfmdlm_license_key
    #@license_apikey = Sidfm.sidfmdlm_license_apikey
    #@register_apikey = Sidfm.sidfmdlm_register_apikey
    ret = {}
    err_message = ''
    content_data = get_content
    if content_data[:err_message] then
      render :layout => false, :json => { :err_message => content_data[:err_message] } and return
    end

    unless content_data[:err_message] then
      @hosts = content_data[:content]
      license_number = @hosts[:license_number]
      used_license_number = @hosts[:used_license_number]
      deleted_license_number = @hosts[:deleted_license_number]
      if !@sidfm_roles.include?(:admin) then
        @hosts[:hosts].delete_if {|x| x[:flg_delete] }
      end
      get_summary_prior_category
      @hosts[:per_page] = f_params[:per_page].to_i
      @hosts[:offset] = (f_params[:offset] || 0).to_i
      #@hosts[:per_page] = (@summary_table_lines || f_params[:per_page] || 100).to_i
      @hosts[:order] = (f_params[:order] || 'hostname_asc')
      @hosts[:checked] = JSON.parse(f_params[:checked] || '[]')
      @hosts[:all_checked] = ((f_params[:all_checked] || '') == 'true')
      @hosts[:filters] = JSON.parse(f_params[:filters] || '{}').symbolize_keys
      check_filters
      @hosts[:hosts_filtered] = apply_filters
      @hosts[:content] = @hosts[:hosts_filtered].slice(@hosts[:offset], @hosts[:per_page])
      @hosts[:content] = [] unless @hosts[:content]

      user = User.current

      display_content = create_display_content(
        @hosts[:content].map{|host| host[:sidfmd_host] ? host[:sidfmd_host].id : nil}.compact
      ) 
      ret = {
        :rows => @hosts[:content].map{|host|
          create_summary_host_html(
            host,
            display_content[(host[:sidfmd_host] ? host[:sidfmd_host].id : -1)],
            {
              :checked => @hosts[:checked],
              :user => user,
              :summary_prior_category => @summary_prior_category,
              :license_number => license_number,
              :used_license_number => used_license_number
            }
          )
        },
        :all_ids => @hosts[:hosts_filtered].map{|host| if host[:flg_delete] then nil else host[:hostname] end }.compact,
        :total => @hosts[:hosts_filtered].size,
        :sidfm_roles => @sidfm_roles,
        :user_cats => (@role_admin ? nil : @user_category_ids)
      }
      ret[:new_host_registerable] = 
        (@sidfm_roles & [ :host_admin ]).size > 0 ?
        license_number > used_license_number :
        false 
      if @role_admin then
        ret[:license_number] = license_number
        ret[:used_license_number] = used_license_number
        ret[:deleted_license_number] = deleted_license_number
      end
    end
    render :layout => false, :json => ret
  end

  # ホストのパッケージを返す
  def api_get_host_pkgs
    f_params = params.permit(:sidfmd_host_id)
    host_id = f_params[:sidfmd_host_id].to_i
    if !@role_admin and !@user_host_ids.include?(host_id) then
      render :layout => false, :json => { :err_message => 'ログインユーザが操作できないホストです。' } and return
    end
    queue = SidfmdQueue.where(:sidfmd_host_id => host_id).order(:created_at => :desc).first
    pkgs = []
    if queue then
      pkgs = queue.sidfmd_queue_pkgs.map{|pkg| pkg.pkg}
    end
    render :layout => false, :json => { :pkgs => pkgs }
  end

  # ホストの検出アプリを返す
  def api_get_host_apps
    f_params = params.permit(:sidfmd_host_id)
    host_id = f_params[:sidfmd_host_id].to_i
    if !@role_admin and !@user_host_ids.include?(host_id) then
      render :layout => false, :json => { :err_message => 'ログインユーザが操作できないホストです。' } and return
    end
    queue = SidfmdQueue.where(:sidfmd_host_id => host_id).order(:created_at => :desc).first
    disabled_apps = SidfmdHostDisabledApp.where(:sidfmd_host_id => host_id).pluck(:name, :version, :path) 
    apps = []
    is_win = false
    if queue then
      win_apps = nil
      queue_win = queue.sidfmd_queue_win
      if queue_win then
        win_apps = queue_win.sidfmd_queue_win_apps
      end
      if win_apps && win_apps.exists? then
        is_win = true
        apps = win_apps.map{|win_app| #今はLinuxのみでdisabled登録なし
          {
            :name => win_app.display_name,
            :id => win_app.id,
            :is_disabled => false,
          }
        }
      else
        apps = queue.sidfmd_queue_apps.map{|app|
          {
            :name => "#{app.name} #{app.version}",
            :path => "#{app.path}",
            :id => app.id,
            :is_disabled => (disabled_apps.find{|d_app|
              d_app[0] == app.name && d_app[1] == app.version &&  d_app[2] == app.path
            } != nil),
          }
        }
      end
    end
    render :layout => false, 
      :json => {
        :apps => apps,
        :is_win => is_win,
        :is_toggle_auth => (@sidfm_roles.include?(:host_admin) or @sidfm_roles.include?(:host_limited_admin))
      }
  end

  #  ホストの検出アプリの有効/無効を設定する
  def api_toggle_host_app_state
    if !@sidfm_roles.include?(:host_admin) && !@sidfm_roles.include?(:host_limited_admin) then
      render :layout => false, :json => { :is_enabled => nil } and return #nilが返るならエラー
    end
    f_params = params.permit(:is_win, :sidfmd_host_id, :id)
    id = f_params[:id] 
    host_id = f_params[:sidfmd_host_id].to_i
    unless @role_admin then
      if !@user_host_ids.include?(host_id) then
        render :layout => false, :json => { :is_enabled => nil } and return #nilが返るならエラー
      end
    end
    is_win = f_params[:is_win] == 'true'
    is_disabled_app_deleted = nil #nilが返るならエラー
    begin
      host = SidfmdHost.find_by_id(host_id)
      if host then
        toggle_ret = host.toggle_host_app_state(is_win: is_win, queue_app_id: id)
        is_disabled_app_deleted = toggle_ret[:value] if toggle_ret[:success] 
      end
    rescue Exception => e
      log_exception(e)
    end
    render :layout => false, :json => { :is_enabled => is_disabled_app_deleted }
  end

  def api_get_host_lapps
    f_params = params.permit(:sidfmd_host_id)
    host_id = f_params[:sidfmd_host_id].to_i
    if !@role_admin and !@user_host_ids.include?(host_id) then
      render :layout => false, :json => { :err_message => 'ログインユーザが操作できないホストです。' } and return
    end
    lapps = []
    host = SidfmdHost.find_by(:id => host_id)
    if host then
      lapps = host.sidfmd_host_apps.map{|lapp| lapp.get_name }.compact.sort_by{|lapp| lapp.downcase }
      lproducts = host.sidfmd_host_lapps.map{|lprod| lprod.get_name }.sort_by{|lprod| lprod.downcase }
      lapps.concat(lproducts)
    end
    render :layout => false, :json => { :lapps => lapps }
  end

end
